@@ -20,8 +20,8 @@ package org.apache.spark.util.kvstore;
 import java.util.NoSuchElementException;
 
 import com.google.common.collect.ImmutableSet;
-import org.junit.Test;
-import static org.junit.Assert.*;
+import org.junit.jupiter.api.Test;
+import static org.junit.jupiter.api.Assertions.*;
 
 public class InMemoryStoreSuite {
 
