@@ -126,20 +126,20 @@ class AuthEngine implements Closeable {
   private byte[] decryptEphemeralPublicKey(
       AuthMessage encryptedPublicKey,
       byte[] transcript) throws GeneralSecurityException {
-    Preconditions.checkArgument(appId.equals(encryptedPublicKey.appId));
+    Preconditions.checkArgument(appId.equals(encryptedPublicKey.appId()));
     // Mix in the app ID, salt, and transcript into HKDF and use it as AES-GCM AAD
-    byte[] aadState = Bytes.concat(appId.getBytes(UTF_8), encryptedPublicKey.salt, transcript);
+    byte[] aadState = Bytes.concat(appId.getBytes(UTF_8), encryptedPublicKey.salt(), transcript);
     // Use HKDF to derive an AES_GCM key from the pre-shared key, non-secret salt, and AAD state
     byte[] derivedKeyEncryptingKey = Hkdf.computeHkdf(
         MAC_ALGORITHM,
         preSharedSecret,
-        encryptedPublicKey.salt,
+        encryptedPublicKey.salt(),
         aadState,
         AES_GCM_KEY_SIZE_BYTES);
     // If the AES-GCM payload is modified at all or if the AAD state does not match, decryption
     // will throw a GeneralSecurityException.
     return new AesGcmJce(derivedKeyEncryptingKey)
-        .decrypt(encryptedPublicKey.ciphertext, aadState);
+        .decrypt(encryptedPublicKey.ciphertext(), aadState);
   }
 
   /**
@@ -162,7 +162,7 @@ class AuthEngine implements Closeable {
    * @return An encrypted server ephemeral public key to be sent to the client.
    */
   AuthMessage response(AuthMessage encryptedClientPublicKey) throws GeneralSecurityException {
-    Preconditions.checkArgument(appId.equals(encryptedClientPublicKey.appId));
+    Preconditions.checkArgument(appId.equals(encryptedClientPublicKey.appId()));
     // Compute a shared secret given the client public key and the server private key
     byte[] clientPublicKey =
         decryptEphemeralPublicKey(encryptedClientPublicKey, EMPTY_TRANSCRIPT);
@@ -190,8 +190,8 @@ class AuthEngine implements Closeable {
    */
   void deriveSessionCipher(AuthMessage encryptedClientPublicKey,
                            AuthMessage encryptedServerPublicKey) throws GeneralSecurityException {
-    Preconditions.checkArgument(appId.equals(encryptedClientPublicKey.appId));
-    Preconditions.checkArgument(appId.equals(encryptedServerPublicKey.appId));
+    Preconditions.checkArgument(appId.equals(encryptedClientPublicKey.appId()));
+    Preconditions.checkArgument(appId.equals(encryptedServerPublicKey.appId()));
     // Compute a shared secret given the server public key and the client private key,
     // mixing in the protocol transcript.
     byte[] serverPublicKey = decryptEphemeralPublicKey(
