@@ -25,9 +25,11 @@ import com.google.common.base.Throwables;
 import io.netty.buffer.ByteBuf;
 import io.netty.buffer.Unpooled;
 import io.netty.channel.Channel;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
 
+import org.apache.spark.internal.SparkLogger;
+import org.apache.spark.internal.SparkLoggerFactory;
+import org.apache.spark.internal.LogKeys;
+import org.apache.spark.internal.MDC;
 import org.apache.spark.network.client.RpcResponseCallback;
 import org.apache.spark.network.client.TransportClient;
 import org.apache.spark.network.sasl.SecretKeyHolder;
@@ -46,7 +48,7 @@ import org.apache.spark.network.util.TransportConf;
  * authenticated. A connection may be authenticated at most once.
  */
 class AuthRpcHandler extends AbstractAuthRpcHandler {
-  private static final Logger LOG = LoggerFactory.getLogger(AuthRpcHandler.class);
+  private static final SparkLogger LOG = SparkLoggerFactory.getLogger(AuthRpcHandler.class);
 
   /** Transport configuration. */
   private final TransportConf conf;
@@ -91,7 +93,7 @@ class AuthRpcHandler extends AbstractAuthRpcHandler {
     } catch (RuntimeException e) {
       if (conf.saslFallback()) {
         LOG.warn("Failed to parse new auth challenge, reverting to SASL for client {}.",
-          channel.remoteAddress());
+          MDC.of(LogKeys.HOST_PORT$.MODULE$, channel.remoteAddress()));
         saslHandler = new SaslRpcHandler(conf, channel, null, secretKeyHolder);
         message.position(position);
         message.limit(limit);
@@ -108,17 +110,17 @@ class AuthRpcHandler extends AbstractAuthRpcHandler {
     // Here we have the client challenge, so perform the new auth protocol and set up the channel.
     AuthEngine engine = null;
     try {
-      String secret = secretKeyHolder.getSecretKey(challenge.appId);
+      String secret = secretKeyHolder.getSecretKey(challenge.appId());
       Preconditions.checkState(secret != null,
-        "Trying to authenticate non-registered app %s.", challenge.appId);
-      LOG.debug("Authenticating challenge for app {}.", challenge.appId);
-      engine = new AuthEngine(challenge.appId, secret, conf);
+        "Trying to authenticate non-registered app %s.", challenge.appId());
+      LOG.debug("Authenticating challenge for app {}.", challenge.appId());
+      engine = new AuthEngine(challenge.appId(), secret, conf);
       AuthMessage response = engine.response(challenge);
       ByteBuf responseData = Unpooled.buffer(response.encodedLength());
       response.encode(responseData);
       callback.onSuccess(responseData.nioBuffer());
       engine.sessionCipher().addToChannel(channel);
-      client.setClientId(challenge.appId);
+      client.setClientId(challenge.appId());
     } catch (Exception e) {
       // This is a fatal error: authentication has failed. Close the channel explicitly.
       LOG.debug("Authentication failed for client {}, closing channel.", channel.remoteAddress());
@@ -130,7 +132,8 @@ class AuthRpcHandler extends AbstractAuthRpcHandler {
         try {
           engine.close();
         } catch (Exception e) {
-          throw Throwables.propagate(e);
+          Throwables.throwIfUnchecked(e);
+          throw new RuntimeException(e);
         }
       }
     }
