@@ -30,7 +30,6 @@ import org.apache.spark.network.util.ByteBufferWriteableChannel;
 
 import javax.crypto.spec.SecretKeySpec;
 import java.io.IOException;
-import java.nio.Buffer;
 import java.nio.ByteBuffer;
 import java.nio.channels.WritableByteChannel;
 import java.security.GeneralSecurityException;
@@ -128,7 +127,8 @@ public class GcmTransportCipher implements TransportCipher {
             // If the ciphertext buffer cannot be fully written the target, transferTo may
             // return with it containing some unwritten data. The initial call we'll explicitly
             // set its limit to 0 to indicate the first call to transferTo.
-            ((Buffer) this.ciphertextBuffer).limit(0);
+            this.ciphertextBuffer.limit(0);
+
             this.bytesToRead = getReadableBytes();
             this.encryptedCount =
                     LENGTH_HEADER_BYTES + aesGcmHkdfStreaming.expectedCiphertextSize(bytesToRead);
@@ -141,12 +141,11 @@ public class GcmTransportCipher implements TransportCipher {
         // [8 byte length][Internal IV and header][Ciphertext][Auth Tag]
         private ByteBuffer createHeaderByteBuffer() {
             ByteBuffer encrypterHeader = encrypter.getHeader();
-            ByteBuffer output = ByteBuffer
+            return ByteBuffer
                     .allocate(encrypterHeader.remaining() + LENGTH_HEADER_BYTES)
                     .putLong(encryptedCount)
-                    .put(encrypterHeader);
-            ((Buffer) output).flip();
-            return output;
+                    .put(encrypterHeader)
+                    .flip();
         }
 
         @Override
@@ -167,11 +166,9 @@ public class GcmTransportCipher implements TransportCipher {
         @Override
         public GcmEncryptedMessage touch(Object o) {
             super.touch(o);
-            if (plaintextMessage instanceof ByteBuf) {
-                ByteBuf byteBuf = (ByteBuf) plaintextMessage;
+            if (plaintextMessage instanceof ByteBuf byteBuf) {
                 byteBuf.touch(o);
-            } else if (plaintextMessage instanceof FileRegion) {
-                FileRegion fileRegion = (FileRegion) plaintextMessage;
+            } else if (plaintextMessage instanceof FileRegion fileRegion) {
                 fileRegion.touch(o);
             }
             return this;
@@ -180,11 +177,9 @@ public class GcmTransportCipher implements TransportCipher {
         @Override
         public GcmEncryptedMessage retain(int increment) {
             super.retain(increment);
-            if (plaintextMessage instanceof ByteBuf) {
-                ByteBuf byteBuf = (ByteBuf) plaintextMessage;
+            if (plaintextMessage instanceof ByteBuf byteBuf) {
                 byteBuf.retain(increment);
-            } else if (plaintextMessage instanceof FileRegion) {
-                FileRegion fileRegion = (FileRegion) plaintextMessage;
+            } else if (plaintextMessage instanceof FileRegion fileRegion) {
                 fileRegion.retain(increment);
             }
             return this;
@@ -192,11 +187,9 @@ public class GcmTransportCipher implements TransportCipher {
 
         @Override
         public boolean release(int decrement) {
-            if (plaintextMessage instanceof ByteBuf) {
-                ByteBuf byteBuf = (ByteBuf) plaintextMessage;
+            if (plaintextMessage instanceof ByteBuf byteBuf) {
                 byteBuf.release(decrement);
-            } else if (plaintextMessage instanceof FileRegion) {
-                FileRegion fileRegion = (FileRegion) plaintextMessage;
+            } else if (plaintextMessage instanceof FileRegion fileRegion) {
                 fileRegion.release(decrement);
             }
             return super.release(decrement);
@@ -227,14 +220,12 @@ public class GcmTransportCipher implements TransportCipher {
                 long readableBytes = getReadableBytes();
                 int readLimit =
                         (int) Math.min(readableBytes, plaintextBuffer.remaining());
-                if (plaintextMessage instanceof ByteBuf) {
-                    ByteBuf byteBuf = (ByteBuf) plaintextMessage;
+                if (plaintextMessage instanceof ByteBuf byteBuf) {
                     Preconditions.checkState(0 == plaintextBuffer.position());
-                    ((Buffer) plaintextBuffer).limit(readLimit);
+                    plaintextBuffer.limit(readLimit);
                     byteBuf.readBytes(plaintextBuffer);
                     Preconditions.checkState(readLimit == plaintextBuffer.position());
-                } else if (plaintextMessage instanceof FileRegion) {
-                    FileRegion fileRegion = (FileRegion) plaintextMessage;
+                } else if (plaintextMessage instanceof FileRegion fileRegion) {
                     ByteBufferWriteableChannel plaintextChannel =
                             new ByteBufferWriteableChannel(plaintextBuffer);
                     long plaintextRead =
@@ -246,16 +237,16 @@ public class GcmTransportCipher implements TransportCipher {
                     }
                 }
                 boolean lastSegment = getReadableBytes() == 0;
-                ((Buffer) plaintextBuffer).flip();
+                plaintextBuffer.flip();
                 bytesRead += plaintextBuffer.remaining();
-                ((Buffer) ciphertextBuffer).clear();
+                ciphertextBuffer.clear();
                 try {
                     encrypter.encryptSegment(plaintextBuffer, lastSegment, ciphertextBuffer);
                 } catch (GeneralSecurityException e) {
                     throw new IllegalStateException("GeneralSecurityException from encrypter", e);
                 }
-                ((Buffer) plaintextBuffer).clear();
-                ((Buffer) ciphertextBuffer).flip();
+                plaintextBuffer.clear();
+                ciphertextBuffer.flip();
                 int written = target.write(ciphertextBuffer);
                 transferredThisCall += written;
                 this.transferred += written;
@@ -269,11 +260,9 @@ public class GcmTransportCipher implements TransportCipher {
         }
 
         private long getReadableBytes() {
-            if (plaintextMessage instanceof ByteBuf) {
-                ByteBuf byteBuf = (ByteBuf) plaintextMessage;
+            if (plaintextMessage instanceof ByteBuf byteBuf) {
                 return byteBuf.readableBytes();
-            } else if (plaintextMessage instanceof FileRegion) {
-                FileRegion fileRegion = (FileRegion) plaintextMessage;
+            } else if (plaintextMessage instanceof FileRegion fileRegion) {
                 return fileRegion.count() - fileRegion.transferred();
             } else {
                 throw new IllegalArgumentException("Unsupported message type: " +
@@ -283,8 +272,8 @@ public class GcmTransportCipher implements TransportCipher {
 
         @Override
         protected void deallocate() {
-            if (plaintextMessage instanceof ReferenceCounted) {
-                ((ReferenceCounted) plaintextMessage).release();
+            if (plaintextMessage instanceof ReferenceCounted referenceCounted) {
+                referenceCounted.release();
             }
             plaintextBuffer.clear();
             ciphertextBuffer.clear();
@@ -322,7 +311,7 @@ public class GcmTransportCipher implements TransportCipher {
                     // We did not read enough bytes to initialize the expected length.
                     return false;
                 }
-                ((Buffer) expectedLengthBuffer).flip();
+                expectedLengthBuffer.flip();
                 expectedLength = expectedLengthBuffer.getLong();
                 if (expectedLength < 0) {
                     throw new IllegalStateException("Invalid expected ciphertext length.");
@@ -342,7 +331,7 @@ public class GcmTransportCipher implements TransportCipher {
                     // We did not read enough bytes to initialize the header.
                     return false;
                 }
-                ((Buffer) headerBuffer).flip();
+                headerBuffer.flip();
                 byte[] lengthAad = Longs.toByteArray(expectedLength);
                 decrypter.init(headerBuffer, lengthAad);
                 decrypterInit = true;
@@ -383,8 +372,7 @@ public class GcmTransportCipher implements TransportCipher {
                     int expectedRemaining = (int) (expectedLength - ciphertextRead);
                     int bytesToRead = Integer.min(readableBytes, expectedRemaining);
                     // The smallest ciphertext size is 16 bytes for the auth tag
-                    ((Buffer) ciphertextBuffer).limit(
-                            ((Buffer) ciphertextBuffer).position() + bytesToRead);
+                    ciphertextBuffer.limit(ciphertextBuffer.position() + bytesToRead);
                     ciphertextNettyBuf.readBytes(ciphertextBuffer);
                     ciphertextRead += bytesToRead;
                     // Check if this is the last segment
@@ -397,7 +385,7 @@ public class GcmTransportCipher implements TransportCipher {
                     // then decrypt it and fire a read.
                     if (ciphertextBuffer.limit() == ciphertextBuffer.capacity() || completed) {
                         ByteBuffer plaintextBuffer = ByteBuffer.allocate(plaintextSegmentSize);
-                        ((Buffer) ciphertextBuffer).flip();
+                        ciphertextBuffer.flip();
                         decrypter.decryptSegment(
                                 ciphertextBuffer,
                                 segmentNumber,
@@ -405,12 +393,12 @@ public class GcmTransportCipher implements TransportCipher {
                                 plaintextBuffer);
                         segmentNumber++;
                         // Clear the ciphertext buffer because it's been read
-                        ((Buffer) ciphertextBuffer).clear();
-                        ((Buffer) plaintextBuffer).flip();
+                        ciphertextBuffer.clear();
+                        plaintextBuffer.flip();
                         ctx.fireChannelRead(Unpooled.wrappedBuffer(plaintextBuffer));
                     } else {
                         // Set the ciphertext buffer up to read the next chunk
-                        ((Buffer) ciphertextBuffer).limit(ciphertextBuffer.capacity());
+                        ciphertextBuffer.limit(ciphertextBuffer.capacity());
                     }
                     nettyBufReadableBytes = ciphertextNettyBuf.readableBytes();
                 }
