@@ -25,8 +25,8 @@ import com.google.common.collect.ImmutableMap;
 import com.google.crypto.tink.subtle.Hex;
 import org.apache.spark.network.util.*;
 
-import static org.junit.Assert.*;
-import org.junit.Test;
+import static org.junit.jupiter.api.Assertions.*;
+import org.junit.jupiter.api.Test;
 
 abstract class AuthEngineSuite {
   static final String clientPrivate =
@@ -92,7 +92,7 @@ abstract class AuthEngineSuite {
          AuthEngine server = new AuthEngine("appId", "secret", conf)) {
       AuthMessage clientChallenge = client.challenge();
       AuthMessage corruptChallenge =
-              new AuthMessage("junk", clientChallenge.salt, clientChallenge.ciphertext);
+              new AuthMessage("junk", clientChallenge.salt(), clientChallenge.ciphertext());
       assertThrows(IllegalArgumentException.class, () -> server.response(corruptChallenge));
     }
   }
@@ -102,7 +102,7 @@ abstract class AuthEngineSuite {
     try (AuthEngine client = new AuthEngine("appId", "secret", conf);
          AuthEngine server = new AuthEngine("appId", "secret", conf)) {
       AuthMessage clientChallenge = client.challenge();
-      clientChallenge.salt[0] ^= 1;
+      clientChallenge.salt()[0] ^= 1;
       assertThrows(GeneralSecurityException.class, () -> server.response(clientChallenge));
     }
   }
@@ -112,7 +112,7 @@ abstract class AuthEngineSuite {
     try (AuthEngine client = new AuthEngine("appId", "secret", conf);
          AuthEngine server = new AuthEngine("appId", "secret", conf)) {
       AuthMessage clientChallenge = client.challenge();
-      clientChallenge.ciphertext[0] ^= 1;
+      clientChallenge.ciphertext()[0] ^= 1;
       assertThrows(GeneralSecurityException.class, () -> server.response(clientChallenge));
     }
   }
@@ -124,7 +124,7 @@ abstract class AuthEngineSuite {
       AuthMessage clientChallenge = client.challenge();
       AuthMessage serverResponse = server.response(clientChallenge);
       AuthMessage corruptResponse =
-              new AuthMessage("junk", serverResponse.salt, serverResponse.ciphertext);
+              new AuthMessage("junk", serverResponse.salt(), serverResponse.ciphertext());
       assertThrows(IllegalArgumentException.class,
         () -> client.deriveSessionCipher(clientChallenge, corruptResponse));
     }
@@ -136,7 +136,7 @@ abstract class AuthEngineSuite {
          AuthEngine server = new AuthEngine("appId", "secret", conf)) {
       AuthMessage clientChallenge = client.challenge();
       AuthMessage serverResponse = server.response(clientChallenge);
-      serverResponse.salt[0] ^= 1;
+      serverResponse.salt()[0] ^= 1;
       assertThrows(GeneralSecurityException.class,
               () -> client.deriveSessionCipher(clientChallenge, serverResponse));
     }
@@ -148,7 +148,7 @@ abstract class AuthEngineSuite {
          AuthEngine server = new AuthEngine("appId", "secret", conf)) {
       AuthMessage clientChallenge = client.challenge();
       AuthMessage serverResponse = server.response(clientChallenge);
-      serverResponse.ciphertext[0] ^= 1;
+      serverResponse.ciphertext()[0] ^= 1;
       assertThrows(GeneralSecurityException.class,
         () -> client.deriveSessionCipher(clientChallenge, serverResponse));
     }
