@@ -57,8 +57,7 @@ public class StreamHandle extends BlockTransferMessage {
 
   @Override
   public boolean equals(Object other) {
-    if (other instanceof StreamHandle) {
-      StreamHandle o = (StreamHandle) other;
+    if (other instanceof StreamHandle o) {
       return Objects.equals(streamId, o.streamId)
         && Objects.equals(numChunks, o.numChunks);
     }
