@@ -30,13 +30,12 @@ import java.util.concurrent.TimeoutException;
 
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.Sets;
-
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 import org.mockito.stubbing.Answer;
 import org.mockito.stubbing.Stubber;
 
-import static org.junit.Assert.*;
+import static org.junit.jupiter.api.Assertions.*;
 import static org.mockito.Mockito.*;
 
 import org.apache.spark.network.buffer.ManagedBuffer;
@@ -60,7 +59,7 @@ public class RetryingBlockTransferorSuite {
 
   private static final int MAX_RETRIES = 2;
 
-  @Before
+  @BeforeEach
   public void initMap() {
     configMap = new HashMap<String, String>() {{
       put("spark.shuffle.io.maxRetries", Integer.toString(MAX_RETRIES));
@@ -289,7 +288,7 @@ public class RetryingBlockTransferorSuite {
     verify(listener, timeout(5000)).onBlockTransferSuccess("b0", block0);
     verify(listener).getTransferType();
     verifyNoMoreInteractions(listener);
-    assert(_retryingBlockTransferor.getRetryCount() == 0);
+    assertEquals(0, _retryingBlockTransferor.getRetryCount());
   }
 
   @Test
@@ -311,7 +310,7 @@ public class RetryingBlockTransferorSuite {
     verify(listener, timeout(5000)).onBlockTransferFailure("b0", saslTimeoutException);
     verify(listener, times(3)).getTransferType();
     verifyNoMoreInteractions(listener);
-    assert(_retryingBlockTransferor.getRetryCount() == MAX_RETRIES);
+    assertEquals(MAX_RETRIES, _retryingBlockTransferor.getRetryCount());
   }
 
   @Test
@@ -340,7 +339,7 @@ public class RetryingBlockTransferorSuite {
     // This should be equal to 1 because after the SASL exception is retried,
     // retryCount should be set back to 0. Then after that b1 encounters an
     // exception that is retried.
-    assert(_retryingBlockTransferor.getRetryCount() == 1);
+    assertEquals(1, _retryingBlockTransferor.getRetryCount());
   }
 
   @Test
@@ -369,7 +368,33 @@ public class RetryingBlockTransferorSuite {
     verify(listener, timeout(5000)).onBlockTransferFailure("b0", saslExceptionFinal);
     verify(listener, atLeastOnce()).getTransferType();
     verifyNoMoreInteractions(listener);
-    assert(_retryingBlockTransferor.getRetryCount() == MAX_RETRIES);
+    assertEquals(MAX_RETRIES, _retryingBlockTransferor.getRetryCount());
+  }
+
+  @Test
+  public void testRetryInitiationFailure() throws IOException, InterruptedException {
+    BlockFetchingListener listener = mock(BlockFetchingListener.class);
+
+    List<? extends Map<String, Object>> interactions = Arrays.asList(
+        // IOException will initiate a retry, but the initiation will fail
+        ImmutableMap.<String, Object>builder()
+            .put("b0", new IOException("Connection failed or something"))
+            .put("b1", block1)
+            .build()
+    );
+
+    configureInteractions(interactions, listener);
+    _retryingBlockTransferor = spy(_retryingBlockTransferor);
+    // Simulate a failure to initiate a retry.
+    doReturn(false).when(_retryingBlockTransferor).initiateRetry(any());
+    // Override listener, so that it delegates to the spied instance and not the original class.
+    _retryingBlockTransferor.setCurrentListener(
+        _retryingBlockTransferor.new RetryingBlockTransferListener());
+    _retryingBlockTransferor.start();
+
+    verify(listener, timeout(5000)).onBlockTransferFailure(eq("b0"), any());
+    verify(listener, timeout(5000)).onBlockTransferSuccess("b1", block1);
+    verifyNoMoreInteractions(listener);
   }
 
   /**
@@ -384,6 +409,13 @@ public class RetryingBlockTransferorSuite {
    */
   private static void performInteractions(List<? extends Map<String, Object>> interactions,
                                           BlockFetchingListener listener)
+      throws IOException, InterruptedException {
+    configureInteractions(interactions, listener);
+    _retryingBlockTransferor.start();
+  }
+
+  private static void configureInteractions(List<? extends Map<String, Object>> interactions,
+                                          BlockFetchingListener listener)
     throws IOException, InterruptedException {
 
     MapConfigProvider provider = new MapConfigProvider(configMap);
@@ -412,10 +444,10 @@ public class RetryingBlockTransferorSuite {
             String blockId = block.getKey();
             Object blockValue = block.getValue();
 
-            if (blockValue instanceof ManagedBuffer) {
-              retryListener.onBlockFetchSuccess(blockId, (ManagedBuffer) blockValue);
-            } else if (blockValue instanceof Exception) {
-              retryListener.onBlockFetchFailure(blockId, (Exception) blockValue);
+            if (blockValue instanceof ManagedBuffer managedBuffer) {
+              retryListener.onBlockFetchSuccess(blockId, managedBuffer);
+            } else if (blockValue instanceof Exception exception) {
+              retryListener.onBlockFetchFailure(blockId, exception);
             } else {
               fail("Can only handle ManagedBuffers and Exceptions, got " + blockValue);
             }
@@ -440,6 +472,5 @@ public class RetryingBlockTransferorSuite {
     String[] blockIdArray = blockIds.toArray(new String[blockIds.size()]);
     _retryingBlockTransferor =
         new RetryingBlockTransferor(conf, fetchStarter, blockIdArray, listener);
-    _retryingBlockTransferor.start();
   }
 }
