@@ -25,11 +25,13 @@ import java.util.LinkedList;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.common.io.Closeables;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
 
 import org.apache.spark.SparkEnv;
 import org.apache.spark.executor.ShuffleWriteMetrics;
+import org.apache.spark.internal.LogKeys;
+import org.apache.spark.internal.SparkLogger;
+import org.apache.spark.internal.SparkLoggerFactory;
+import org.apache.spark.internal.MDC;
 import org.apache.spark.memory.MemoryConsumer;
 import org.apache.spark.memory.SparkOutOfMemoryError;
 import org.apache.spark.memory.TaskMemoryManager;
@@ -66,7 +68,7 @@ import org.apache.spark.util.collection.unsafe.sort.UnsafeSorterSpillWriter;
  */
 public final class BytesToBytesMap extends MemoryConsumer {
 
-  private static final Logger logger = LoggerFactory.getLogger(BytesToBytesMap.class);
+  private static final SparkLogger logger = SparkLoggerFactory.getLogger(BytesToBytesMap.class);
 
   private static final HashMapGrowthStrategy growthStrategy = HashMapGrowthStrategy.DOUBLING;
 
@@ -392,7 +394,8 @@ public final class BytesToBytesMap extends MemoryConsumer {
         // remove the spill file from disk
         File file = spillWriters.removeFirst().getFile();
         if (file != null && file.exists() && !file.delete()) {
-          logger.error("Was unable to delete spill file {}", file.getAbsolutePath());
+          logger.error("Was unable to delete spill file {}",
+            MDC.of(LogKeys.PATH$.MODULE$, file.getAbsolutePath()));
         }
       }
     }
@@ -893,7 +896,8 @@ public final class BytesToBytesMap extends MemoryConsumer {
       File file = spillWriters.removeFirst().getFile();
       if (file != null && file.exists()) {
         if (!file.delete()) {
-          logger.error("Was unable to delete spill file {}", file.getAbsolutePath());
+          logger.error("Was unable to delete spill file {}",
+            MDC.of(LogKeys.PATH$.MODULE$, file.getAbsolutePath()));
         }
       }
     }
