@@ -18,24 +18,73 @@
 package org.apache.spark.api.python
 
 import java.io.{DataInputStream, DataOutputStream, EOFException, File, InputStream}
-import java.net.{InetAddress, ServerSocket, Socket, SocketException}
+import java.net.{InetAddress, InetSocketAddress, SocketException}
+import java.net.SocketTimeoutException
+import java.nio.channels._
 import java.util.Arrays
 import java.util.concurrent.TimeUnit
 import javax.annotation.concurrent.GuardedBy
 
-import scala.collection.JavaConverters._
 import scala.collection.mutable
+import scala.jdk.CollectionConverters._
+import scala.jdk.OptionConverters._
 
 import org.apache.spark._
 import org.apache.spark.errors.SparkCoreErrors
-import org.apache.spark.internal.Logging
-import org.apache.spark.internal.config.Python._
+import org.apache.spark.internal.{Logging, MDC}
+import org.apache.spark.internal.LogKeys._
 import org.apache.spark.security.SocketAuthHelper
 import org.apache.spark.util.{RedirectThread, Utils}
 
-private[spark] class PythonWorkerFactory(pythonExec: String, envVars: Map[String, String])
+case class PythonWorker(channel: SocketChannel) {
+
+  private[this] var selectorOpt: Option[Selector] = None
+  private[this] var selectionKeyOpt: Option[SelectionKey] = None
+
+  def selector: Selector = selectorOpt.orNull
+  def selectionKey: SelectionKey = selectionKeyOpt.orNull
+
+  private def closeSelector(): Unit = {
+    selectionKeyOpt.foreach(_.cancel())
+    selectorOpt.foreach(_.close())
+  }
+
+  def refresh(): this.type = synchronized {
+    closeSelector()
+    if (channel.isBlocking) {
+      selectorOpt = None
+      selectionKeyOpt = None
+    } else {
+      val selector = Selector.open()
+      selectorOpt = Some(selector)
+      selectionKeyOpt =
+        Some(channel.register(selector, SelectionKey.OP_READ | SelectionKey.OP_WRITE))
+    }
+    this
+  }
+
+  def stop(): Unit = synchronized {
+    closeSelector()
+    Option(channel).foreach(_.close())
+  }
+}
+
+private[spark] class PythonWorkerFactory(
+    pythonExec: String,
+    workerModule: String,
+    daemonModule: String,
+    envVars: Map[String, String],
+    val useDaemonEnabled: Boolean)
   extends Logging { self =>
 
+  def this(
+      pythonExec: String,
+      workerModule: String,
+      envVars: Map[String, String],
+      useDaemonEnabled: Boolean) =
+    this(pythonExec, workerModule, PythonWorkerFactory.defaultDaemonModule,
+      envVars, useDaemonEnabled)
+
   import PythonWorkerFactory._
 
   // Because forking processes from Java is expensive, we prefer to launch a single Python daemon,
@@ -43,36 +92,10 @@ private[spark] class PythonWorkerFactory(pythonExec: String, envVars: Map[String
   // currently only works on UNIX-based systems now because it uses signals for child management,
   // so we can also fall back to launching workers, pyspark/worker.py (by default) directly.
   private val useDaemon = {
-    val useDaemonEnabled = SparkEnv.get.conf.get(PYTHON_USE_DAEMON)
-
     // This flag is ignored on Windows as it's unable to fork.
     !System.getProperty("os.name").startsWith("Windows") && useDaemonEnabled
   }
 
-  // WARN: Both configurations, 'spark.python.daemon.module' and 'spark.python.worker.module' are
-  // for very advanced users and they are experimental. This should be considered
-  // as expert-only option, and shouldn't be used before knowing what it means exactly.
-
-  // This configuration indicates the module to run the daemon to execute its Python workers.
-  private val daemonModule =
-    SparkEnv.get.conf.get(PYTHON_DAEMON_MODULE).map { value =>
-      logInfo(
-        s"Python daemon module in PySpark is set to [$value] in '${PYTHON_DAEMON_MODULE.key}', " +
-        "using this to start the daemon up. Note that this configuration only has an effect when " +
-        s"'${PYTHON_USE_DAEMON.key}' is enabled and the platform is not Windows.")
-      value
-    }.getOrElse("pyspark.daemon")
-
-  // This configuration indicates the module to run each Python worker.
-  private val workerModule =
-    SparkEnv.get.conf.get(PYTHON_WORKER_MODULE).map { value =>
-      logInfo(
-        s"Python worker module in PySpark is set to [$value] in '${PYTHON_WORKER_MODULE.key}', " +
-        "using this to start the worker up. Note that this configuration only has an effect when " +
-        s"'${PYTHON_USE_DAEMON.key}' is disabled or the platform is Windows.")
-      value
-    }.getOrElse("pyspark.worker")
-
   private val authHelper = new SocketAuthHelper(SparkEnv.get.conf)
 
   @GuardedBy("self")
@@ -81,57 +104,68 @@ private[spark] class PythonWorkerFactory(pythonExec: String, envVars: Map[String
   @GuardedBy("self")
   private var daemonPort: Int = 0
   @GuardedBy("self")
-  private val daemonWorkers = new mutable.WeakHashMap[Socket, Int]()
+  private val daemonWorkers = new mutable.WeakHashMap[PythonWorker, ProcessHandle]()
   @GuardedBy("self")
-  private val idleWorkers = new mutable.Queue[Socket]()
+  private val idleWorkers = new mutable.Queue[PythonWorker]()
   @GuardedBy("self")
   private var lastActivityNs = 0L
   new MonitorThread().start()
 
   @GuardedBy("self")
-  private val simpleWorkers = new mutable.WeakHashMap[Socket, Process]()
+  private val simpleWorkers = new mutable.WeakHashMap[PythonWorker, Process]()
 
   private val pythonPath = PythonUtils.mergePythonPaths(
     PythonUtils.sparkPythonPath,
     envVars.getOrElse("PYTHONPATH", ""),
     sys.env.getOrElse("PYTHONPATH", ""))
 
-  def create(): (Socket, Option[Int]) = {
+  def create(): (PythonWorker, Option[ProcessHandle]) = {
     if (useDaemon) {
       self.synchronized {
-        if (idleWorkers.nonEmpty) {
+        // Pull from idle workers until we one that is alive, otherwise create a new one.
+        while (idleWorkers.nonEmpty) {
           val worker = idleWorkers.dequeue()
-          return (worker, daemonWorkers.get(worker))
+          val workerHandle = daemonWorkers(worker)
+          if (workerHandle.isAlive()) {
+            try {
+              return (worker.refresh(), Some(workerHandle))
+            } catch {
+              case c: CancelledKeyException => /* pass */
+            }
+          }
+          logWarning(log"Worker ${MDC(WORKER, worker)} " +
+            log"process from idle queue is dead, discarding.")
+          stopWorker(worker)
         }
       }
       createThroughDaemon()
     } else {
-      createSimpleWorker(workerModule)
+      createSimpleWorker(blockingMode = false)
     }
   }
 
-  /** Creates a Python worker with streaming worker module. */
-  def createStreamingWorker(streamingWorkerModule: String): (Socket, Option[Int]) = {
-    createSimpleWorker(streamingWorkerModule)
-  }
-
   /**
    * Connect to a worker launched through pyspark/daemon.py (by default), which forks python
    * processes itself to avoid the high cost of forking from Java. This currently only works
    * on UNIX-based systems.
    */
-  private def createThroughDaemon(): (Socket, Option[Int]) = {
+  private def createThroughDaemon(): (PythonWorker, Option[ProcessHandle]) = {
 
-    def createSocket(): (Socket, Option[Int]) = {
-      val socket = new Socket(daemonHost, daemonPort)
-      val pid = new DataInputStream(socket.getInputStream).readInt()
+    def createWorker(): (PythonWorker, Option[ProcessHandle]) = {
+      val socketChannel = SocketChannel.open(new InetSocketAddress(daemonHost, daemonPort))
+      // These calls are blocking.
+      val pid = new DataInputStream(Channels.newInputStream(socketChannel)).readInt()
       if (pid < 0) {
         throw new IllegalStateException("Python daemon failed to launch worker with code " + pid)
       }
-
-      authHelper.authToServer(socket)
-      daemonWorkers.put(socket, pid)
-      (socket, Some(pid))
+      val processHandle = ProcessHandle.of(pid).orElseThrow(
+        () => new IllegalStateException("Python daemon failed to launch worker.")
+      )
+      authHelper.authToServer(socketChannel.socket())
+      socketChannel.configureBlocking(false)
+      val worker = PythonWorker(socketChannel)
+      daemonWorkers.put(worker, processHandle)
+      (worker.refresh(), Some(processHandle))
     }
 
     self.synchronized {
@@ -140,14 +174,14 @@ private[spark] class PythonWorkerFactory(pythonExec: String, envVars: Map[String
 
       // Attempt to connect, restart and retry once if it fails
       try {
-        createSocket()
+        createWorker()
       } catch {
         case exc: SocketException =>
           logWarning("Failed to open socket to Python daemon:", exc)
           logWarning("Assuming that daemon unexpectedly quit, attempting to restart")
           stopDaemon()
           startDaemon()
-          createSocket()
+          createWorker()
       }
     }
   }
@@ -155,10 +189,12 @@ private[spark] class PythonWorkerFactory(pythonExec: String, envVars: Map[String
   /**
    * Launch a worker by executing worker.py (by default) directly and telling it to connect to us.
    */
-  private def createSimpleWorker(workerModule: String): (Socket, Option[Int]) = {
-    var serverSocket: ServerSocket = null
+  private[spark] def createSimpleWorker(
+      blockingMode: Boolean): (PythonWorker, Option[ProcessHandle]) = {
+    var serverSocketChannel: ServerSocketChannel = null
     try {
-      serverSocket = new ServerSocket(0, 1, InetAddress.getLoopbackAddress())
+      serverSocketChannel = ServerSocketChannel.open()
+      serverSocketChannel.bind(new InetSocketAddress(InetAddress.getLoopbackAddress(), 0), 1)
 
       // Create and start the worker
       val pb = new ProcessBuilder(Arrays.asList(pythonExec, "-m", workerModule))
@@ -173,41 +209,53 @@ private[spark] class PythonWorkerFactory(pythonExec: String, envVars: Map[String
       workerEnv.put("PYTHONPATH", pythonPath)
       // This is equivalent to setting the -u flag; we use it because ipython doesn't support -u:
       workerEnv.put("PYTHONUNBUFFERED", "YES")
-      workerEnv.put("PYTHON_WORKER_FACTORY_PORT", serverSocket.getLocalPort.toString)
+      workerEnv.put("PYTHON_WORKER_FACTORY_PORT", serverSocketChannel.socket().getLocalPort
+        .toString)
       workerEnv.put("PYTHON_WORKER_FACTORY_SECRET", authHelper.secret)
       if (Utils.preferIPv6) {
         workerEnv.put("SPARK_PREFER_IPV6", "True")
       }
-      val worker = pb.start()
+      val workerProcess = pb.start()
 
       // Redirect worker stdout and stderr
-      redirectStreamsToStderr(worker.getInputStream, worker.getErrorStream)
+      redirectStreamsToStderr(workerProcess.getInputStream, workerProcess.getErrorStream)
 
       // Wait for it to connect to our socket, and validate the auth secret.
-      serverSocket.setSoTimeout(10000)
-
       try {
-        val socket = serverSocket.accept()
-        authHelper.authClient(socket)
-        // TODO: When we drop JDK 8, we can just use worker.pid()
-        val pid = new DataInputStream(socket.getInputStream).readInt()
+        // Wait up to 10 seconds for client to connect.
+        serverSocketChannel.configureBlocking(false)
+        val serverSelector = Selector.open()
+        serverSocketChannel.register(serverSelector, SelectionKey.OP_ACCEPT)
+        val socketChannel =
+          if (serverSelector.select(10 * 1000) > 0) { // Wait up to 10 seconds.
+            serverSocketChannel.accept()
+          } else {
+            throw new SocketTimeoutException(
+              "Timed out while waiting for the Python worker to connect back")
+          }
+        authHelper.authClient(socketChannel.socket())
+        // TODO: When we drop JDK 8, we can just use workerProcess.pid()
+        val pid = new DataInputStream(Channels.newInputStream(socketChannel)).readInt()
         if (pid < 0) {
           throw new IllegalStateException("Python failed to launch worker with code " + pid)
         }
-        self.synchronized {
-          simpleWorkers.put(socket, worker)
+        if (!blockingMode) {
+          socketChannel.configureBlocking(false)
         }
-        return (socket, Some(pid))
+        val worker = PythonWorker(socketChannel)
+        self.synchronized {
+          simpleWorkers.put(worker, workerProcess)
+        }
+        (worker.refresh(), ProcessHandle.of(pid).toScala)
       } catch {
         case e: Exception =>
           throw new SparkException("Python worker failed to connect back.", e)
       }
     } finally {
-      if (serverSocket != null) {
-        serverSocket.close()
+      if (serverSocketChannel != null) {
+        serverSocketChannel.close()
       }
     }
-    null
   }
 
   private def startDaemon(): Unit = {
@@ -219,7 +267,7 @@ private[spark] class PythonWorkerFactory(pythonExec: String, envVars: Map[String
 
       try {
         // Create and start the daemon
-        val command = Arrays.asList(pythonExec, "-m", daemonModule)
+        val command = Arrays.asList(pythonExec, "-m", daemonModule, workerModule)
         val pb = new ProcessBuilder(command)
         val jobArtifactUUID = envVars.getOrElse("SPARK_JOB_ARTIFACT_UUID", "default")
         if (jobArtifactUUID != "default") {
@@ -339,11 +387,10 @@ private[spark] class PythonWorkerFactory(pythonExec: String, envVars: Map[String
     while (idleWorkers.nonEmpty) {
       val worker = idleWorkers.dequeue()
       try {
-        // the worker will exit after closing the socket
-        worker.close()
+        worker.stop()
       } catch {
         case e: Exception =>
-          logWarning("Failed to close worker socket", e)
+          logWarning("Failed to stop worker socket", e)
       }
     }
   }
@@ -361,7 +408,7 @@ private[spark] class PythonWorkerFactory(pythonExec: String, envVars: Map[String
         daemon = null
         daemonPort = 0
       } else {
-        simpleWorkers.mapValues(_.destroy())
+        simpleWorkers.values.foreach(_.destroy())
       }
     }
   }
@@ -370,14 +417,14 @@ private[spark] class PythonWorkerFactory(pythonExec: String, envVars: Map[String
     stopDaemon()
   }
 
-  def stopWorker(worker: Socket): Unit = {
+  def stopWorker(worker: PythonWorker): Unit = {
     self.synchronized {
       if (useDaemon) {
         if (daemon != null) {
-          daemonWorkers.get(worker).foreach { pid =>
+          daemonWorkers.get(worker).foreach { processHandle =>
             // tell daemon to kill worker by pid
             val output = new DataOutputStream(daemon.getOutputStream)
-            output.writeInt(pid)
+            output.writeInt(processHandle.pid().toInt)
             output.flush()
             daemon.getOutputStream.flush()
           }
@@ -386,28 +433,34 @@ private[spark] class PythonWorkerFactory(pythonExec: String, envVars: Map[String
         simpleWorkers.get(worker).foreach(_.destroy())
       }
     }
-    worker.close()
+    worker.stop()
   }
 
-  def releaseWorker(worker: Socket): Unit = {
+  def releaseWorker(worker: PythonWorker): Unit = {
     if (useDaemon) {
       self.synchronized {
         lastActivityNs = System.nanoTime()
         idleWorkers.enqueue(worker)
       }
     } else {
-      // Cleanup the worker socket. This will also cause the Python worker to exit.
       try {
-        worker.close()
+        worker.stop()
       } catch {
         case e: Exception =>
-          logWarning("Failed to close worker socket", e)
+          logWarning("Failed to close worker", e)
       }
     }
   }
+
+  def isWorkerStopped(worker: PythonWorker): Boolean = {
+    assert(!useDaemon, "isWorkerStopped() is not supported for daemon mode")
+    simpleWorkers.get(worker).exists(!_.isAlive)
+  }
 }
 
-private object PythonWorkerFactory {
+private[spark] object PythonWorkerFactory {
   val PROCESS_WAIT_TIMEOUT_MS = 10000
   val IDLE_WORKER_TIMEOUT_NS = TimeUnit.MINUTES.toNanos(1)  // kill idle workers after 1 minute
+
+  private[spark] val defaultDaemonModule = "pyspark.daemon"
 }
