@@ -110,4 +110,8 @@ private[spark] object RUtils {
   def isEncryptionEnabled(sc: JavaSparkContext): Boolean = {
     sc.conf.get(org.apache.spark.internal.config.IO_ENCRYPTION_ENABLED)
   }
+
+  def getJobTags(sc: JavaSparkContext): Array[String] = {
+    sc.getJobTags().toArray().map(_.asInstanceOf[String])
+  }
 }
