@@ -19,7 +19,8 @@ package org.apache.spark.deploy.worker
 
 import java.util.concurrent.atomic.AtomicBoolean
 
-import org.apache.spark.internal.Logging
+import org.apache.spark.internal.{Logging, LogKeys, MDC}
+import org.apache.spark.internal.LogKeys.WORKER_URL
 import org.apache.spark.rpc._
 
 /**
@@ -34,7 +35,7 @@ private[spark] class WorkerWatcher(
     isChildProcessStopping: AtomicBoolean = new AtomicBoolean(false))
   extends RpcEndpoint with Logging {
 
-  logInfo(s"Connecting to worker $workerUrl")
+  logInfo(log"Connecting to worker ${MDC(WORKER_URL, workerUrl)}")
   if (!isTesting) {
     rpcEnv.asyncSetupEndpointRefByURI(workerUrl)
   }
@@ -63,19 +64,19 @@ private[spark] class WorkerWatcher(
     }
 
   override def receive: PartialFunction[Any, Unit] = {
-    case e => logWarning(s"Received unexpected message: $e")
+    case e => logWarning(log"Received unexpected message: ${MDC(LogKeys.ERROR, e)}")
   }
 
   override def onConnected(remoteAddress: RpcAddress): Unit = {
     if (isWorker(remoteAddress)) {
-      logInfo(s"Successfully connected to $workerUrl")
+      logInfo(log"Successfully connected to ${MDC(WORKER_URL, workerUrl)}")
     }
   }
 
   override def onDisconnected(remoteAddress: RpcAddress): Unit = {
     if (isWorker(remoteAddress)) {
       // This log message will never be seen
-      logError(s"Lost connection to worker rpc endpoint $workerUrl. Exiting.")
+      logError(log"Lost connection to worker rpc endpoint ${MDC(WORKER_URL, workerUrl)}. Exiting.")
       exitNonZero()
     }
   }
@@ -83,8 +84,9 @@ private[spark] class WorkerWatcher(
   override def onNetworkError(cause: Throwable, remoteAddress: RpcAddress): Unit = {
     if (isWorker(remoteAddress)) {
       // These logs may not be seen if the worker (and associated pipe) has died
-      logError(s"Could not initialize connection to worker $workerUrl. Exiting.")
-      logError(s"Error was: $cause")
+      logError(
+        log"Could not initialize connection to worker ${MDC(WORKER_URL, workerUrl)}. Exiting.",
+        cause)
       exitNonZero()
     }
   }
