@@ -22,7 +22,8 @@ import javax.annotation.concurrent.GuardedBy
 import scala.util.control.NonFatal
 
 import org.apache.spark.SparkException
-import org.apache.spark.internal.Logging
+import org.apache.spark.internal.{Logging, MDC}
+import org.apache.spark.internal.LogKeys._
 import org.apache.spark.rpc.{RpcAddress, RpcEndpoint, ThreadSafeRpcEndpoint}
 
 
@@ -193,7 +194,8 @@ private[netty] class Inbox(val endpointName: String, val endpoint: RpcEndpoint)
    * Exposed for testing.
    */
   protected def onDrop(message: InboxMessage): Unit = {
-    logWarning(s"Drop $message because endpoint $endpointName is stopped")
+    logWarning(log"Drop ${MDC(MESSAGE, message)} " +
+      log"because endpoint ${MDC(END_POINT, endpointName)} is stopped")
   }
 
   /**
@@ -206,7 +208,8 @@ private[netty] class Inbox(val endpointName: String, val endpoint: RpcEndpoint)
         // Should reduce the number of active threads before throw the error.
         numActiveThreads -= 1
       }
-      logError(s"An error happened while processing message in the inbox for $endpointName", fatal)
+      logError(log"An error happened while processing message in the inbox for" +
+        log" ${MDC(END_POINT, endpointName)}", fatal)
       throw fatal
     }
 
