@@ -19,16 +19,16 @@ package org.apache.spark.shuffle
 
 import java.io.{File, FileNotFoundException}
 import java.net.ConnectException
-import java.nio.ByteBuffer
 import java.util.concurrent.ExecutorService
 
 import scala.collection.mutable.{ArrayBuffer, HashMap, HashSet, Queue}
 import scala.util.control.NonFatal
 
-import org.apache.spark.{ShuffleDependency, SparkConf, SparkContext, SparkEnv}
+import org.apache.spark.{SecurityManager, ShuffleDependency, SparkConf, SparkContext, SparkEnv}
 import org.apache.spark.annotation.Since
 import org.apache.spark.executor.{CoarseGrainedExecutorBackend, ExecutorBackend}
-import org.apache.spark.internal.Logging
+import org.apache.spark.internal.{Logging, MDC}
+import org.apache.spark.internal.LogKeys._
 import org.apache.spark.internal.config._
 import org.apache.spark.launcher.SparkLauncher
 import org.apache.spark.network.buffer.{FileSegmentManagedBuffer, ManagedBuffer, NioManagedBuffer}
@@ -108,7 +108,9 @@ private[spark] class ShuffleBlockPusher(conf: SparkConf) extends Logging {
       dep: ShuffleDependency[_, _, _],
       mapIndex: Int): Unit = {
     val numPartitions = dep.partitioner.numPartitions
-    val transportConf = SparkTransportConf.fromSparkConf(conf, "shuffle")
+    val securityManager = new SecurityManager(conf)
+    val transportConf = SparkTransportConf.fromSparkConf(
+      conf, "shuffle", sslOptions = Some(securityManager.getRpcSSLOptions()))
     this.shuffleId = dep.shuffleId
     this.shuffleMergeId = dep.shuffleMergeId
     this.mapIndex = mapIndex
@@ -247,7 +249,8 @@ private[spark] class ShuffleBlockPusher(conf: SparkConf) extends Logging {
         if (!errorHandler.shouldLogError(exception)) {
           logTrace(s"Pushing block $blockId to $address failed.", exception)
         } else {
-          logWarning(s"Pushing block $blockId to $address failed.", exception)
+          logWarning(log"Pushing block ${MDC(BLOCK_ID, blockId)} " +
+            log"to ${MDC(HOST_PORT, address)} failed.", exception)
         }
         handleResult(PushResult(blockId, exception))
       }
@@ -293,7 +296,7 @@ private[spark] class ShuffleBlockPusher(conf: SparkConf) extends Logging {
         case (offset, size) =>
           new NioManagedBuffer(inMemoryBuffer.duplicate()
             .position(offset)
-            .limit(offset + size).asInstanceOf[ByteBuffer].slice())
+            .limit(offset + size).slice())
       }.toArray
     }
   }
@@ -328,9 +331,9 @@ private[spark] class ShuffleBlockPusher(conf: SparkConf) extends Logging {
         unreachableBlockMgrs.add(address)
         removed += pushRequests.dequeueAll(req => req.address == address).length
         removed += deferredPushRequests.remove(address).map(_.length).getOrElse(0)
-        logWarning(s"Received a ConnectException from $address. " +
-          s"Dropping $removed push-requests and " +
-          s"not pushing any more blocks to this address.")
+        logWarning(log"Received a ConnectException from ${MDC(HOST_PORT, address)}. " +
+          log"Dropping ${MDC(NUM_REQUESTS, removed)} push-requests and " +
+          log"not pushing any more blocks to this address.")
       }
     }
     if (pushResult.failure != null && !errorHandler.shouldRetryError(pushResult.failure)) {
@@ -359,7 +362,8 @@ private[spark] class ShuffleBlockPusher(conf: SparkConf) extends Logging {
         case Some(cb: CoarseGrainedExecutorBackend) =>
           cb.notifyDriverAboutPushCompletion(shuffleId, shuffleMergeId, mapIndex)
         case Some(eb: ExecutorBackend) =>
-          logWarning(s"Currently $eb doesn't support push-based shuffle")
+          logWarning(log"Currently ${MDC(EXECUTOR_BACKEND, eb)} " +
+            log"doesn't support push-based shuffle")
         case None =>
       }
       pushCompletionNotified = true
