@@ -20,9 +20,9 @@ package org.apache.spark.status
 import java.util.Date
 import java.util.concurrent.atomic.AtomicInteger
 
-import scala.collection.JavaConverters._
 import scala.collection.immutable.{HashSet, TreeSet}
 import scala.collection.mutable.HashMap
+import scala.jdk.CollectionConverters._
 
 import org.apache.spark.JobExecutionStatus
 import org.apache.spark.executor.{ExecutorMetrics, TaskMetrics}
@@ -364,7 +364,7 @@ private[spark] class LiveExecutor(val executorId: String, _addTime: Long) extend
       executorLogs,
       memoryMetrics,
       excludedInStages,
-      Some(peakExecutorMetrics).filter(_.isSet),
+      Some(peakExecutorMetrics).filter(_.isSet()),
       attributes,
       resources,
       resourceProfileId,
@@ -408,7 +408,7 @@ private class LiveExecutorStageSummary(
       metrics.memoryBytesSpilled,
       metrics.diskBytesSpilled,
       isExcluded,
-      Some(peakExecutorMetrics).filter(_.isSet),
+      Some(peakExecutorMetrics).filter(_.isSet()),
       isExcluded)
     new ExecutorStageSummaryWrapper(stageId, attemptId, executorId, info)
   }
@@ -472,7 +472,7 @@ private class LiveStage(var info: StageInfo) extends LiveEntity {
   val peakExecutorMetrics = new ExecutorMetrics()
 
   lazy val speculationStageSummary: LiveSpeculationStageSummary =
-    new LiveSpeculationStageSummary(info.stageId, info.attemptNumber)
+    new LiveSpeculationStageSummary(info.stageId, info.attemptNumber())
 
   // Used for cleanup of tasks after they reach the configured limit. Not written to the store.
   @volatile var cleaning = false
@@ -480,14 +480,14 @@ private class LiveStage(var info: StageInfo) extends LiveEntity {
 
   def executorSummary(executorId: String): LiveExecutorStageSummary = {
     executorSummaries.getOrElseUpdate(executorId,
-      new LiveExecutorStageSummary(info.stageId, info.attemptNumber, executorId))
+      new LiveExecutorStageSummary(info.stageId, info.attemptNumber(), executorId))
   }
 
   def toApi(): v1.StageData = {
     new v1.StageData(
       status = status,
       stageId = info.stageId,
-      attemptId = info.attemptNumber,
+      attemptId = info.attemptNumber(),
       numTasks = info.numTasks,
       numActiveTasks = activeTasks,
       numCompleteTasks = completedTasks,
@@ -559,7 +559,7 @@ private class LiveStage(var info: StageInfo) extends LiveEntity {
       speculationSummary = None,
       killedTasksSummary = killedSummary,
       resourceProfileId = info.resourceProfileId,
-      peakExecutorMetrics = Some(peakExecutorMetrics).filter(_.isSet),
+      peakExecutorMetrics = Some(peakExecutorMetrics).filter(_.isSet()),
       taskMetricsDistributions = None,
       executorMetricsDistributions = None,
       isShufflePushEnabled = info.isShufflePushEnabled,
@@ -859,49 +859,40 @@ private[spark] object LiveEntityHelpers {
     }
 
     createMetrics(
-      executorDeserializeTime = updateMetricValue(m.executorDeserializeTime),
-      executorDeserializeCpuTime = updateMetricValue(m.executorDeserializeCpuTime),
-      executorRunTime = updateMetricValue(m.executorRunTime),
-      executorCpuTime = updateMetricValue(m.executorCpuTime),
-      resultSize = updateMetricValue(m.resultSize),
-      jvmGcTime = updateMetricValue(m.jvmGcTime),
-      resultSerializationTime = updateMetricValue(m.resultSerializationTime),
-      memoryBytesSpilled = updateMetricValue(m.memoryBytesSpilled),
-      diskBytesSpilled = updateMetricValue(m.diskBytesSpilled),
-      peakExecutionMemory = updateMetricValue(m.peakExecutionMemory),
-      inputBytesRead = updateMetricValue(m.inputMetrics.bytesRead),
-      inputRecordsRead = updateMetricValue(m.inputMetrics.recordsRead),
-      outputBytesWritten = updateMetricValue(m.outputMetrics.bytesWritten),
-      outputRecordsWritten = updateMetricValue(m.outputMetrics.recordsWritten),
-      shuffleRemoteBlocksFetched = updateMetricValue(m.shuffleReadMetrics.remoteBlocksFetched),
-      shuffleLocalBlocksFetched = updateMetricValue(m.shuffleReadMetrics.localBlocksFetched),
-      shuffleFetchWaitTime = updateMetricValue(m.shuffleReadMetrics.fetchWaitTime),
-      shuffleRemoteBytesRead = updateMetricValue(m.shuffleReadMetrics.remoteBytesRead),
-      shuffleRemoteBytesReadToDisk = updateMetricValue(m.shuffleReadMetrics.remoteBytesReadToDisk),
-      shuffleLocalBytesRead = updateMetricValue(m.shuffleReadMetrics.localBytesRead),
-      shuffleRecordsRead = updateMetricValue(m.shuffleReadMetrics.recordsRead),
-      shuffleCorruptMergedBlockChunks =
-        updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.corruptMergedBlockChunks),
-      shuffleMergedFetchFallbackCount =
-        updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.mergedFetchFallbackCount),
-      shuffleMergedRemoteBlocksFetched =
-        updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.remoteMergedBlocksFetched),
-      shuffleMergedLocalBlocksFetched =
-        updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.localMergedBlocksFetched),
-      shuffleMergedRemoteChunksFetched =
-        updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.remoteMergedChunksFetched),
-      shuffleMergedLocalChunksFetched =
-        updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.localMergedChunksFetched),
-      shuffleMergedRemoteBytesRead =
-        updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.remoteMergedBytesRead),
-      shuffleMergedLocalBytesRead =
-        updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.localMergedBytesRead),
-      shuffleRemoteReqsDuration = updateMetricValue(m.shuffleReadMetrics.remoteReqsDuration),
-      shuffleMergedRemoteReqsDuration =
-        updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.remoteMergedReqsDuration),
-      shuffleBytesWritten = updateMetricValue(m.shuffleWriteMetrics.bytesWritten),
-      shuffleWriteTime = updateMetricValue(m.shuffleWriteMetrics.writeTime),
-      shuffleRecordsWritten = updateMetricValue(m.shuffleWriteMetrics.recordsWritten))
+      updateMetricValue(m.executorDeserializeTime),
+      updateMetricValue(m.executorDeserializeCpuTime),
+      updateMetricValue(m.executorRunTime),
+      updateMetricValue(m.executorCpuTime),
+      updateMetricValue(m.resultSize),
+      updateMetricValue(m.jvmGcTime),
+      updateMetricValue(m.resultSerializationTime),
+      updateMetricValue(m.memoryBytesSpilled),
+      updateMetricValue(m.diskBytesSpilled),
+      updateMetricValue(m.peakExecutionMemory),
+      updateMetricValue(m.inputMetrics.bytesRead),
+      updateMetricValue(m.inputMetrics.recordsRead),
+      updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.corruptMergedBlockChunks),
+      updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.mergedFetchFallbackCount),
+      updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.remoteMergedBlocksFetched),
+      updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.localMergedBlocksFetched),
+      updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.remoteMergedChunksFetched),
+      updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.localMergedChunksFetched),
+      updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.remoteMergedBytesRead),
+      updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.localMergedBytesRead),
+      updateMetricValue(m.shuffleReadMetrics.remoteReqsDuration),
+      updateMetricValue(m.shuffleReadMetrics.shufflePushReadMetrics.remoteMergedReqsDuration),
+      updateMetricValue(m.outputMetrics.bytesWritten),
+      updateMetricValue(m.outputMetrics.recordsWritten),
+      updateMetricValue(m.shuffleReadMetrics.remoteBlocksFetched),
+      updateMetricValue(m.shuffleReadMetrics.localBlocksFetched),
+      updateMetricValue(m.shuffleReadMetrics.fetchWaitTime),
+      updateMetricValue(m.shuffleReadMetrics.remoteBytesRead),
+      updateMetricValue(m.shuffleReadMetrics.remoteBytesReadToDisk),
+      updateMetricValue(m.shuffleReadMetrics.localBytesRead),
+      updateMetricValue(m.shuffleReadMetrics.recordsRead),
+      updateMetricValue(m.shuffleWriteMetrics.bytesWritten),
+      updateMetricValue(m.shuffleWriteMetrics.writeTime),
+      updateMetricValue(m.shuffleWriteMetrics.recordsWritten))
   }
 
   private def addMetrics(m1: v1.TaskMetrics, m2: v1.TaskMetrics, mult: Int): v1.TaskMetrics = {
