@@ -24,7 +24,8 @@ import java.util.zip.Checksum
 
 import org.apache.spark.SparkException
 import org.apache.spark.errors.SparkCoreErrors
-import org.apache.spark.internal.Logging
+import org.apache.spark.internal.{Logging, MDC}
+import org.apache.spark.internal.LogKeys._
 import org.apache.spark.io.MutableCheckedOutputStream
 import org.apache.spark.serializer.{SerializationStream, SerializerInstance, SerializerManager}
 import org.apache.spark.shuffle.ShuffleWriteMetricsReporter
@@ -63,7 +64,7 @@ private[spark] class DiskBlockObjectWriter(
    */
   private trait ManualCloseOutputStream extends OutputStream {
     abstract override def close(): Unit = {
-      flush()
+      this.flush()
     }
 
     def manualClose(): Unit = {
@@ -74,8 +75,8 @@ private[spark] class DiskBlockObjectWriter(
         // get IOException when flushing the buffered data. We should catch and log the exception
         // to ensure the revertPartialWritesAndClose() function doesn't throw an exception.
         case e: IOException =>
-          logError("Exception occurred while manually close the output stream to file "
-            + file + ", " + e.getMessage)
+          logError(log"Exception occurred while manually close the output stream to file "
+            + log"${MDC(PATH, file)}, ${MDC(ERROR, e.getMessage)}")
       }
     }
   }
@@ -192,7 +193,8 @@ private[spark] class DiskBlockObjectWriter(
       }
     } catch {
       case e: IOException =>
-        logInfo("Exception occurred while closing the output stream: " + e.getMessage)
+        logInfo(log"Exception occurred while closing the output stream" +
+          log"${MDC(ERROR, e.getMessage)}")
     } finally {
       if (initialized) {
         Utils.tryWithSafeFinally {
@@ -289,10 +291,11 @@ private[spark] class DiskBlockObjectWriter(
         // don't log the exception stack trace to avoid confusing users.
         // See: SPARK-28340
         case ce: ClosedByInterruptException =>
-          logError("Exception occurred while reverting partial writes to file "
-            + file + ", " + ce.getMessage)
+          logError(log"Exception occurred while reverting partial writes to file "
+            + log"${MDC(PATH, file)}, ${MDC(ERROR, ce.getMessage)}")
         case e: Exception =>
-          logError("Uncaught exception while reverting partial writes to file " + file, e)
+          logError(
+            log"Uncaught exception while reverting partial writes to file ${MDC(PATH, file)}", e)
       } finally {
         if (truncateStream != null) {
           truncateStream.close()
@@ -317,7 +320,7 @@ private[spark] class DiskBlockObjectWriter(
       }
     } {
       if (!Files.deleteIfExists(file.toPath)) {
-        logWarning(s"Error deleting $file")
+        logWarning(log"Error deleting ${MDC(FILE_NAME, file)}")
       }
     }
   }
