@@ -25,9 +25,9 @@ import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 
-import org.junit.Test;
-import static org.junit.Assert.*;
-import static org.junit.Assume.*;
+import org.junit.jupiter.api.Test;
+import static org.junit.jupiter.api.Assertions.*;
+import static org.junit.jupiter.api.Assumptions.*;
 import static org.mockito.Mockito.*;
 
 import org.apache.spark.SparkContext;
@@ -187,16 +187,9 @@ public class SparkLauncherSuite extends BaseSuite {
       InProcessLauncher launcher = new InProcessLauncher()
         .setAppResource(SparkLauncher.NO_RESOURCE);
       switch (args.length) {
-        case 2:
-          launcher.addSparkArg(args[0], args[1]);
-          break;
-
-        case 1:
-          launcher.addSparkArg(args[0]);
-          break;
-
-        default:
-          fail("FIXME: invalid test.");
+        case 2 -> launcher.addSparkArg(args[0], args[1]);
+        case 1 -> launcher.addSparkArg(args[0]);
+        default -> fail("FIXME: invalid test.");
       }
 
       SparkAppHandle handle = launcher.startApplication();
@@ -280,9 +273,8 @@ public class SparkLauncherSuite extends BaseSuite {
     // Here DAGScheduler is stopped, while SparkContext.clearActiveContext may not be called yet.
     // Wait for a reasonable amount of time to avoid creating two active SparkContext in JVM.
     // See SPARK-23019 and SparkContext.stop() for details.
-    eventually(Duration.ofSeconds(5), Duration.ofMillis(10), () -> {
-      assertTrue("SparkContext is still alive.", SparkContext$.MODULE$.getActive().isEmpty());
-    });
+    eventually(Duration.ofSeconds(5), Duration.ofMillis(10), () ->
+      assertTrue(SparkContext$.MODULE$.getActive().isEmpty(), "SparkContext is still alive."));
   }
 
   public static class SparkLauncherTestApp {
