@@ -30,9 +30,9 @@ import scala.*;
 import scala.collection.Iterator;
 
 import com.google.common.collect.HashMultiset;
-import org.junit.After;
-import org.junit.Before;
-import org.junit.Test;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
 import org.mockito.Mock;
 import org.mockito.MockitoAnnotations;
 
@@ -54,7 +54,7 @@ import org.apache.spark.shuffle.sort.io.LocalDiskShuffleExecutorComponents;
 import org.apache.spark.storage.*;
 import org.apache.spark.util.Utils;
 
-import static org.junit.Assert.*;
+import static org.junit.jupiter.api.Assertions.*;
 import static org.mockito.Answers.RETURNS_SMART_NULLS;
 import static org.mockito.Mockito.*;
 
@@ -81,7 +81,7 @@ public class UnsafeShuffleWriterSuite implements ShuffleChecksumTestHelper {
   @Mock(answer = RETURNS_SMART_NULLS) TaskContext taskContext;
   @Mock(answer = RETURNS_SMART_NULLS) ShuffleDependency<Object, Object, Object> shuffleDep;
 
-  @After
+  @AfterEach
   public void tearDown() {
     Utils.deleteRecursively(tempDir);
     final long leakedMemory = taskMemoryManager.cleanUpAllAllocatedMemory();
@@ -90,7 +90,7 @@ public class UnsafeShuffleWriterSuite implements ShuffleChecksumTestHelper {
     }
   }
 
-  @Before
+  @BeforeEach
   public void setUp() throws Exception {
     MockitoAnnotations.openMocks(this).close();
     tempDir = Utils.createTempDir(null, "test");
@@ -204,8 +204,8 @@ public class UnsafeShuffleWriterSuite implements ShuffleChecksumTestHelper {
 
   private void assertSpillFilesWereCleanedUp() {
     for (File spillFile : spillFilesCreated) {
-      assertFalse("Spill file " + spillFile.getPath() + " was not cleaned up",
-        spillFile.exists());
+      assertFalse(spillFile.exists(),
+        "Spill file " + spillFile.getPath() + " was not cleaned up");
     }
   }
 
@@ -215,22 +215,24 @@ public class UnsafeShuffleWriterSuite implements ShuffleChecksumTestHelper {
     for (int i = 0; i < NUM_PARTITIONS; i++) {
       final long partitionSize = partitionSizesInMergedFile[i];
       if (partitionSize > 0) {
-        FileInputStream fin = new FileInputStream(mergedOutputFile);
-        fin.getChannel().position(startOffset);
-        InputStream in = new LimitedInputStream(fin, partitionSize);
-        in = blockManager.serializerManager().wrapForEncryption(in);
-        if ((boolean) conf.get(package$.MODULE$.SHUFFLE_COMPRESS())) {
-          in = CompressionCodec$.MODULE$.createCodec(conf).compressedInputStream(in);
-        }
-        try (DeserializationStream recordsStream = serializer.newInstance().deserializeStream(in)) {
-          Iterator<Tuple2<Object, Object>> records = recordsStream.asKeyValueIterator();
-          while (records.hasNext()) {
-            Tuple2<Object, Object> record = records.next();
-            assertEquals(i, hashPartitioner.getPartition(record._1()));
-            recordsList.add(record);
+        try (FileInputStream fin = new FileInputStream(mergedOutputFile)) {
+          fin.getChannel().position(startOffset);
+          InputStream in = new LimitedInputStream(fin, partitionSize);
+          in = blockManager.serializerManager().wrapForEncryption(in);
+          if ((boolean) conf.get(package$.MODULE$.SHUFFLE_COMPRESS())) {
+            in = CompressionCodec$.MODULE$.createCodec(conf).compressedInputStream(in);
           }
+          try (
+            DeserializationStream recordsStream = serializer.newInstance().deserializeStream(in)) {
+            Iterator<Tuple2<Object, Object>> records = recordsStream.asKeyValueIterator();
+            while (records.hasNext()) {
+              Tuple2<Object, Object> record = records.next();
+              assertEquals(i, hashPartitioner.getPartition(record._1()));
+              recordsList.add(record);
+            }
+          }
+          startOffset += partitionSize;
         }
-        startOffset += partitionSize;
       }
     }
     return recordsList;
@@ -242,7 +244,7 @@ public class UnsafeShuffleWriterSuite implements ShuffleChecksumTestHelper {
   }
 
   @Test
-  public void doNotNeedToCallWriteBeforeUnsuccessfulStop() throws IOException, SparkException {
+  public void doNotNeedToCallWriteBeforeUnsuccessfulStop() throws SparkException {
     createWriter(false).stop(false);
   }
 
