@@ -17,15 +17,12 @@
 package org.apache.spark.security
 
 import java.io._
-import java.nio.ByteBuffer
-import java.nio.channels.{Channels, ReadableByteChannel}
+import java.nio.channels.Channels
 import java.nio.charset.StandardCharsets.UTF_8
 import java.nio.file.Files
 import java.util.{Arrays, Random, UUID}
 
 import com.google.common.io.ByteStreams
-import org.mockito.ArgumentMatchers.any
-import org.mockito.Mockito._
 
 import org.apache.spark._
 import org.apache.spark.internal.config._
@@ -167,36 +164,6 @@ class CryptoStreamUtilsSuite extends SparkFunSuite {
     }
   }
 
-  test("error handling wrapper") {
-    val wrapped = mock(classOf[ReadableByteChannel])
-    val decrypted = mock(classOf[ReadableByteChannel])
-    val errorHandler = new CryptoStreamUtils.ErrorHandlingReadableChannel(decrypted, wrapped)
-
-    when(decrypted.read(any(classOf[ByteBuffer])))
-      .thenThrow(new IOException())
-      .thenThrow(new InternalError())
-      .thenReturn(1)
-
-    val out = ByteBuffer.allocate(1)
-    intercept[IOException] {
-      errorHandler.read(out)
-    }
-    intercept[InternalError] {
-      errorHandler.read(out)
-    }
-
-    val e = intercept[IOException] {
-      errorHandler.read(out)
-    }
-    assert(e.getMessage().contains("is closed"))
-    errorHandler.close()
-
-    verify(decrypted, times(2)).read(any(classOf[ByteBuffer]))
-    verify(wrapped, never()).read(any(classOf[ByteBuffer]))
-    verify(decrypted, never()).close()
-    verify(wrapped, times(1)).close()
-  }
-
   private def createConf(extra: (String, String)*): SparkConf = {
     val conf = new SparkConf()
     extra.foreach { case (k, v) => conf.set(k, v) }
