@@ -37,7 +37,7 @@ import sparktestsupport.modules as modules
 
 def setup_test_environ(environ):
     print("[info] Setup the following environment variables for tests: ")
-    for (k, v) in environ.items():
+    for k, v in environ.items():
         print("%s=%s" % (k, v))
         os.environ[k] = v
 
@@ -181,7 +181,6 @@ def get_scala_profiles(scala_version):
         return []  # assume it's default.
 
     sbt_maven_scala_profiles = {
-        "scala2.12": ["-Pscala-2.12"],
         "scala2.13": ["-Pscala-2.13"],
     }
 
@@ -331,7 +330,6 @@ def run_scala_tests_maven(test_profiles):
 
 
 def run_scala_tests_sbt(test_modules, test_profiles):
-
     sbt_test_goals = list(itertools.chain.from_iterable(m.sbt_test_goals for m in test_modules))
 
     if not sbt_test_goals:
@@ -363,12 +361,13 @@ def run_scala_tests(build_tool, extra_profiles, test_modules, excluded_tags, inc
     if excluded_tags:
         test_profiles += ["-Dtest.exclude.tags=" + ",".join(excluded_tags)]
 
-    # set up java11 env if this is a pull request build with 'test-java11' in the title
-    if "ghprbPullTitle" in os.environ:
-        if "test-java11" in os.environ["ghprbPullTitle"].lower():
-            os.environ["JAVA_HOME"] = "/usr/java/jdk-11.0.1"
-            os.environ["PATH"] = "%s/bin:%s" % (os.environ["JAVA_HOME"], os.environ["PATH"])
-            test_profiles += ["-Djava.version=11"]
+    # SPARK-45296: legacy code for Jenkins. If we move to Jenkins, we should
+    # revive this logic with a different combination of JDK.
+    # if "ghprbPullTitle" in os.environ:
+    #     if "test-java11" in os.environ["ghprbPullTitle"].lower():
+    #         os.environ["JAVA_HOME"] = "/usr/java/jdk-11.0.1"
+    #         os.environ["PATH"] = "%s/bin:%s" % (os.environ["JAVA_HOME"], os.environ["PATH"])
+    #         test_profiles += ["-Djava.version=11"]
 
     if build_tool == "maven":
         run_scala_tests_maven(test_profiles)
@@ -376,7 +375,7 @@ def run_scala_tests(build_tool, extra_profiles, test_modules, excluded_tags, inc
         run_scala_tests_sbt(test_modules, test_profiles)
 
 
-def run_python_tests(test_modules, parallelism, with_coverage=False):
+def run_python_tests(test_modules, test_pythons, parallelism, with_coverage=False):
     set_title_and_block("Running PySpark tests", "BLOCK_PYSPARK_UNIT_TESTS")
 
     if with_coverage:
@@ -391,6 +390,7 @@ def run_python_tests(test_modules, parallelism, with_coverage=False):
     if test_modules != [modules.root]:
         command.append("--modules=%s" % ",".join(m.name for m in test_modules))
     command.append("--parallelism=%i" % parallelism)
+    command.append("--python-executables=%s" % test_pythons)
     run_cmd(command)
 
 
@@ -424,6 +424,12 @@ def parse_opts():
         default=8,
         help="The number of suites to test in parallel (default %(default)d)",
     )
+    parser.add_argument(
+        "--python-executables",
+        type=str,
+        default="python3.9",
+        help="A comma-separated list of Python executables to test against (default: %(default)s)",
+    )
     parser.add_argument(
         "-m",
         "--modules",
@@ -472,6 +478,8 @@ def main():
     rm_r(os.path.join(SPARK_HOME, "work"))
     rm_r(os.path.join(USER_HOME, ".ivy2", "local", "org.apache.spark"))
     rm_r(os.path.join(USER_HOME, ".ivy2", "cache", "org.apache.spark"))
+    rm_r(os.path.join(USER_HOME, ".ivy2.5.2", "local", "org.apache.spark"))
+    rm_r(os.path.join(USER_HOME, ".ivy2.5.2", "cache", "org.apache.spark"))
 
     os.environ["CURRENT_BLOCK"] = str(ERROR_CODES["BLOCK_GENERAL"])
 
@@ -652,6 +660,7 @@ def main():
     if modules_with_python_tests and not os.environ.get("SKIP_PYTHON"):
         run_python_tests(
             modules_with_python_tests,
+            opts.python_executables,
             opts.parallelism,
             with_coverage=os.environ.get("PYSPARK_CODECOV", "false") == "true",
         )
