@@ -15,9 +15,12 @@
  * limitations under the License.
  */
 
+/* global $, anchors */
+
 /* Custom JavaScript code in the MarkDown docs */
 
 // Enable language-specific code tabs
+
 function codeTabs() {
   var counter = 0;
   var langImages = {
@@ -48,12 +51,13 @@ function codeTabs() {
       var capitalizedLang = lang.substr(0, 1).toUpperCase() + lang.substr(1);
       var id = "tab_" + lang + "_" + counter;
       $(this).attr("id", id);
+      var buttonLabel = "";
       if (image != null && langImages[lang]) {
-        var buttonLabel = "<img src='" +langImages[lang] + "' alt='" + capitalizedLang + "' />";
+        buttonLabel = "<img src='" +langImages[lang] + "' alt='" + capitalizedLang + "' />";
       } else if (notabs == null) {
-        var buttonLabel = "<b>" + capitalizedLang + "</b>";
+        buttonLabel = "<b>" + capitalizedLang + "</b>";
       } else {
-        var buttonLabel = ""
+        buttonLabel = ""
       }
 
       // Add the link to the tab
@@ -83,15 +87,6 @@ function codeTabs() {
 }
 
 
-// A script to fix internal hash links because we have an overlapping top bar.
-// Based on https://github.com/twitter/bootstrap/issues/193#issuecomment-2281510
-function maybeScrollToHash() {
-  if (window.location.hash && $(window.location.hash).length) {
-    var newTop = $(window.location.hash).offset().top - 57;
-    $(window).scrollTop(newTop);
-  }
-}
-
 $(function() {
   codeTabs();
   // Display anchor links when hovering over headers. For documentation of the
@@ -101,14 +96,6 @@ $(function() {
   };
   anchors.add();
 
-  $(window).bind('hashchange', function() {
-    maybeScrollToHash();
-  });
-
-  // Scroll now too in case we had opened the page on a hash, but wait a bit because some browsers
-  // will try to do *their* initial scroll after running the onReady handler.
-  $(window).on('load', function() { setTimeout(function() { maybeScrollToHash(); }, 25); });
-
   // Make dropdown menus in nav bars show on hover instead of click
   // using solution at https://webdesign.tutsplus.com/tutorials/how-
   // to-make-the-bootstrap-navbar-dropdown-work-on-hover--cms-33840
