@@ -26,11 +26,16 @@ import org.apache.spark.ml.linalg.{Vector, Vectors, VectorUDT}
 import org.apache.spark.rdd.RDD
 import org.apache.spark.sql.Column
 import org.apache.spark.sql.catalyst.InternalRow
-import org.apache.spark.sql.catalyst.expressions.{Expression, ImplicitCastInputTypes}
+import org.apache.spark.sql.catalyst.expressions.{Expression, ImplicitCastInputTypes, Literal}
 import org.apache.spark.sql.catalyst.expressions.aggregate.TypedImperativeAggregate
 import org.apache.spark.sql.catalyst.trees.BinaryLike
+import org.apache.spark.sql.catalyst.util.ArrayData
+import org.apache.spark.sql.classic.ClassicConversions.ColumnConstructorExt
+import org.apache.spark.sql.classic.ExpressionUtils.expression
 import org.apache.spark.sql.functions.lit
 import org.apache.spark.sql.types._
+import org.apache.spark.unsafe.types.UTF8String
+import org.apache.spark.util.Utils
 
 /**
  * A builder object that provides summary statistics about a given column.
@@ -247,16 +252,13 @@ private[ml] class SummaryBuilderImpl(
   ) extends SummaryBuilder {
 
   override def summary(featuresCol: Column, weightCol: Column): Column = {
-
-    val agg = SummaryBuilderImpl.MetricsAggregate(
+    Column(SummaryBuilderImpl.MetricsAggregate(
       requestedMetrics,
       requestedCompMetrics,
-      featuresCol.expr,
-      weightCol.expr,
+      expression(featuresCol),
+      expression(weightCol),
       mutableAggBufferOffset = 0,
-      inputAggBufferOffset = 0)
-
-    new Column(agg.toAggregateExpression())
+      inputAggBufferOffset = 0))
   }
 }
 
@@ -290,6 +292,14 @@ private[spark] object SummaryBuilderImpl extends Logging {
     StructType(fields)
   }
 
+  private def extractRequestedMetrics(metrics: Expression): (Seq[Metric], Seq[ComputeMetric]) = {
+    metrics.eval() match {
+      case arrayData: ArrayData =>
+        val requested = arrayData.toSeq[UTF8String](StringType)
+        getRelevantMetrics(requested.map(_.toString))
+    }
+  }
+
   private val vectorUDT = new VectorUDT
 
   /**
@@ -342,7 +352,7 @@ private[spark] object SummaryBuilderImpl extends Logging {
   private[stat] case object ComputeMin extends ComputeMetric
 
 
-  private case class MetricsAggregate(
+  private[spark] case class MetricsAggregate(
       requestedMetrics: Seq[Metric],
       requestedComputeMetrics: Seq[ComputeMetric],
       featuresExpr: Expression,
@@ -353,6 +363,27 @@ private[spark] object SummaryBuilderImpl extends Logging {
     with ImplicitCastInputTypes
     with BinaryLike[Expression] {
 
+    // helper constructor
+    def this(
+        metrics: (Seq[Metric], Seq[ComputeMetric]),
+        featuresExpr: Expression,
+        weightExpr: Expression) = {
+      this(metrics._1, metrics._2, featuresExpr, weightExpr, 0, 0)
+    }
+
+    def this(
+        requestedMetrics: Expression,
+        featuresExpr: Expression,
+        weightExpr: Expression) = {
+      this(extractRequestedMetrics(requestedMetrics), featuresExpr, weightExpr)
+    }
+
+    def this(
+        requestedMetrics: Expression,
+        featuresExpr: Expression) = {
+      this(requestedMetrics, featuresExpr, Literal(1.0))
+    }
+
     override def eval(state: SummarizerBuffer): Any = {
       val metrics = requestedMetrics.map {
         case Mean => vectorUDT.serialize(state.mean)
@@ -397,17 +428,12 @@ private[spark] object SummaryBuilderImpl extends Logging {
 
     override def serialize(state: SummarizerBuffer): Array[Byte] = {
       // TODO: Use ByteBuffer to optimize
-      val bos = new ByteArrayOutputStream()
-      val oos = new ObjectOutputStream(bos)
-      oos.writeObject(state)
-      bos.toByteArray
+      Utils.serialize(state)
     }
 
     override def deserialize(bytes: Array[Byte]): SummarizerBuffer = {
       // TODO: Use ByteBuffer to optimize
-      val bis = new ByteArrayInputStream(bytes)
-      val ois = new ObjectInputStream(bis)
-      ois.readObject().asInstanceOf[SummarizerBuffer]
+      Utils.deserialize(bytes)
     }
 
     override def withNewMutableAggBufferOffset(newMutableAggBufferOffset: Int): MetricsAggregate = {
