@@ -24,6 +24,7 @@ import org.apache.spark.ml.util.TestingUtils._
 import org.apache.spark.mllib.feature.{IDFModel => OldIDFModel}
 import org.apache.spark.mllib.linalg.VectorImplicits._
 import org.apache.spark.sql.Row
+import org.apache.spark.util.ArrayImplicits._
 
 class IDFSuite extends MLTest with DefaultReadWriteTest {
 
@@ -38,7 +39,7 @@ class IDFSuite extends MLTest with DefaultReadWriteTest {
         val res = data.indices.zip(data.values).map { case (id, value) =>
           (id, value * model(id))
         }
-        Vectors.sparse(data.size, res)
+        Vectors.sparse(data.size, res.toImmutableArraySeq)
     }
   }
 
