@@ -513,10 +513,10 @@ class MatricesSuite extends SparkFunSuite {
     mat.toString(Int.MinValue, Int.MinValue)
     mat.toString(Int.MaxValue, Int.MaxValue)
     var lines = mat.toString(6, 50).split('\n')
-    assert(lines.size == 5 && lines.forall(_.size <= 50))
+    assert(lines.length == 5 && lines.forall(_.length <= 50))
 
     lines = mat.toString(5, 100).split('\n')
-    assert(lines.size == 5 && lines.forall(_.size <= 100))
+    assert(lines.length == 5 && lines.forall(_.length <= 100))
   }
 
   test("numNonzeros and numActives") {
