@@ -21,6 +21,7 @@ import pandas as pd
 from pandas.api.types import CategoricalDtype
 
 from pyspark import pandas as ps
+from pyspark.testing.pandasutils import PandasOnSparkTestCase
 from pyspark.pandas.tests.data_type_ops.testing_utils import OpsTestBase
 
 
@@ -236,7 +237,11 @@ class DatetimeOpsTestsMixin:
         self.assert_eq(pdf["this"] >= pdf["this"], psdf["this"] >= psdf["this"])
 
 
-class DatetimeOpsTests(DatetimeOpsTestsMixin, OpsTestBase):
+class DatetimeOpsTests(
+    DatetimeOpsTestsMixin,
+    OpsTestBase,
+    PandasOnSparkTestCase,
+):
     pass
 
 
