@@ -17,22 +17,17 @@
 
 import datetime
 import decimal
-from distutils.version import LooseVersion
 
 import numpy as np
 import pandas as pd
 
 import pyspark.pandas as ps
-from pyspark.pandas.typedef import extension_dtypes
-
 from pyspark.pandas.typedef.typehints import (
     extension_dtypes_available,
     extension_float_dtypes_available,
     extension_object_dtypes_available,
 )
 
-from pyspark.testing.pandasutils import ComparisonTestBase
-
 if extension_dtypes_available:
     from pandas import Int8Dtype, Int16Dtype, Int32Dtype, Int64Dtype
 
@@ -43,7 +38,7 @@ if extension_object_dtypes_available:
     from pandas import BooleanDtype, StringDtype
 
 
-class OpsTestBase(ComparisonTestBase):
+class OpsTestBase:
     """The test base for arithmetic operations of different data types."""
 
     @property
@@ -52,20 +47,13 @@ class OpsTestBase(ComparisonTestBase):
         sers = [pd.Series([1, 2, 3], dtype=dtype) for dtype in dtypes]
         sers.append(pd.Series([decimal.Decimal(1), decimal.Decimal(2), decimal.Decimal(3)]))
         sers.append(pd.Series([1, 2, np.nan], dtype=float))
-        # Skip decimal_nan test before v1.3.0, it not supported by pandas on spark yet.
-        if LooseVersion(pd.__version__) >= LooseVersion("1.3.0"):
-            sers.append(
-                pd.Series([decimal.Decimal(1), decimal.Decimal(2), decimal.Decimal(np.nan)])
-            )
+        sers.append(pd.Series([decimal.Decimal(1), decimal.Decimal(2), decimal.Decimal(np.nan)]))
         pdf = pd.concat(sers, axis=1)
-        if LooseVersion(pd.__version__) >= LooseVersion("1.3.0"):
-            pdf.columns = [dtype.__name__ for dtype in dtypes] + [
-                "decimal",
-                "float_nan",
-                "decimal_nan",
-            ]
-        else:
-            pdf.columns = [dtype.__name__ for dtype in dtypes] + ["decimal", "float_nan"]
+        pdf.columns = [dtype.__name__ for dtype in dtypes] + [
+            "decimal",
+            "float_nan",
+            "decimal_nan",
+        ]
         return pdf
 
     @property
@@ -112,6 +100,10 @@ class OpsTestBase(ComparisonTestBase):
     def pdf(self):
         return pd.concat([self.numeric_pdf, self.non_numeric_pdf], axis=1)
 
+    @property
+    def psdf(self):
+        return ps.from_pandas(self.pdf)
+
     @property
     def df_cols(self):
         return self.pdf.columns
@@ -220,9 +212,4 @@ class OpsTestBase(ComparisonTestBase):
         This utility is to adjust an issue for comparing numeric ExtensionDtypes in specific
         pandas versions. Please refer to https://github.com/pandas-dev/pandas/issues/39410.
         """
-        if LooseVersion("1.1") <= LooseVersion(pd.__version__) < LooseVersion("1.2.2"):
-            self.assert_eq(left, right, check_exact=False)
-            self.assertTrue(isinstance(left.dtype, extension_dtypes))
-            self.assertTrue(isinstance(right.dtype, extension_dtypes))
-        else:
-            self.assert_eq(left, right)
+        self.assert_eq(left, right)
