@@ -28,13 +28,22 @@ from typing import (
 from warnings import warn
 
 from pyspark.errors.exceptions.captured import unwrap_spark_exception
-from pyspark.rdd import _load_from_socket
+from pyspark.loose_version import LooseVersion
+from pyspark.util import _load_from_socket
 from pyspark.sql.pandas.serializers import ArrowCollectSerializer
 from pyspark.sql.pandas.types import _dedup_names
-from pyspark.sql.types import ArrayType, MapType, TimestampType, StructType, DataType, _create_row
+from pyspark.sql.types import (
+    ArrayType,
+    MapType,
+    TimestampType,
+    StructType,
+    DataType,
+    _create_row,
+    StringType,
+)
 from pyspark.sql.utils import is_timestamp_ntz_preferred
 from pyspark.traceback_utils import SCCallSiteSync
-from pyspark.errors import PySparkTypeError
+from pyspark.errors import PySparkTypeError, PySparkValueError
 
 if TYPE_CHECKING:
     import numpy as np
@@ -47,35 +56,11 @@ if TYPE_CHECKING:
 
 class PandasConversionMixin:
     """
-    Mix-in for the conversion from Spark to pandas. Currently, only :class:`DataFrame`
-    can use this class.
+    Mix-in for the conversion from Spark to pandas and PyArrow. Currently, only
+    :class:`DataFrame` can use this class.
     """
 
     def toPandas(self) -> "PandasDataFrameLike":
-        """
-        Returns the contents of this :class:`DataFrame` as Pandas ``pandas.DataFrame``.
-
-        This is only available if Pandas is installed and available.
-
-        .. versionadded:: 1.3.0
-
-        .. versionchanged:: 3.4.0
-            Supports Spark Connect.
-
-        Notes
-        -----
-        This method should only be used if the resulting Pandas ``pandas.DataFrame`` is
-        expected to be small, as all the data is loaded into the driver's memory.
-
-        Usage with ``spark.sql.execution.arrow.pyspark.enabled=True`` is experimental.
-
-        Examples
-        --------
-        >>> df.toPandas()  # doctest: +SKIP
-           age   name
-        0    2  Alice
-        1    5    Bob
-        """
         from pyspark.sql.dataframe import DataFrame
 
         assert isinstance(self, DataFrame)
@@ -96,9 +81,8 @@ class PandasConversionMixin:
                 from pyspark.sql.pandas.utils import require_minimum_pyarrow_version
 
                 require_minimum_pyarrow_version()
-                to_arrow_schema(self.schema)
+                to_arrow_schema(self.schema, prefers_large_types=jconf.arrowUseLargeVarTypes())
             except Exception as e:
-
                 if jconf.arrowPySparkFallbackEnabled():
                     msg = (
                         "toPandas attempted Arrow optimization because "
@@ -125,12 +109,12 @@ class PandasConversionMixin:
             # of PyArrow is found, if 'spark.sql.execution.arrow.pyspark.enabled' is enabled.
             if use_arrow:
                 try:
-                    import pyarrow
+                    import pyarrow as pa
 
                     self_destruct = jconf.arrowPySparkSelfDestructEnabled()
                     batches = self._collect_as_arrow(split_batches=self_destruct)
                     if len(batches) > 0:
-                        table = pyarrow.Table.from_batches(batches)
+                        table = pa.Table.from_batches(batches)
                         # Ensure only the table has a reference to the batches, so that
                         # self_destruct (if enabled) is effective
                         del batches
@@ -138,6 +122,17 @@ class PandasConversionMixin:
                         # values, but we should use datetime.date to match the behavior with when
                         # Arrow optimization is disabled.
                         pandas_options = {"date_as_object": True}
+
+                        if LooseVersion(pa.__version__) >= LooseVersion("13.0.0"):
+                            # A legacy option to coerce date32, date64, duration, and timestamp
+                            # time units to nanoseconds when converting to pandas.
+                            # This option can only be added since 13.0.0.
+                            pandas_options.update(
+                                {
+                                    "coerce_temporal_nanoseconds": True,
+                                }
+                            )
+
                         if self_destruct:
                             # Configure PyArrow to use as little memory as possible:
                             # self_destruct - free columns as they are converted
@@ -230,15 +225,53 @@ class PandasConversionMixin:
         else:
             return pdf
 
-    def _collect_as_arrow(self, split_batches: bool = False) -> List["pa.RecordBatch"]:
+    def toArrow(self) -> "pa.Table":
+        from pyspark.sql.dataframe import DataFrame
+
+        assert isinstance(self, DataFrame)
+
+        jconf = self.sparkSession._jconf
+
+        from pyspark.sql.pandas.types import to_arrow_schema
+        from pyspark.sql.pandas.utils import require_minimum_pyarrow_version
+
+        require_minimum_pyarrow_version()
+        prefers_large_var_types = jconf.arrowUseLargeVarTypes()
+        schema = to_arrow_schema(
+            self.schema,
+            error_on_duplicated_field_names_in_struct=True,
+            prefers_large_types=prefers_large_var_types,
+        )
+
+        import pyarrow as pa
+
+        self_destruct = jconf.arrowPySparkSelfDestructEnabled()
+        batches = self._collect_as_arrow(
+            split_batches=self_destruct, empty_list_if_zero_records=False
+        )
+        table = pa.Table.from_batches(batches).cast(schema)
+        # Ensure only the table has a reference to the batches, so that
+        # self_destruct (if enabled) is effective
+        del batches
+        return table
+
+    def _collect_as_arrow(
+        self,
+        split_batches: bool = False,
+        empty_list_if_zero_records: bool = True,
+    ) -> List["pa.RecordBatch"]:
         """
-        Returns all records as a list of ArrowRecordBatches, pyarrow must be installed
+        Returns all records as a list of Arrow RecordBatches. PyArrow must be installed
         and available on driver and worker Python environments.
         This is an experimental feature.
 
         :param split_batches: split batches such that each column is in its own allocation, so
             that the selfDestruct optimization is effective; default False.
 
+        :param empty_list_if_zero_records: If True (the default), returns an empty list if the
+            result has 0 records. Otherwise, returns a list of length 1 containing an empty
+            Arrow RecordBatch which includes the schema.
+
         .. note:: Experimental.
         """
         from pyspark.sql.dataframe import DataFrame
@@ -287,14 +320,23 @@ class PandasConversionMixin:
         batches = results[:-1]
         batch_order = results[-1]
 
-        # Re-order the batch list using the correct order
-        return [batches[i] for i in batch_order]
+        if len(batches) or empty_list_if_zero_records:
+            # Re-order the batch list using the correct order
+            return [batches[i] for i in batch_order]
+        else:
+            from pyspark.sql.pandas.types import to_arrow_schema
+            import pyarrow as pa
+
+            prefers_large_var_types = self.sparkSession._jconf.arrowUseLargeVarTypes()
+            schema = to_arrow_schema(self.schema, prefers_large_types=prefers_large_var_types)
+            empty_arrays = [pa.array([], type=field.type) for field in schema]
+            return [pa.RecordBatch.from_arrays(empty_arrays, schema=schema)]
 
 
 class SparkConversionMixin:
     """
-    Min-in for the conversion from pandas to Spark. Currently, only :class:`SparkSession`
-    can use this class.
+    Min-in for the conversion from pandas and PyArrow to Spark. Currently, only
+    :class:`SparkSession` can use this class.
     """
 
     _jsparkSession: "JavaObject"
@@ -305,6 +347,12 @@ class SparkConversionMixin:
     ) -> "DataFrame":
         ...
 
+    @overload
+    def createDataFrame(
+        self, data: "pa.Table", samplingRatio: Optional[float] = ...
+    ) -> "DataFrame":
+        ...
+
     @overload
     def createDataFrame(
         self,
@@ -314,9 +362,18 @@ class SparkConversionMixin:
     ) -> "DataFrame":
         ...
 
+    @overload
+    def createDataFrame(
+        self,
+        data: "pa.Table",
+        schema: Union[StructType, str],
+        verifySchema: bool = ...,
+    ) -> "DataFrame":
+        ...
+
     def createDataFrame(  # type: ignore[misc]
         self,
-        data: "PandasDataFrameLike",
+        data: Union["PandasDataFrameLike", "pa.Table"],
         schema: Optional[Union[StructType, List[str]]] = None,
         samplingRatio: Optional[float] = None,
         verifySchema: bool = True,
@@ -325,12 +382,29 @@ class SparkConversionMixin:
 
         assert isinstance(self, SparkSession)
 
+        timezone = self._jconf.sessionLocalTimeZone()
+
+        if type(data).__name__ == "Table":
+            # `data` is a PyArrow Table
+            from pyspark.sql.pandas.utils import require_minimum_pyarrow_version
+
+            require_minimum_pyarrow_version()
+
+            import pyarrow as pa
+
+            assert isinstance(data, pa.Table)
+
+            # If no schema supplied by user then get the names of columns only
+            if schema is None:
+                schema = data.schema.names
+
+            return self._create_from_arrow_table(data, schema, timezone)
+
+        # `data` is a PandasDataFrameLike object
         from pyspark.sql.pandas.utils import require_minimum_pandas_version
 
         require_minimum_pandas_version()
 
-        timezone = self._jconf.sessionLocalTimeZone()
-
         # If no schema supplied by user then get the names of columns only
         if schema is None:
             schema = [str(x) if not isinstance(x, str) else x for x in data.columns]
@@ -373,7 +447,6 @@ class SparkConversionMixin:
         list
             list of records
         """
-        import pandas as pd
         from pyspark.sql import SparkSession
 
         assert isinstance(self, SparkSession)
@@ -383,7 +456,8 @@ class SparkConversionMixin:
                 _check_series_convert_timestamps_tz_local,
                 _get_local_timezone,
             )
-            from pandas.core.dtypes.common import is_datetime64tz_dtype, is_timedelta64_dtype
+            import pandas as pd
+            from pandas.core.dtypes.common import is_timedelta64_dtype
 
             copied = False
             if isinstance(schema, StructType):
@@ -452,7 +526,7 @@ class SparkConversionMixin:
                                 else:
                                     return (
                                         pd.Timestamp(value)
-                                        .tz_localize(timezone, ambiguous=False)  # type: ignore
+                                        .tz_localize(timezone, ambiguous=False)
                                         .tz_convert(_get_local_timezone())
                                         .tz_localize(None)
                                         .to_pydatetime()
@@ -482,7 +556,11 @@ class SparkConversionMixin:
                 should_localize = not is_timestamp_ntz_preferred()
                 for column, series in pdf.items():
                     s = series
-                    if should_localize and is_datetime64tz_dtype(s.dtype) and s.dt.tz is not None:
+                    if (
+                        should_localize
+                        and isinstance(s.dtype, pd.DatetimeTZDtype)
+                        and s.dt.tz is not None
+                    ):
                         s = _check_series_convert_timestamps_tz_local(series, timezone)
                     if s is not series:
                         if not copied:
@@ -579,21 +657,45 @@ class SparkConversionMixin:
         require_minimum_pandas_version()
         require_minimum_pyarrow_version()
 
+        import pandas as pd
         from pandas.api.types import (  # type: ignore[attr-defined]
             is_datetime64_dtype,
-            is_datetime64tz_dtype,
         )
         import pyarrow as pa
 
+        infer_pandas_dict_as_map = (
+            str(self.conf.get("spark.sql.execution.pandas.inferPandasDictAsMap")).lower() == "true"
+        )
+
         # Create the Spark schema from list of names passed in with Arrow types
         if isinstance(schema, (list, tuple)):
             arrow_schema = pa.Schema.from_pandas(pdf, preserve_index=False)
-            struct = StructType()
             prefer_timestamp_ntz = is_timestamp_ntz_preferred()
-            for name, field in zip(schema, arrow_schema):
-                struct.add(
-                    name, from_arrow_type(field.type, prefer_timestamp_ntz), nullable=field.nullable
-                )
+            struct = StructType()
+            if infer_pandas_dict_as_map:
+                spark_type: Union[MapType, DataType]
+                for name, field in zip(schema, arrow_schema):
+                    field_type = field.type
+                    if isinstance(field_type, pa.StructType):
+                        if len(field_type) == 0:
+                            raise PySparkValueError(
+                                errorClass="CANNOT_INFER_EMPTY_SCHEMA",
+                                messageParameters={},
+                            )
+                        arrow_type = field_type.field(0).type
+                        spark_type = MapType(
+                            StringType(), from_arrow_type(arrow_type, prefer_timestamp_ntz)
+                        )
+                    else:
+                        spark_type = from_arrow_type(field_type)
+                    struct.add(name, spark_type, nullable=field.nullable)
+            else:
+                for name, field in zip(schema, arrow_schema):
+                    struct.add(
+                        name,
+                        from_arrow_type(field.type, prefer_timestamp_ntz),
+                        nullable=field.nullable,
+                    )
             schema = struct
 
         # Determine arrow types to coerce data when creating batches
@@ -601,13 +703,15 @@ class SparkConversionMixin:
             spark_types = [_deduplicate_field_names(f.dataType) for f in schema.fields]
         elif isinstance(schema, DataType):
             raise PySparkTypeError(
-                error_class="UNSUPPORTED_DATA_TYPE_FOR_ARROW",
-                message_parameters={"data_type": str(schema)},
+                errorClass="UNSUPPORTED_DATA_TYPE_FOR_ARROW",
+                messageParameters={"data_type": str(schema)},
             )
         else:
             # Any timestamps must be coerced to be compatible with Spark
             spark_types = [
-                TimestampType() if is_datetime64_dtype(t) or is_datetime64tz_dtype(t) else None
+                TimestampType()
+                if is_datetime64_dtype(t) or isinstance(t, pd.DatetimeTZDtype)
+                else None
                 for t in pdf.dtypes
             ]
 
@@ -617,9 +721,16 @@ class SparkConversionMixin:
         pdf_slices = (pdf.iloc[start : start + step] for start in range(0, len(pdf), step))
 
         # Create list of Arrow (columns, arrow_type, spark_type) for serializer dump_stream
+        prefers_large_var_types = self._jconf.arrowUseLargeVarTypes()
         arrow_data = [
             [
-                (c, to_arrow_type(t) if t is not None else None, t)
+                (
+                    c,
+                    to_arrow_type(t, prefers_large_types=prefers_large_var_types)
+                    if t is not None
+                    else None,
+                    t,
+                )
                 for (_, c), t in zip(pdf_slice.items(), spark_types)
             ]
             for pdf_slice in pdf_slices
@@ -646,6 +757,80 @@ class SparkConversionMixin:
         df._schema = schema
         return df
 
+    def _create_from_arrow_table(
+        self, table: "pa.Table", schema: Union[StructType, List[str]], timezone: str
+    ) -> "DataFrame":
+        """
+        Create a DataFrame from a given pyarrow.Table by slicing it into partitions then
+        sending to the JVM to parallelize.
+        """
+        from pyspark.sql import SparkSession
+        from pyspark.sql.dataframe import DataFrame
+
+        assert isinstance(self, SparkSession)
+
+        from pyspark.sql.pandas.serializers import ArrowStreamSerializer
+        from pyspark.sql.pandas.types import (
+            from_arrow_type,
+            from_arrow_schema,
+            to_arrow_schema,
+            _check_arrow_table_timestamps_localize,
+        )
+        from pyspark.sql.pandas.utils import require_minimum_pyarrow_version
+
+        require_minimum_pyarrow_version()
+
+        prefer_timestamp_ntz = is_timestamp_ntz_preferred()
+
+        # Create the Spark schema from list of names passed in with Arrow types
+        if isinstance(schema, (list, tuple)):
+            table = table.rename_columns(schema)
+            arrow_schema = table.schema
+            struct = StructType()
+            for name, field in zip(schema, arrow_schema):
+                struct.add(
+                    name,
+                    from_arrow_type(field.type, prefer_timestamp_ntz),
+                    nullable=field.nullable,
+                )
+            schema = struct
+
+        if not isinstance(schema, StructType):
+            schema = from_arrow_schema(table.schema, prefer_timestamp_ntz=prefer_timestamp_ntz)
+
+        prefers_large_var_types = self._jconf.arrowUseLargeVarTypes()
+        table = _check_arrow_table_timestamps_localize(table, schema, True, timezone).cast(
+            to_arrow_schema(
+                schema,
+                error_on_duplicated_field_names_in_struct=True,
+                prefers_large_types=prefers_large_var_types,
+            )
+        )
+
+        # Chunk the Arrow Table into RecordBatches
+        chunk_size = self._jconf.arrowMaxRecordsPerBatch()
+        arrow_data = table.to_batches(max_chunksize=chunk_size)
+
+        jsparkSession = self._jsparkSession
+
+        ser = ArrowStreamSerializer()
+
+        @no_type_check
+        def reader_func(temp_filename):
+            return self._jvm.PythonSQLUtils.readArrowStreamFromFile(temp_filename)
+
+        @no_type_check
+        def create_iter_server():
+            return self._jvm.ArrowIteratorServer()
+
+        # Create Spark DataFrame from Arrow stream file, using one batch per partition
+        jiter = self._sc._serialize_to_jvm(arrow_data, ser, reader_func, create_iter_server)
+        assert self._jvm is not None
+        jdf = self._jvm.PythonSQLUtils.toDataFrame(jiter, schema.json(), jsparkSession)
+        df = DataFrame(jdf, self)
+        df._schema = schema
+        return df
+
 
 def _test() -> None:
     import doctest
