@@ -23,7 +23,7 @@ import tempfile
 import unittest
 import zipfile
 
-from pyspark.testing.utils import SPARK_HOME
+from pyspark.testing.sqlutils import SPARK_HOME
 
 
 class SparkSubmitTests(unittest.TestCase):
@@ -298,7 +298,7 @@ if __name__ == "__main__":
     from pyspark.tests.test_appsubmit import *  # noqa: F401
 
     try:
-        import xmlrunner  # type: ignore[import]
+        import xmlrunner
 
         testRunner = xmlrunner.XMLTestRunner(output="target/test-reports", verbosity=2)
     except ImportError:
