@@ -60,11 +60,11 @@ LOG_FILE = os.path.join(SPARK_HOME, "python/unit-tests.log")
 FAILURE_REPORTING_LOCK = Lock()
 LOGGER = logging.getLogger()
 
+# Find out where the assembly jars are located.
+# TODO: revisit for Scala 2.13
 SPARK_DIST_CLASSPATH = ""
 if "SPARK_SKIP_CONNECT_COMPAT_TESTS" not in os.environ:
-    # Find out where the assembly jars are located.
-    # TODO: revisit for Scala 2.13
-    for scala in ["2.12", "2.13"]:
+    for scala in ["2.13"]:
         build_dir = os.path.join(SPARK_HOME, "assembly", "target", "scala-" + scala)
         if os.path.isdir(build_dir):
             SPARK_DIST_CLASSPATH = os.path.join(build_dir, "jars", "*")
@@ -117,9 +117,12 @@ def run_individual_python_test(target_dir, test_name, pyspark_python, keep_test_
         metastore_dir = os.path.join(metastore_dir, str(uuid.uuid4()))
     os.mkdir(metastore_dir)
 
-    # Also override the JVM's temp directory by setting driver and executor options.
-    java_options = "-Djava.io.tmpdir={0}".format(tmp_dir)
-    java_options = java_options + " -Dio.netty.tryReflectionSetAccessible=true -Xss4M"
+    # Also override the JVM's temp directory and log4j conf by setting driver and executor options.
+    log4j2_path = os.path.join(SPARK_HOME, "python/test_support/log4j2.properties")
+    java_options = "-Djava.io.tmpdir={0} -Dlog4j.configurationFile={1}".format(
+        tmp_dir, log4j2_path
+    )
+    java_options = java_options + " -Xss4M"
     spark_args = [
         "--conf", "spark.driver.extraJavaOptions='{0}'".format(java_options),
         "--conf", "spark.executor.extraJavaOptions='{0}'".format(java_options),
