@@ -22,11 +22,14 @@ import java.time.ZoneOffset
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.expressions.codegen._
 import org.apache.spark.sql.catalyst.expressions.codegen.Block._
-import org.apache.spark.sql.catalyst.util.{ArrayData, DateFormatter, IntervalStringStyles, IntervalUtils, MapData, SparkStringUtils, TimestampFormatter}
+import org.apache.spark.sql.catalyst.util.{ArrayData, CharVarcharCodegenUtils, DateFormatter, IntervalStringStyles, IntervalUtils, MapData, SparkStringUtils, TimestampFormatter}
 import org.apache.spark.sql.catalyst.util.IntervalStringStyles.ANSI_STYLE
+import org.apache.spark.sql.internal.SQLConf
+import org.apache.spark.sql.internal.SQLConf.BinaryOutputStyle
 import org.apache.spark.sql.types._
 import org.apache.spark.unsafe.UTF8StringBuilder
 import org.apache.spark.unsafe.types.{CalendarInterval, UTF8String}
+import org.apache.spark.util.ArrayImplicits._
 
 trait ToStringBase { self: UnaryExpression with TimeZoneAwareExpression =>
 
@@ -43,19 +46,27 @@ trait ToStringBase { self: UnaryExpression with TimeZoneAwareExpression =>
 
   protected def useDecimalPlainString: Boolean
 
-  protected def useHexFormatForBinary: Boolean
+  protected val binaryFormatter: BinaryFormatter = UTF8String.fromBytes
 
   // Makes the function accept Any type input by doing `asInstanceOf[T]`.
-  @inline private def acceptAny[T](func: T => Any): Any => Any = i => func(i.asInstanceOf[T])
+  @inline private def acceptAny[T](func: T => UTF8String): Any => UTF8String =
+    i => func(i.asInstanceOf[T])
 
   // Returns a function to convert a value to pretty string. The function assumes input is not null.
-  protected final def castToString(from: DataType): Any => Any = from match {
+  protected final def castToString(
+      from: DataType, to: StringConstraint = NoConstraint): Any => UTF8String =
+    to match {
+      case FixedLength(length) =>
+        s => CharVarcharCodegenUtils.charTypeWriteSideCheck(castToString(from)(s), length)
+      case MaxLength(length) =>
+        s => CharVarcharCodegenUtils.varcharTypeWriteSideCheck(castToString(from)(s), length)
+      case NoConstraint => castToString(from)
+    }
+
+  private def castToString(from: DataType): Any => UTF8String = from match {
     case CalendarIntervalType =>
       acceptAny[CalendarInterval](i => UTF8String.fromString(i.toString))
-    case BinaryType if useHexFormatForBinary =>
-      acceptAny[Array[Byte]](binary => UTF8String.fromString(SparkStringUtils.getHexString(binary)))
-    case BinaryType =>
-      acceptAny[Array[Byte]](UTF8String.fromBytes)
+    case BinaryType => acceptAny[Array[Byte]](binaryFormatter.apply)
     case DateType =>
       acceptAny[Int](d => UTF8String.fromString(dateFormatter.format(d)))
     case TimestampType =>
@@ -66,21 +77,21 @@ trait ToStringBase { self: UnaryExpression with TimeZoneAwareExpression =>
       acceptAny[ArrayData](array => {
         val builder = new UTF8StringBuilder
         builder.append("[")
-        if (array.numElements > 0) {
+        if (array.numElements() > 0) {
           val toUTF8String = castToString(et)
           if (array.isNullAt(0)) {
             if (nullString.nonEmpty) builder.append(nullString)
           } else {
-            builder.append(toUTF8String(array.get(0, et)).asInstanceOf[UTF8String])
+            builder.append(toUTF8String(array.get(0, et)))
           }
           var i = 1
-          while (i < array.numElements) {
+          while (i < array.numElements()) {
             builder.append(",")
             if (array.isNullAt(i)) {
               if (nullString.nonEmpty) builder.append(" " + nullString)
             } else {
               builder.append(" ")
-              builder.append(toUTF8String(array.get(i, et)).asInstanceOf[UTF8String])
+              builder.append(toUTF8String(array.get(i, et)))
             }
             i += 1
           }
@@ -92,30 +103,29 @@ trait ToStringBase { self: UnaryExpression with TimeZoneAwareExpression =>
       acceptAny[MapData](map => {
         val builder = new UTF8StringBuilder
         builder.append(leftBracket)
-        if (map.numElements > 0) {
+        if (map.numElements() > 0) {
           val keyArray = map.keyArray()
           val valueArray = map.valueArray()
           val keyToUTF8String = castToString(kt)
           val valueToUTF8String = castToString(vt)
-          builder.append(keyToUTF8String(keyArray.get(0, kt)).asInstanceOf[UTF8String])
+          builder.append(keyToUTF8String(keyArray.get(0, kt)))
           builder.append(" ->")
           if (valueArray.isNullAt(0)) {
             if (nullString.nonEmpty) builder.append(" " + nullString)
           } else {
             builder.append(" ")
-            builder.append(valueToUTF8String(valueArray.get(0, vt)).asInstanceOf[UTF8String])
+            builder.append(valueToUTF8String(valueArray.get(0, vt)))
           }
           var i = 1
-          while (i < map.numElements) {
+          while (i < map.numElements()) {
             builder.append(", ")
-            builder.append(keyToUTF8String(keyArray.get(i, kt)).asInstanceOf[UTF8String])
+            builder.append(keyToUTF8String(keyArray.get(i, kt)))
             builder.append(" ->")
             if (valueArray.isNullAt(i)) {
               if (nullString.nonEmpty) builder.append(" " + nullString)
             } else {
               builder.append(" ")
-              builder.append(valueToUTF8String(valueArray.get(i, vt))
-                .asInstanceOf[UTF8String])
+              builder.append(valueToUTF8String(valueArray.get(i, vt)))
             }
             i += 1
           }
@@ -133,7 +143,7 @@ trait ToStringBase { self: UnaryExpression with TimeZoneAwareExpression =>
           if (row.isNullAt(0)) {
             if (nullString.nonEmpty) builder.append(nullString)
           } else {
-            builder.append(toUTF8StringFuncs(0)(row.get(0, st(0))).asInstanceOf[UTF8String])
+            builder.append(toUTF8StringFuncs(0)(row.get(0, st(0))))
           }
           var i = 1
           while (i < row.numFields) {
@@ -142,7 +152,7 @@ trait ToStringBase { self: UnaryExpression with TimeZoneAwareExpression =>
               if (nullString.nonEmpty) builder.append(" " + nullString)
             } else {
               builder.append(" ")
-              builder.append(toUTF8StringFuncs(i)(row.get(i, st(i))).asInstanceOf[UTF8String])
+              builder.append(toUTF8StringFuncs(i)(row.get(i, st(i))))
             }
             i += 1
           }
@@ -161,22 +171,44 @@ trait ToStringBase { self: UnaryExpression with TimeZoneAwareExpression =>
         IntervalUtils.toDayTimeIntervalString(i, ANSI_STYLE, startField, endField)))
     case _: DecimalType if useDecimalPlainString =>
       acceptAny[Decimal](d => UTF8String.fromString(d.toPlainString))
-    case StringType => identity
+    case _: StringType => acceptAny[UTF8String](identity[UTF8String])
     case _ => o => UTF8String.fromString(o.toString)
   }
 
   // Returns a function to generate code to convert a value to pretty string. It assumes the input
   // is not null.
-  @scala.annotation.tailrec
   protected final def castToStringCode(
+      from: DataType,
+      ctx: CodegenContext,
+      to: StringConstraint = NoConstraint): (ExprValue, ExprValue) => Block =
+    (c, evPrim) => {
+      val tmpVar = ctx.freshVariable("tmp", classOf[UTF8String])
+      val castToString = castToStringCode(from, ctx)(c, tmpVar)
+      val maintainConstraint = to match {
+        case FixedLength(length) =>
+          code"""$evPrim = org.apache.spark.sql.catalyst.util.CharVarcharCodegenUtils
+                .charTypeWriteSideCheck($tmpVar, $length);""".stripMargin
+        case MaxLength(length) =>
+          code"""$evPrim = org.apache.spark.sql.catalyst.util.CharVarcharCodegenUtils
+                .varcharTypeWriteSideCheck($tmpVar, $length);""".stripMargin
+        case NoConstraint => code"$evPrim = $tmpVar;"
+      }
+      code"""
+            UTF8String $tmpVar;
+            $castToString
+            $maintainConstraint
+          """
+    }
+
+  @scala.annotation.tailrec
+  private def castToStringCode(
       from: DataType, ctx: CodegenContext): (ExprValue, ExprValue) => Block = {
     from match {
-      case BinaryType if useHexFormatForBinary =>
-        (c, evPrim) =>
-          val utilCls = SparkStringUtils.getClass.getName.stripSuffix("$")
-          code"$evPrim = UTF8String.fromString($utilCls.getHexString($c));"
       case BinaryType =>
-        (c, evPrim) => code"$evPrim = UTF8String.fromBytes($c);"
+        val bf = JavaCode.global(
+          ctx.addReferenceObj("binaryFormatter", binaryFormatter),
+          classOf[BinaryFormatter])
+        (c, evPrim) => code"$evPrim = $bf.apply($c);"
       case DateType =>
         val df = JavaCode.global(
           ctx.addReferenceObj("dateFormatter", dateFormatter),
@@ -221,7 +253,8 @@ trait ToStringBase { self: UnaryExpression with TimeZoneAwareExpression =>
           val row = ctx.freshVariable("row", classOf[InternalRow])
           val buffer = ctx.freshVariable("buffer", classOf[UTF8StringBuilder])
           val bufferClass = JavaCode.javaType(classOf[UTF8StringBuilder])
-          val writeStructCode = writeStructToStringBuilder(fields.map(_.dataType), row, buffer, ctx)
+          val writeStructCode =
+            writeStructToStringBuilder(fields.map(_.dataType).toImmutableArraySeq, row, buffer, ctx)
           code"""
              |InternalRow $row = $c;
              |$bufferClass $buffer = new $bufferClass();
@@ -255,7 +288,7 @@ trait ToStringBase { self: UnaryExpression with TimeZoneAwareExpression =>
       // notation if an exponent is needed.
       case _: DecimalType if useDecimalPlainString =>
         (c, evPrim) => code"$evPrim = UTF8String.fromString($c.toPlainString());"
-      case StringType =>
+      case _: StringType =>
         (c, evPrim) => code"$evPrim = $c;"
       case _ =>
         (c, evPrim) => code"$evPrim = UTF8String.fromString(String.valueOf($c));"
@@ -412,3 +445,25 @@ trait ToStringBase { self: UnaryExpression with TimeZoneAwareExpression =>
      """.stripMargin
   }
 }
+
+object ToStringBase {
+  def getBinaryFormatter: BinaryFormatter = {
+    val style = SQLConf.get.getConf(SQLConf.BINARY_OUTPUT_STYLE)
+    style.map(BinaryOutputStyle.withName) match {
+      case Some(BinaryOutputStyle.UTF8) =>
+        (array: Array[Byte]) => UTF8String.fromBytes(array)
+      case Some(BinaryOutputStyle.BASIC) =>
+        (array: Array[Byte]) => UTF8String.fromString(array.mkString("[", ", ", "]"))
+      case Some(BinaryOutputStyle.BASE64) =>
+        (array: Array[Byte]) =>
+          UTF8String.fromString(java.util.Base64.getEncoder.withoutPadding().encodeToString(array))
+      case Some(BinaryOutputStyle.HEX) =>
+        (array: Array[Byte]) => Hex.hex(array)
+      case _ =>
+        (array: Array[Byte]) => UTF8String.fromString(SparkStringUtils.getHexString(array))
+    }
+  }
+}
+
+trait BinaryFormatter extends (Array[Byte] => UTF8String) with Serializable
+
