@@ -17,6 +17,9 @@
 
 package org.apache.spark.sql.catalyst.optimizer
 
+import org.apache.spark.SparkIllegalArgumentException
+import org.apache.spark.internal.LogKeys.{SQL_TEXT, UNSUPPORTED_EXPR}
+import org.apache.spark.internal.MDC
 import org.apache.spark.sql.catalyst.expressions.{And, ArrayExists, ArrayFilter, CaseWhen, EqualNullSafe, Expression, If, In, InSet, LambdaFunction, Literal, MapFilter, Not, Or}
 import org.apache.spark.sql.catalyst.expressions.Literal.{FalseLiteral, TrueLiteral}
 import org.apache.spark.sql.catalyst.plans.logical.{DeleteAction, DeleteFromTable, Filter, InsertAction, InsertStarAction, Join, LogicalPlan, MergeAction, MergeIntoTable, ReplaceData, UpdateAction, UpdateStarAction, UpdateTable, WriteDelta}
@@ -54,7 +57,7 @@ object ReplaceNullWithFalseInPredicate extends Rule[LogicalPlan] {
     _.containsAnyPattern(NULL_LITERAL, TRUE_OR_FALSE_LITERAL, INSET), ruleId) {
     case f @ Filter(cond, _) => f.copy(condition = replaceNullWithFalse(cond))
     case j @ Join(_, _, _, Some(cond), _) => j.copy(condition = Some(replaceNullWithFalse(cond)))
-    case rd @ ReplaceData(_, cond, _, _, groupFilterCond, _) =>
+    case rd @ ReplaceData(_, cond, _, _, _, groupFilterCond, _) =>
       val newCond = replaceNullWithFalse(cond)
       val newGroupFilterCond = groupFilterCond.map(replaceNullWithFalse)
       rd.copy(condition = newCond, groupFilterCondition = newGroupFilterCond)
@@ -130,11 +133,16 @@ object ReplaceNullWithFalseInPredicate extends Rule[LogicalPlan] {
     case e if e.dataType == BooleanType =>
       e
     case e =>
-      val message = "Expected a Boolean type expression in replaceNullWithFalse, " +
-        s"but got the type `${e.dataType.catalogString}` in `${e.sql}`."
       if (Utils.isTesting) {
-        throw new IllegalArgumentException(message)
+        throw new SparkIllegalArgumentException(
+          errorClass = "_LEGACY_ERROR_TEMP_3215",
+          messageParameters = Map(
+            "dataType" -> e.dataType.catalogString,
+            "expr" -> e.sql))
       } else {
+        val message = log"Expected a Boolean type expression in replaceNullWithFalse, " +
+          log"but got the type `${MDC(UNSUPPORTED_EXPR, e.dataType.catalogString)}` " +
+          log"in `${MDC(SQL_TEXT, e.sql)}`."
         logWarning(message)
         e
       }
