@@ -21,8 +21,7 @@ import scala.collection.mutable
 
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.catalyst.SQLConfHelper
-import org.apache.spark.sql.catalyst.analysis.NoSuchDatabaseException
-import org.apache.spark.sql.catalyst.catalog.SessionCatalog
+import org.apache.spark.sql.catalyst.catalog.{SessionCatalog, TempVariableManager}
 import org.apache.spark.sql.catalyst.util.StringUtils
 import org.apache.spark.sql.errors.QueryCompilationErrors
 import org.apache.spark.sql.internal.SQLConf
@@ -47,6 +46,9 @@ class CatalogManager(
 
   private val catalogs = mutable.HashMap.empty[String, CatalogPlugin]
 
+  // TODO: create a real SYSTEM catalog to host `TempVariableManager` under the SESSION namespace.
+  val tempVariableManager: TempVariableManager = new TempVariableManager
+
   def catalog(name: String): CatalogPlugin = synchronized {
     if (name.equalsIgnoreCase(SESSION_CATALOG_NAME)) {
       v2SessionCatalog
@@ -83,9 +85,10 @@ class CatalogManager(
    * in the fallback configuration, spark.sql.sources.useV1SourceList
    */
   private[sql] def v2SessionCatalog: CatalogPlugin = {
-    conf.getConf(SQLConf.V2_SESSION_CATALOG_IMPLEMENTATION).map { _ =>
-      catalogs.getOrElseUpdate(SESSION_CATALOG_NAME, loadV2SessionCatalog())
-    }.getOrElse(defaultSessionCatalog)
+    conf.getConf(SQLConf.V2_SESSION_CATALOG_IMPLEMENTATION) match {
+      case "builtin" => defaultSessionCatalog
+      case _ => catalogs.getOrElseUpdate(SESSION_CATALOG_NAME, loadV2SessionCatalog())
+    }
   }
 
   private var _currentNamespace: Option[Array[String]] = None
@@ -116,13 +119,7 @@ class CatalogManager(
     if (isSessionCatalog(currentCatalog) && namespace.length == 1) {
       v1SessionCatalog.setCurrentDatabaseWithNameCheck(
         namespace.head,
-        name => {
-          currentCatalog match {
-            case catalog: SupportsNamespaces if !catalog.namespaceExists(namespace) =>
-              throw new NoSuchDatabaseException(name)
-            case _ =>
-          }
-        })
+        _ => assertNamespaceExist(namespace))
     } else {
       assertNamespaceExist(namespace)
     }
@@ -163,4 +160,6 @@ class CatalogManager(
 
 private[sql] object CatalogManager {
   val SESSION_CATALOG_NAME: String = "spark_catalog"
+  val SYSTEM_CATALOG_NAME = "system"
+  val SESSION_NAMESPACE = "session"
 }
