@@ -137,14 +137,14 @@ class KeyValueGroupedDataset[K, V] private[sql] () extends sql.KeyValueGroupedDa
   }
 
   /** @inheritdoc */
-  private[sql] def transformWithState[U: Encoder](
+  def transformWithState[U: Encoder](
       statefulProcessor: StatefulProcessor[K, V, U],
       timeMode: TimeMode,
       outputMode: OutputMode): Dataset[U] =
     unsupported()
 
   /** @inheritdoc */
-  private[sql] def transformWithState[U: Encoder, S: Encoder](
+  def transformWithState[U: Encoder, S: Encoder](
       statefulProcessor: StatefulProcessorWithInitialState[K, V, U, S],
       timeMode: TimeMode,
       outputMode: OutputMode,
@@ -152,13 +152,13 @@ class KeyValueGroupedDataset[K, V] private[sql] () extends sql.KeyValueGroupedDa
     unsupported()
 
   /** @inheritdoc */
-  override private[sql] def transformWithState[U: Encoder](
+  override def transformWithState[U: Encoder](
       statefulProcessor: StatefulProcessor[K, V, U],
       eventTimeColumnName: String,
       outputMode: OutputMode): Dataset[U] = unsupported()
 
   /** @inheritdoc */
-  override private[sql] def transformWithState[U: Encoder, S: Encoder](
+  override def transformWithState[U: Encoder, S: Encoder](
       statefulProcessor: StatefulProcessorWithInitialState[K, V, U, S],
       eventTimeColumnName: String,
       outputMode: OutputMode,
@@ -241,29 +241,29 @@ class KeyValueGroupedDataset[K, V] private[sql] () extends sql.KeyValueGroupedDa
     initialState)
 
   /** @inheritdoc */
-  override private[sql] def transformWithState[U: Encoder](
+  override def transformWithState[U: Encoder](
       statefulProcessor: StatefulProcessor[K, V, U],
       timeMode: TimeMode,
       outputMode: OutputMode,
-      outputEncoder: Encoder[U]) =
+      outputEncoder: Encoder[U]): Dataset[U] =
     super.transformWithState(statefulProcessor, timeMode, outputMode, outputEncoder)
 
   /** @inheritdoc */
-  override private[sql] def transformWithState[U: Encoder](
+  override def transformWithState[U: Encoder](
       statefulProcessor: StatefulProcessor[K, V, U],
       eventTimeColumnName: String,
       outputMode: OutputMode,
-      outputEncoder: Encoder[U]) =
+      outputEncoder: Encoder[U]): Dataset[U] =
     super.transformWithState(statefulProcessor, eventTimeColumnName, outputMode, outputEncoder)
 
   /** @inheritdoc */
-  override private[sql] def transformWithState[U: Encoder, S: Encoder](
+  override def transformWithState[U: Encoder, S: Encoder](
       statefulProcessor: StatefulProcessorWithInitialState[K, V, U, S],
       timeMode: TimeMode,
       outputMode: OutputMode,
       initialState: sql.KeyValueGroupedDataset[K, S],
       outputEncoder: Encoder[U],
-      initialStateEncoder: Encoder[S]) = super.transformWithState(
+      initialStateEncoder: Encoder[S]): Dataset[U] = super.transformWithState(
     statefulProcessor,
     timeMode,
     outputMode,
@@ -272,13 +272,13 @@ class KeyValueGroupedDataset[K, V] private[sql] () extends sql.KeyValueGroupedDa
     initialStateEncoder)
 
   /** @inheritdoc */
-  override private[sql] def transformWithState[U: Encoder, S: Encoder](
+  override def transformWithState[U: Encoder, S: Encoder](
       statefulProcessor: StatefulProcessorWithInitialState[K, V, U, S],
       outputMode: OutputMode,
       initialState: sql.KeyValueGroupedDataset[K, S],
       eventTimeColumnName: String,
       outputEncoder: Encoder[U],
-      initialStateEncoder: Encoder[S]) = super.transformWithState(
+      initialStateEncoder: Encoder[S]): Dataset[U] = super.transformWithState(
     statefulProcessor,
     outputMode,
     initialState,
