@@ -29,6 +29,7 @@ import org.apache.parquet.column.values.bitpacking.Packer;
 import org.apache.parquet.io.ParquetDecodingException;
 import org.apache.parquet.io.api.Binary;
 
+import org.apache.spark.SparkUnsupportedOperationException;
 import org.apache.spark.sql.execution.vectorized.WritableColumnVector;
 
 /**
@@ -147,13 +148,10 @@ public final class VectorizedRleValuesReader extends ValuesReader
     if (this.currentCount == 0) { this.readNextGroup(); }
 
     this.currentCount--;
-    switch (mode) {
-      case RLE:
-        return this.currentValue;
-      case PACKED:
-        return this.currentBuffer[currentBufferIdx++];
-    }
-    throw new RuntimeException("Unreachable");
+    return switch (mode) {
+      case RLE -> this.currentValue;
+      case PACKED -> this.currentBuffer[currentBufferIdx++];
+    };
   }
 
   /**
@@ -239,15 +237,15 @@ public final class VectorizedRleValuesReader extends ValuesReader
         n = (int) (end - start + 1);
 
         switch (mode) {
-          case RLE:
+          case RLE -> {
             if (currentValue == state.maxDefinitionLevel) {
               updater.readValues(n, state.valueOffset, values, valueReader);
             } else {
               nulls.putNulls(state.valueOffset, n);
             }
             state.valueOffset += n;
-            break;
-          case PACKED:
+          }
+          case PACKED -> {
             for (int i = 0; i < n; ++i) {
               int currentValue = currentBuffer[currentBufferIdx++];
               if (currentValue == state.maxDefinitionLevel) {
@@ -256,7 +254,7 @@ public final class VectorizedRleValuesReader extends ValuesReader
                 nulls.putNull(state.valueOffset++);
               }
             }
-            break;
+          }
         }
         state.levelOffset += n;
         leftInBatch -= n;
@@ -407,7 +405,7 @@ public final class VectorizedRleValuesReader extends ValuesReader
       long rangeEnd = state.currentRangeEnd();
 
       switch (mode) {
-        case RLE:
+        case RLE -> {
           // This RLE block is consist of top-level rows, so we'll need to check
           // if the rows should be skipped according to row indexes.
           if (currentValue == 0) {
@@ -465,8 +463,8 @@ public final class VectorizedRleValuesReader extends ValuesReader
             leftInPage -= valuesLeftInBlock;
             currentCount -= valuesLeftInBlock;
           }
-          break;
-        case PACKED:
+        }
+        case PACKED -> {
           int i = 0;
 
           for (; i < valuesLeftInBlock; i++) {
@@ -501,7 +499,7 @@ public final class VectorizedRleValuesReader extends ValuesReader
           leftInPage -= i;
           currentCount -= i;
           currentBufferIdx += i;
-          break;
+        }
       }
     }
 
@@ -629,7 +627,7 @@ public final class VectorizedRleValuesReader extends ValuesReader
       VectorizedValuesReader valueReader,
       ParquetVectorUpdater updater) {
     switch (mode) {
-      case RLE:
+      case RLE -> {
         if (currentValue == state.maxDefinitionLevel) {
           updater.readValues(n, state.valueOffset, values, valueReader);
         } else {
@@ -637,8 +635,8 @@ public final class VectorizedRleValuesReader extends ValuesReader
         }
         state.valueOffset += n;
         defLevels.putInts(state.levelOffset, n, currentValue);
-        break;
-      case PACKED:
+      }
+      case PACKED -> {
         for (int i = 0; i < n; ++i) {
           int currentValue = currentBuffer[currentBufferIdx++];
           if (currentValue == state.maxDefinitionLevel) {
@@ -648,7 +646,7 @@ public final class VectorizedRleValuesReader extends ValuesReader
           }
           defLevels.putInt(state.levelOffset + i, currentValue);
         }
-        break;
+      }
     }
   }
 
@@ -665,14 +663,14 @@ public final class VectorizedRleValuesReader extends ValuesReader
       if (currentCount == 0 && !readNextGroup()) break;
       int num = Math.min(n, this.currentCount);
       switch (mode) {
-        case RLE:
+        case RLE -> {
           // We only need to skip non-null values from `valuesReader` since nulls are represented
           // via definition levels which are skipped here via decrementing `currentCount`.
           if (currentValue == state.maxDefinitionLevel) {
             updater.skipValues(num, valuesReader);
           }
-          break;
-        case PACKED:
+        }
+        case PACKED -> {
           int totalSkipNum = 0;
           for (int i = 0; i < num; ++i) {
             // Same as above, only skip non-null values from `valuesReader`
@@ -681,7 +679,7 @@ public final class VectorizedRleValuesReader extends ValuesReader
             }
           }
           updater.skipValues(totalSkipNum, valuesReader);
-          break;
+        }
       }
       currentCount -= num;
       n -= num;
@@ -698,13 +696,11 @@ public final class VectorizedRleValuesReader extends ValuesReader
       if (currentCount == 0 && !readNextGroup()) break;
       int n = Math.min(left, this.currentCount);
       switch (mode) {
-        case RLE:
-          c.putInts(rowId, n, currentValue);
-          break;
-        case PACKED:
+        case RLE -> c.putInts(rowId, n, currentValue);
+        case PACKED -> {
           c.putInts(rowId, n, currentBuffer, currentBufferIdx);
           currentBufferIdx += n;
-          break;
+        }
       }
       rowId += n;
       left -= n;
@@ -714,43 +710,43 @@ public final class VectorizedRleValuesReader extends ValuesReader
 
   @Override
   public void readUnsignedIntegers(int total, WritableColumnVector c, int rowId) {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
   public void readUnsignedLongs(int total, WritableColumnVector c, int rowId) {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
   public void readIntegersWithRebase(
       int total, WritableColumnVector c, int rowId, boolean failIfRebase) {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
   public byte readByte() {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
   public short readShort() {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
   public void readBytes(int total, WritableColumnVector c, int rowId) {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
   public void readShorts(int total, WritableColumnVector c, int rowId) {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
   public void readLongs(int total, WritableColumnVector c, int rowId) {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
@@ -760,12 +756,12 @@ public final class VectorizedRleValuesReader extends ValuesReader
       int rowId,
       boolean failIfRebase,
       String timeZone) {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
   public void readBinary(int total, WritableColumnVector c, int rowId) {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
@@ -775,15 +771,13 @@ public final class VectorizedRleValuesReader extends ValuesReader
       if (this.currentCount == 0) this.readNextGroup();
       int n = Math.min(left, this.currentCount);
       switch (mode) {
-        case RLE:
-          c.putBooleans(rowId, n, currentValue != 0);
-          break;
-        case PACKED:
+        case RLE -> c.putBooleans(rowId, n, currentValue != 0);
+        case PACKED -> {
           for (int i = 0; i < n; ++i) {
             // For Boolean types, `currentBuffer[currentBufferIdx++]` can only be 0 or 1
             c.putByte(rowId + i, (byte) currentBuffer[currentBufferIdx++]);
           }
-          break;
+        }
       }
       rowId += n;
       left -= n;
@@ -793,17 +787,17 @@ public final class VectorizedRleValuesReader extends ValuesReader
 
   @Override
   public void readFloats(int total, WritableColumnVector c, int rowId) {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
   public void readDoubles(int total, WritableColumnVector c, int rowId) {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
   public Binary readBinary(int len) {
-    throw new UnsupportedOperationException("only readInts is valid.");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3187");
   }
 
   @Override
@@ -818,37 +812,37 @@ public final class VectorizedRleValuesReader extends ValuesReader
 
   @Override
   public void skipBytes(int total) {
-    throw new UnsupportedOperationException("only skipIntegers is valid");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3188");
   }
 
   @Override
   public void skipShorts(int total) {
-    throw new UnsupportedOperationException("only skipIntegers is valid");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3188");
   }
 
   @Override
   public void skipLongs(int total) {
-    throw new UnsupportedOperationException("only skipIntegers is valid");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3188");
   }
 
   @Override
   public void skipFloats(int total) {
-    throw new UnsupportedOperationException("only skipIntegers is valid");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3188");
   }
 
   @Override
   public void skipDoubles(int total) {
-    throw new UnsupportedOperationException("only skipIntegers is valid");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3188");
   }
 
   @Override
   public void skipBinary(int total) {
-    throw new UnsupportedOperationException("only skipIntegers is valid");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3188");
   }
 
   @Override
   public void skipFixedLenByteArray(int total, int len) {
-    throw new UnsupportedOperationException("only skipIntegers is valid");
+    throw new SparkUnsupportedOperationException("_LEGACY_ERROR_TEMP_3188");
   }
 
   /**
@@ -881,27 +875,23 @@ public final class VectorizedRleValuesReader extends ValuesReader
    * Reads the next byteWidth little endian int.
    */
   private int readIntLittleEndianPaddedOnBitWidth() throws IOException {
-    switch (bytesWidth) {
-      case 0:
-        return 0;
-      case 1:
-        return in.read();
-      case 2: {
+    return switch (bytesWidth) {
+      case 0 -> 0;
+      case 1 -> in.read();
+      case 2 -> {
         int ch2 = in.read();
         int ch1 = in.read();
-        return (ch1 << 8) + ch2;
+        yield (ch1 << 8) + ch2;
       }
-      case 3: {
+      case 3 -> {
         int ch3 = in.read();
         int ch2 = in.read();
         int ch1 = in.read();
-        return (ch1 << 16) + (ch2 << 8) + (ch3 << 0);
+        yield (ch1 << 16) + (ch2 << 8) + (ch3 << 0);
       }
-      case 4: {
-        return readIntLittleEndian();
-      }
-    }
-    throw new RuntimeException("Unreachable");
+      case 4 -> readIntLittleEndian();
+      default -> throw new RuntimeException("Unreachable");
+    };
   }
 
   /**
@@ -917,11 +907,11 @@ public final class VectorizedRleValuesReader extends ValuesReader
       int header = readUnsignedVarInt();
       this.mode = (header & 1) == 0 ? MODE.RLE : MODE.PACKED;
       switch (mode) {
-        case RLE:
+        case RLE -> {
           this.currentCount = header >>> 1;
           this.currentValue = readIntLittleEndianPaddedOnBitWidth();
-          break;
-        case PACKED:
+        }
+        case PACKED -> {
           int numGroups = header >>> 1;
           this.currentCount = numGroups * 8;
 
@@ -936,9 +926,7 @@ public final class VectorizedRleValuesReader extends ValuesReader
             this.packer.unpack8Values(buffer, buffer.position(), this.currentBuffer, valueIndex);
             valueIndex += 8;
           }
-          break;
-        default:
-          throw new ParquetDecodingException("not a valid mode " + this.mode);
+        }
       }
     } catch (IOException e) {
       throw new ParquetDecodingException("Failed to read from input stream", e);
@@ -956,11 +944,8 @@ public final class VectorizedRleValuesReader extends ValuesReader
       if (this.currentCount == 0 && !readNextGroup()) break;
       int num = Math.min(left, this.currentCount);
       switch (mode) {
-        case RLE:
-          break;
-        case PACKED:
-          currentBufferIdx += num;
-          break;
+        case RLE -> {}
+        case PACKED -> currentBufferIdx += num;
       }
       currentCount -= num;
       left -= num;
