@@ -20,7 +20,7 @@ package org.apache.spark.sql.execution.datasources.parquet
 import java.nio.{ByteBuffer, ByteOrder}
 import java.util
 
-import scala.collection.JavaConverters.mapAsJavaMapConverter
+import scala.jdk.CollectionConverters.MapHasAsJava
 
 import org.apache.hadoop.conf.Configuration
 import org.apache.parquet.column.ParquetProperties
@@ -29,7 +29,7 @@ import org.apache.parquet.hadoop.api.WriteSupport
 import org.apache.parquet.hadoop.api.WriteSupport.WriteContext
 import org.apache.parquet.io.api.{Binary, RecordConsumer}
 
-import org.apache.spark.SPARK_VERSION_SHORT
+import org.apache.spark.{SPARK_VERSION_SHORT, SparkException}
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.{SPARK_LEGACY_DATETIME_METADATA_KEY, SPARK_LEGACY_INT96_METADATA_KEY, SPARK_TIMEZONE_METADATA_KEY, SPARK_VERSION_METADATA_KEY}
 import org.apache.spark.sql.catalyst.InternalRow
@@ -38,6 +38,7 @@ import org.apache.spark.sql.catalyst.util.DateTimeUtils
 import org.apache.spark.sql.execution.datasources.DataSourceUtils
 import org.apache.spark.sql.internal.{LegacyBehaviorPolicy, SQLConf}
 import org.apache.spark.sql.types._
+import org.apache.spark.types.variant.Variant
 
 /**
  * A Parquet [[WriteSupport]] implementation that writes Catalyst [[InternalRow]]s as Parquet
@@ -59,6 +60,10 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
   // Schema of the `InternalRow`s to be written
   private var schema: StructType = _
 
+  // Schema of the `InternalRow`s to be written, with VariantType replaced with its shredding
+  // schema, if appropriate.
+  private var shreddedSchema: StructType = _
+
   // `ValueWriter`s for all fields of the schema
   private var rootFieldWriters: Array[ValueWriter] = _
 
@@ -95,7 +100,16 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
 
   override def init(configuration: Configuration): WriteContext = {
     val schemaString = configuration.get(ParquetWriteSupport.SPARK_ROW_SCHEMA)
+    val shreddedSchemaString = configuration.get(ParquetWriteSupport.SPARK_VARIANT_SHREDDING_SCHEMA)
     this.schema = StructType.fromString(schemaString)
+    // If shreddingSchemaString is provided, we use that everywhere in the writer, except for
+    // setting the spark schema in the Parquet metadata. If it isn't provided, it means that there
+    // are no shredded Variant columns, so it is identical to this.schema.
+    this.shreddedSchema = if (shreddedSchemaString == null) {
+      this.schema
+    } else {
+      StructType.fromString(shreddedSchemaString)
+    }
     this.writeLegacyParquetFormat = {
       // `SQLConf.PARQUET_WRITE_LEGACY_FORMAT` should always be explicitly set in ParquetRelation
       assert(configuration.get(SQLConf.PARQUET_WRITE_LEGACY_FORMAT.key) != null)
@@ -108,9 +122,10 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
       SQLConf.ParquetOutputTimestampType.withName(configuration.get(key))
     }
 
-    this.rootFieldWriters = schema.map(_.dataType).map(makeWriter).toArray[ValueWriter]
+    this.rootFieldWriters =
+      shreddedSchema.map(_.dataType).map(makeWriter(_, inShredded = false)).toArray[ValueWriter]
 
-    val messageType = new SparkToParquetSchemaConverter(configuration).convert(schema)
+    val messageType = new SparkToParquetSchemaConverter(configuration).convert(shreddedSchema)
     val metadata = Map(
       SPARK_VERSION_METADATA_KEY -> SPARK_VERSION_SHORT,
       ParquetReadSupport.SPARK_METADATA_KEY -> schemaString
@@ -132,13 +147,23 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
       }
     }
 
-    logInfo(
-      s"""Initialized Parquet WriteSupport with Catalyst schema:
-         |${schema.prettyJson}
-         |and corresponding Parquet message type:
-         |$messageType
-       """.stripMargin)
-
+    if (shreddedSchemaString == null) {
+      logDebug(
+        s"""Initialized Parquet WriteSupport with Catalyst schema:
+           |${schema.prettyJson}
+           |and corresponding Parquet message type:
+           |$messageType
+         """.stripMargin)
+    } else {
+      logDebug(
+        s"""Initialized Parquet WriteSupport with Catalyst schema:
+           |${schema.prettyJson}
+           |and shredding schema:
+           |${shreddedSchema.prettyJson}
+           |and corresponding Parquet message type:
+           |$messageType
+         """.stripMargin)
+    }
     new WriteContext(messageType, metadata.asJava)
   }
 
@@ -148,7 +173,7 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
 
   override def write(row: InternalRow): Unit = {
     consumeMessage {
-      writeFields(row, schema, rootFieldWriters)
+      writeFields(row, shreddedSchema, rootFieldWriters)
     }
   }
 
@@ -165,7 +190,9 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
     }
   }
 
-  private def makeWriter(dataType: DataType): ValueWriter = {
+  // `inShredded` indicates whether the current traversal is nested within a shredded Variant
+  // schema. This affects how timestamp values are written.
+  private def makeWriter(dataType: DataType, inShredded: Boolean): ValueWriter = {
     dataType match {
       case BooleanType =>
         (row: SpecializedGetters, ordinal: Int) =>
@@ -199,31 +226,40 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
         (row: SpecializedGetters, ordinal: Int) =>
           recordConsumer.addDouble(row.getDouble(ordinal))
 
-      case StringType =>
+      case _: StringType =>
         (row: SpecializedGetters, ordinal: Int) =>
           recordConsumer.addBinary(
             Binary.fromReusedByteArray(row.getUTF8String(ordinal).getBytes))
 
       case TimestampType =>
-        outputTimestampType match {
-          case SQLConf.ParquetOutputTimestampType.INT96 =>
-            (row: SpecializedGetters, ordinal: Int) =>
-              val micros = int96RebaseFunc(row.getLong(ordinal))
-              val (julianDay, timeOfDayNanos) = DateTimeUtils.toJulianDay(micros)
-              val buf = ByteBuffer.wrap(timestampBuffer)
-              buf.order(ByteOrder.LITTLE_ENDIAN).putLong(timeOfDayNanos).putInt(julianDay)
-              recordConsumer.addBinary(Binary.fromReusedByteArray(timestampBuffer))
+        if (inShredded) {
+          // The Parquet Variant Shredding schema states that timestamp types must be stored as
+          // INT64. The Spark Timestamp type always has microsecond granularity and therefore, we
+          // always write shredded timestamps in the TIMESTAMP_MICROS configuration.
+          (row: SpecializedGetters, ordinal: Int) =>
+            val micros = row.getLong(ordinal)
+            recordConsumer.addLong(timestampRebaseFunc(micros))
+        } else {
+          outputTimestampType match {
+            case SQLConf.ParquetOutputTimestampType.INT96 =>
+              (row: SpecializedGetters, ordinal: Int) =>
+                val micros = int96RebaseFunc(row.getLong(ordinal))
+                val (julianDay, timeOfDayNanos) = DateTimeUtils.toJulianDay(micros)
+                val buf = ByteBuffer.wrap(timestampBuffer)
+                buf.order(ByteOrder.LITTLE_ENDIAN).putLong(timeOfDayNanos).putInt(julianDay)
+                recordConsumer.addBinary(Binary.fromReusedByteArray(timestampBuffer))
 
-          case SQLConf.ParquetOutputTimestampType.TIMESTAMP_MICROS =>
-            (row: SpecializedGetters, ordinal: Int) =>
-              val micros = row.getLong(ordinal)
-              recordConsumer.addLong(timestampRebaseFunc(micros))
+            case SQLConf.ParquetOutputTimestampType.TIMESTAMP_MICROS =>
+              (row: SpecializedGetters, ordinal: Int) =>
+                val micros = row.getLong(ordinal)
+                recordConsumer.addLong(timestampRebaseFunc(micros))
 
-          case SQLConf.ParquetOutputTimestampType.TIMESTAMP_MILLIS =>
-            (row: SpecializedGetters, ordinal: Int) =>
-              val micros = row.getLong(ordinal)
-              val millis = DateTimeUtils.microsToMillis(timestampRebaseFunc(micros))
-              recordConsumer.addLong(millis)
+            case SQLConf.ParquetOutputTimestampType.TIMESTAMP_MILLIS =>
+              (row: SpecializedGetters, ordinal: Int) =>
+                val micros = row.getLong(ordinal)
+                val millis = DateTimeUtils.microsToMillis(timestampRebaseFunc(micros))
+                recordConsumer.addLong(millis)
+          }
         }
 
       case TimestampNTZType =>
@@ -238,8 +274,33 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
       case DecimalType.Fixed(precision, scale) =>
         makeDecimalWriter(precision, scale)
 
+      case VariantType =>
+        (row: SpecializedGetters, ordinal: Int) =>
+          val v = row.getVariant(ordinal)
+          consumeGroup {
+            consumeField("value", 0) {
+              recordConsumer.addBinary(Binary.fromReusedByteArray(v.getValue))
+            }
+            consumeField("metadata", 1) {
+              recordConsumer.addBinary(Binary.fromReusedByteArray(v.getMetadata))
+            }
+          }
+
+      case s: StructType if SparkShreddingUtils.isVariantShreddingStruct(s) =>
+        val fieldWriters =
+          s.map(_.dataType).map(makeWriter(_, inShredded = true)).toArray[ValueWriter]
+        val variantShreddingSchema = SparkShreddingUtils.buildVariantSchema(s)
+        (row: SpecializedGetters, ordinal: Int) =>
+          val v = row.getVariant(ordinal)
+          val variant = new Variant(v.getValue, v.getMetadata)
+          val shreddedValues = SparkShreddingUtils.castShredded(variant, variantShreddingSchema)
+          consumeGroup {
+            writeFields(shreddedValues, s, fieldWriters)
+          }
+
       case t: StructType =>
-        val fieldWriters = t.map(_.dataType).map(makeWriter).toArray[ValueWriter]
+        val fieldWriters =
+          t.map(_.dataType).map(makeWriter(_, inShredded)).toArray[ValueWriter]
         (row: SpecializedGetters, ordinal: Int) =>
           consumeGroup {
             writeFields(row.getStruct(ordinal, t.length), t, fieldWriters)
@@ -249,9 +310,9 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
 
       case t: MapType => makeMapWriter(t)
 
-      case t: UserDefinedType[_] => makeWriter(t.sqlType)
+      case t: UserDefinedType[_] => makeWriter(t.sqlType, inShredded)
 
-      case _ => throw new IllegalStateException(s"Unsupported data type $dataType.")
+      case _ => throw SparkException.internalError(s"Unsupported data type $dataType.")
     }
   }
 
@@ -329,7 +390,8 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
   }
 
   def makeArrayWriter(arrayType: ArrayType): ValueWriter = {
-    val elementWriter = makeWriter(arrayType.elementType)
+    // The shredded schema should not have an array inside
+    val elementWriter = makeWriter(arrayType.elementType, inShredded = false)
 
     def threeLevelArrayWriter(repeatedGroupName: String, elementFieldName: String): ValueWriter =
       (row: SpecializedGetters, ordinal: Int) => {
@@ -409,8 +471,8 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
   }
 
   private def makeMapWriter(mapType: MapType): ValueWriter = {
-    val keyWriter = makeWriter(mapType.keyType)
-    val valueWriter = makeWriter(mapType.valueType)
+    val keyWriter = makeWriter(mapType.keyType, inShredded = false)
+    val valueWriter = makeWriter(mapType.valueType, inShredded = false)
     val repeatedGroupName = if (writeLegacyParquetFormat) {
       // Legacy mode:
       //
@@ -487,6 +549,10 @@ class ParquetWriteSupport extends WriteSupport[InternalRow] with Logging {
 
 object ParquetWriteSupport {
   val SPARK_ROW_SCHEMA: String = "org.apache.spark.sql.parquet.row.attributes"
+  // A version of `SPARK_ROW_SCHEMA`, where one or more Variant attributes have been replace with a
+  // shredded struct schema.
+  val SPARK_VARIANT_SHREDDING_SCHEMA: String =
+    "org.apache.spark.sql.parquet.variant.shredding.attributes"
 
   def setSchema(schema: StructType, configuration: Configuration): Unit = {
     configuration.set(SPARK_ROW_SCHEMA, schema.json)
@@ -494,4 +560,8 @@ object ParquetWriteSupport {
       ParquetOutputFormat.WRITER_VERSION,
       ParquetProperties.WriterVersion.PARQUET_1_0.toString)
   }
+
+  def setShreddingSchema(shreddingSchema: StructType, configuration: Configuration): Unit = {
+    configuration.set(SPARK_VARIANT_SHREDDING_SCHEMA, shreddingSchema.json)
+  }
 }
