@@ -19,23 +19,25 @@ package org.apache.spark.sql.execution.datasources.v2.jdbc
 import java.sql.SQLException
 import java.util
 
-import scala.collection.JavaConverters._
 import scala.collection.mutable
+import scala.jdk.CollectionConverters._
 
 import org.apache.spark.internal.Logging
 import org.apache.spark.sql.catalyst.analysis.NoSuchFunctionException
-import org.apache.spark.sql.connector.catalog.{FunctionCatalog, Identifier, NamespaceChange, SupportsNamespaces, Table, TableCatalog, TableChange}
+import org.apache.spark.sql.connector.catalog.{CatalogV2Util, Column, FunctionCatalog, Identifier, NamespaceChange, SupportsNamespaces, Table, TableCatalog, TableChange}
 import org.apache.spark.sql.connector.catalog.functions.UnboundFunction
 import org.apache.spark.sql.connector.expressions.Transform
-import org.apache.spark.sql.errors.{QueryCompilationErrors, QueryExecutionErrors}
+import org.apache.spark.sql.errors.{DataTypeErrorsBase, QueryCompilationErrors, QueryExecutionErrors}
 import org.apache.spark.sql.execution.datasources.jdbc.{JDBCOptions, JdbcOptionsInWrite, JDBCRDD, JdbcUtils}
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.jdbc.{JdbcDialect, JdbcDialects}
-import org.apache.spark.sql.types.StructType
 import org.apache.spark.sql.util.CaseInsensitiveStringMap
 
 class JDBCTableCatalog extends TableCatalog
-  with SupportsNamespaces with FunctionCatalog with Logging {
+  with SupportsNamespaces
+  with FunctionCatalog
+  with DataTypeErrorsBase
+  with Logging {
   import org.apache.spark.sql.connector.catalog.CatalogV2Implicits._
 
   private var catalogName: String = null
@@ -65,8 +67,16 @@ class JDBCTableCatalog extends TableCatalog
     checkNamespace(namespace)
     JdbcUtils.withConnection(options) { conn =>
       val schemaPattern = if (namespace.length == 1) namespace.head else null
-      val rs = conn.getMetaData
-        .getTables(null, schemaPattern, "%", Array("TABLE"))
+      val rs = JdbcUtils.classifyException(
+        condition = "FAILED_JDBC.GET_TABLES",
+        messageParameters = Map(
+          "url" -> options.getRedactUrl(),
+          "namespace" -> toSQLId(namespace.toSeq)),
+        dialect,
+        description = s"Failed get tables from: ${namespace.mkString(".")}",
+        isRuntime = false) {
+        conn.getMetaData.getTables(null, schemaPattern, "%", Array("TABLE"))
+      }
       new Iterator[Identifier] {
         def hasNext = rs.next()
         def next() = Identifier.of(namespace, rs.getString("TABLE_NAME"))
@@ -78,7 +88,14 @@ class JDBCTableCatalog extends TableCatalog
     checkNamespace(ident.namespace())
     val writeOptions = new JdbcOptionsInWrite(
       options.parameters + (JDBCOptions.JDBC_TABLE_NAME -> getTableName(ident)))
-    JdbcUtils.classifyException(s"Failed table existence check: $ident", dialect) {
+    JdbcUtils.classifyException(
+      condition = "FAILED_JDBC.TABLE_EXISTS",
+      messageParameters = Map(
+        "url" -> options.getRedactUrl(),
+        "tableName" -> toSQLId(ident)),
+      dialect,
+      description = s"Failed table existence check: $ident",
+      isRuntime = false) {
       JdbcUtils.withConnection(options)(JdbcUtils.tableExists(_, writeOptions))
     }
   }
@@ -98,27 +115,44 @@ class JDBCTableCatalog extends TableCatalog
   override def renameTable(oldIdent: Identifier, newIdent: Identifier): Unit = {
     checkNamespace(oldIdent.namespace())
     JdbcUtils.withConnection(options) { conn =>
-      JdbcUtils.classifyException(s"Failed table renaming from $oldIdent to $newIdent", dialect) {
+      JdbcUtils.classifyException(
+        condition = "FAILED_JDBC.RENAME_TABLE",
+        messageParameters = Map(
+          "url" -> options.getRedactUrl(),
+          "oldName" -> toSQLId(oldIdent),
+          "newName" -> toSQLId(newIdent)),
+        dialect,
+        description = s"Failed table renaming from $oldIdent to $newIdent",
+        isRuntime = false) {
         JdbcUtils.renameTable(conn, oldIdent, newIdent, options)
       }
     }
   }
 
   override def loadTable(ident: Identifier): Table = {
-    checkNamespace(ident.namespace())
+    if (!tableExists(ident)) {
+      throw QueryCompilationErrors.noSuchTableError(ident)
+    }
+
     val optionsWithTableName = new JDBCOptions(
       options.parameters + (JDBCOptions.JDBC_TABLE_NAME -> getTableName(ident)))
-    try {
+    JdbcUtils.classifyException(
+      condition = "FAILED_JDBC.LOAD_TABLE",
+      messageParameters = Map(
+        "url" -> options.getRedactUrl(),
+        "tableName" -> toSQLId(ident)),
+      dialect,
+      description = s"Failed to load table: $ident",
+      isRuntime = false
+    ) {
       val schema = JDBCRDD.resolveTable(optionsWithTableName)
       JDBCTable(ident, schema, optionsWithTableName)
-    } catch {
-      case _: SQLException => throw QueryCompilationErrors.noSuchTableError(ident)
     }
   }
 
   override def createTable(
       ident: Identifier,
-      schema: StructType,
+      columns: Array[Column],
       partitions: Array[Transform],
       properties: java.util.Map[String, String]): Table = {
     checkNamespace(ident.namespace())
@@ -157,8 +191,16 @@ class JDBCTableCatalog extends TableCatalog
 
     val writeOptions = new JdbcOptionsInWrite(tableOptions)
     val caseSensitive = SQLConf.get.caseSensitiveAnalysis
+    val schema = CatalogV2Util.v2ColumnsToStructType(columns)
     JdbcUtils.withConnection(options) { conn =>
-      JdbcUtils.classifyException(s"Failed table creation: $ident", dialect) {
+      JdbcUtils.classifyException(
+        condition = "FAILED_JDBC.CREATE_TABLE",
+        messageParameters = Map(
+          "url" -> options.getRedactUrl(),
+          "tableName" -> toSQLId(ident)),
+        dialect,
+        description = s"Failed table creation: $ident",
+        isRuntime = false) {
         JdbcUtils.createTable(conn, getTableName(ident), schema, caseSensitive, writeOptions)
       }
     }
@@ -169,7 +211,14 @@ class JDBCTableCatalog extends TableCatalog
   override def alterTable(ident: Identifier, changes: TableChange*): Table = {
     checkNamespace(ident.namespace())
     JdbcUtils.withConnection(options) { conn =>
-      JdbcUtils.classifyException(s"Failed table altering: $ident", dialect) {
+      JdbcUtils.classifyException(
+        condition = "FAILED_JDBC.ALTER_TABLE",
+        messageParameters = Map(
+          "url" -> options.getRedactUrl(),
+          "tableName" -> toSQLId(ident)),
+        dialect,
+        description = s"Failed table altering: $ident",
+        isRuntime = false) {
         JdbcUtils.alterTable(conn, getTableName(ident), changes, options)
       }
       loadTable(ident)
@@ -179,14 +228,30 @@ class JDBCTableCatalog extends TableCatalog
   override def namespaceExists(namespace: Array[String]): Boolean = namespace match {
     case Array(db) =>
       JdbcUtils.withConnection(options) { conn =>
-        JdbcUtils.schemaExists(conn, options, db)
+        JdbcUtils.classifyException(
+          condition = "FAILED_JDBC.NAMESPACE_EXISTS",
+          messageParameters = Map(
+            "url" -> options.getRedactUrl(),
+            "namespace" -> toSQLId(namespace.toSeq)),
+          dialect,
+          description = s"Failed namespace exists: ${namespace.mkString}",
+          isRuntime = false) {
+          JdbcUtils.schemaExists(conn, options, db)
+        }
       }
     case _ => false
   }
 
   override def listNamespaces(): Array[Array[String]] = {
     JdbcUtils.withConnection(options) { conn =>
-      JdbcUtils.listSchemas(conn, options)
+      JdbcUtils.classifyException(
+        condition = "FAILED_JDBC.LIST_NAMESPACES",
+        messageParameters = Map("url" -> options.getRedactUrl()),
+        dialect,
+        description = s"Failed list namespaces",
+        isRuntime = false) {
+        JdbcUtils.listSchemas(conn, options)
+      }
     }
   }
 
@@ -232,7 +297,14 @@ class JDBCTableCatalog extends TableCatalog
         }
       }
       JdbcUtils.withConnection(options) { conn =>
-        JdbcUtils.classifyException(s"Failed create name space: $db", dialect) {
+        JdbcUtils.classifyException(
+          condition = "FAILED_JDBC.CREATE_NAMESPACE",
+          messageParameters = Map(
+            "url" -> options.getRedactUrl(),
+            "namespace" -> toSQLId(db)),
+          dialect,
+          description = s"Failed create name space: $db",
+          isRuntime = false) {
           JdbcUtils.createSchema(conn, options, db, comment)
         }
       }
@@ -251,7 +323,14 @@ class JDBCTableCatalog extends TableCatalog
           case set: NamespaceChange.SetProperty =>
             if (set.property() == SupportsNamespaces.PROP_COMMENT) {
               JdbcUtils.withConnection(options) { conn =>
-                JdbcUtils.classifyException(s"Failed create comment on name space: $db", dialect) {
+                JdbcUtils.classifyException(
+                  condition = "FAILED_JDBC.CREATE_NAMESPACE_COMMENT",
+                  messageParameters = Map(
+                    "url" -> options.getRedactUrl(),
+                    "namespace" -> toSQLId(db)),
+                  dialect,
+                  description = s"Failed create comment on name space: $db",
+                  isRuntime = false) {
                   JdbcUtils.alterSchemaComment(conn, options, db, set.value)
                 }
               }
@@ -262,7 +341,14 @@ class JDBCTableCatalog extends TableCatalog
           case unset: NamespaceChange.RemoveProperty =>
             if (unset.property() == SupportsNamespaces.PROP_COMMENT) {
               JdbcUtils.withConnection(options) { conn =>
-                JdbcUtils.classifyException(s"Failed remove comment on name space: $db", dialect) {
+                JdbcUtils.classifyException(
+                  condition = "FAILED_JDBC.REMOVE_NAMESPACE_COMMENT",
+                  messageParameters = Map(
+                    "url" -> options.getRedactUrl(),
+                    "namespace" -> toSQLId(db)),
+                  dialect,
+                  description = s"Failed remove comment on name space: $db",
+                  isRuntime = false) {
                   JdbcUtils.removeSchemaComment(conn, options, db)
                 }
               }
@@ -284,7 +370,14 @@ class JDBCTableCatalog extends TableCatalog
       cascade: Boolean): Boolean = namespace match {
     case Array(db) if namespaceExists(namespace) =>
       JdbcUtils.withConnection(options) { conn =>
-        JdbcUtils.classifyException(s"Failed drop name space: $db", dialect) {
+        JdbcUtils.classifyException(
+          condition = "FAILED_JDBC.DROP_NAMESPACE",
+          messageParameters = Map(
+            "url" -> options.getRedactUrl(),
+            "namespace" -> toSQLId(db)),
+          dialect,
+          description = s"Failed drop name space: $db",
+          isRuntime = false) {
           JdbcUtils.dropSchema(conn, options, db, cascade)
           true
         }
@@ -324,4 +417,8 @@ class JDBCTableCatalog extends TableCatalog
         throw new NoSuchFunctionException(ident)
     }
   }
+
+  private def toSQLId(ident: Identifier): String = {
+    toSQLId(ident.namespace.toSeq :+ ident.name)
+  }
 }
