@@ -21,13 +21,16 @@ import org.apache.spark.{SparkConf, SparkFunSuite}
 import org.apache.spark.internal.config.IO_ENCRYPTION_ENABLED
 import org.apache.spark.internal.config.UI.UI_ENABLED
 import org.apache.spark.sql._
+import org.apache.spark.sql.catalyst.SQLConfHelper
 import org.apache.spark.sql.execution.adaptive._
 import org.apache.spark.sql.execution.adaptive.AQEShuffleReadExec
 import org.apache.spark.sql.execution.exchange.ReusedExchangeExec
 import org.apache.spark.sql.functions._
 import org.apache.spark.sql.internal.SQLConf
+import org.apache.spark.util.ArrayImplicits._
 
-class CoalesceShufflePartitionsSuite extends SparkFunSuite {
+class CoalesceShufflePartitionsSuite extends SparkFunSuite with SQLConfHelper
+    with AdaptiveSparkPlanHelper{
 
   private var originalActiveSparkSession: Option[SparkSession] = _
   private var originalInstantiatedSparkSession: Option[SparkSession] = _
@@ -102,12 +105,11 @@ class CoalesceShufflePartitionsSuite extends SparkFunSuite {
         // Check the answer first.
         QueryTest.checkAnswer(
           agg,
-          spark.range(0, 20).selectExpr("id", "50 as cnt").collect())
+          spark.range(0, 20).selectExpr("id", "50 as cnt").collect().toImmutableArraySeq)
 
         // Then, let's look at the number of post-shuffle partitions estimated
         // by the ExchangeCoordinator.
-        val finalPlan = agg.queryExecution.executedPlan
-          .asInstanceOf[AdaptiveSparkPlanExec].executedPlan
+        val finalPlan = stripAQEPlan(agg.queryExecution.executedPlan)
         val shuffleReads = finalPlan.collect {
           case r @ CoalescedShuffleRead() => r
         }
@@ -148,12 +150,11 @@ class CoalesceShufflePartitionsSuite extends SparkFunSuite {
             .union(spark.range(0, 1000).selectExpr("id % 500 as key", "id as value"))
         QueryTest.checkAnswer(
           join,
-          expectedAnswer.collect())
+          expectedAnswer.collect().toImmutableArraySeq)
 
         // Then, let's look at the number of post-shuffle partitions estimated
         // by the ExchangeCoordinator.
-        val finalPlan = join.queryExecution.executedPlan
-          .asInstanceOf[AdaptiveSparkPlanExec].executedPlan
+        val finalPlan = stripAQEPlan(join.queryExecution.executedPlan)
         val shuffleReads = finalPlan.collect {
           case r @ CoalescedShuffleRead() => r
         }
@@ -199,12 +200,11 @@ class CoalesceShufflePartitionsSuite extends SparkFunSuite {
             .selectExpr("id", "2 as cnt")
         QueryTest.checkAnswer(
           join,
-          expectedAnswer.collect())
+          expectedAnswer.collect().toImmutableArraySeq)
 
         // Then, let's look at the number of post-shuffle partitions estimated
         // by the ExchangeCoordinator.
-        val finalPlan = join.queryExecution.executedPlan
-          .asInstanceOf[AdaptiveSparkPlanExec].executedPlan
+        val finalPlan = stripAQEPlan(join.queryExecution.executedPlan)
         val shuffleReads = finalPlan.collect {
           case r @ CoalescedShuffleRead() => r
         }
@@ -250,12 +250,11 @@ class CoalesceShufflePartitionsSuite extends SparkFunSuite {
             .selectExpr("id % 500 as key", "2 as cnt", "id as value")
         QueryTest.checkAnswer(
           join,
-          expectedAnswer.collect())
+          expectedAnswer.collect().toImmutableArraySeq)
 
         // Then, let's look at the number of post-shuffle partitions estimated
         // by the ExchangeCoordinator.
-        val finalPlan = join.queryExecution.executedPlan
-          .asInstanceOf[AdaptiveSparkPlanExec].executedPlan
+        val finalPlan = stripAQEPlan(join.queryExecution.executedPlan)
         val shuffleReads = finalPlan.collect {
           case r @ CoalescedShuffleRead() => r
         }
@@ -293,11 +292,10 @@ class CoalesceShufflePartitionsSuite extends SparkFunSuite {
             .union(spark.range(500, 1000).selectExpr("id % 500", "id as value"))
           QueryTest.checkAnswer(
             join,
-            expectedAnswer.collect())
+            expectedAnswer.collect().toImmutableArraySeq)
 
           // Then, let's make sure we do not reduce number of post shuffle partitions.
-          val finalPlan = join.queryExecution.executedPlan
-            .asInstanceOf[AdaptiveSparkPlanExec].executedPlan
+          val finalPlan = stripAQEPlan(join.queryExecution.executedPlan)
           val shuffleReads = finalPlan.collect {
             case r @ CoalescedShuffleRead() => r
           }
@@ -315,7 +313,7 @@ class CoalesceShufflePartitionsSuite extends SparkFunSuite {
       import spark.implicits._
       spark.conf.set(SQLConf.AUTO_BROADCASTJOIN_THRESHOLD.key, "1KB")
       spark.conf.set(SQLConf.SKEW_JOIN_SKEWED_PARTITION_THRESHOLD.key, "10KB")
-      spark.conf.set(SQLConf.SKEW_JOIN_SKEWED_PARTITION_FACTOR, 2.0)
+      spark.conf.set(SQLConf.SKEW_JOIN_SKEWED_PARTITION_FACTOR.key, "2.0")
       val df00 = spark.range(0, 1000, 2)
         .selectExpr("id as key", "id as value")
         .union(Seq.fill(100000)((600, 600)).toDF("key", "value"))
@@ -343,7 +341,7 @@ class CoalesceShufflePartitionsSuite extends SparkFunSuite {
       import spark.implicits._
       spark.conf.set(SQLConf.AUTO_BROADCASTJOIN_THRESHOLD.key, "-1")
       spark.conf.set(SQLConf.SKEW_JOIN_SKEWED_PARTITION_THRESHOLD.key, "100B")
-      spark.conf.set(SQLConf.SKEW_JOIN_SKEWED_PARTITION_FACTOR, 2.0)
+      spark.conf.set(SQLConf.SKEW_JOIN_SKEWED_PARTITION_FACTOR.key, "2.0")
       val df00 = spark.range(0, 10, 2)
         .selectExpr("id as key", "id as value")
         .union(Seq.fill(1000)((600, 600)).toDF("key", "value"))
@@ -373,72 +371,71 @@ class CoalesceShufflePartitionsSuite extends SparkFunSuite {
 
   test("SPARK-24705 adaptive query execution works correctly when exchange reuse enabled") {
     val test: SparkSession => Unit = { spark: SparkSession =>
-      spark.sql("SET spark.sql.exchange.reuse=true")
-      val df = spark.range(0, 6, 1).selectExpr("id AS key", "id AS value")
+      withSQLConf("spark.sql.exchange.reuse" -> "true") {
+        val df = spark.range(0, 6, 1).selectExpr("id AS key", "id AS value")
 
-      // test case 1: a query stage has 3 child stages but they are the same stage.
-      // Final Stage 1
-      //   ShuffleQueryStage 0
-      //   ReusedQueryStage 0
-      //   ReusedQueryStage 0
-      val resultDf = df.join(df, "key").join(df, "key")
-      QueryTest.checkAnswer(resultDf, (0 to 5).map(i => Row(i, i, i, i)))
-      val finalPlan = resultDf.queryExecution.executedPlan
-        .asInstanceOf[AdaptiveSparkPlanExec].executedPlan
-      assert(finalPlan.collect {
-        case ShuffleQueryStageExec(_, r: ReusedExchangeExec, _) => r
-      }.length == 2)
-      assert(
-        finalPlan.collect {
-          case r @ CoalescedShuffleRead() => r
-        }.length == 3)
-
-
-      // test case 2: a query stage has 2 parent stages.
-      // Final Stage 3
-      //   ShuffleQueryStage 1
-      //     ShuffleQueryStage 0
-      //   ShuffleQueryStage 2
-      //     ReusedQueryStage 0
-      val grouped = df.groupBy((col("key") + 1).as("key")).agg(max("value").as("value"))
-      val resultDf2 = grouped.groupBy(col("key") + 1).max("value")
-        .union(grouped.groupBy(col("key") + 2).max("value"))
-      QueryTest.checkAnswer(resultDf2, Row(2, 0) :: Row(3, 0) :: Row(3, 1) :: Row(4, 1) ::
-        Row(4, 2) :: Row(5, 2) :: Row(5, 3) :: Row(6, 3) :: Row(6, 4) :: Row(7, 4) :: Row(7, 5) ::
-        Row(8, 5) :: Nil)
-
-      val finalPlan2 = resultDf2.queryExecution.executedPlan
-        .asInstanceOf[AdaptiveSparkPlanExec].executedPlan
-
-      // The result stage has 2 children
-      val level1Stages = finalPlan2.collect { case q: QueryStageExec => q }
-      assert(level1Stages.length == 2)
-
-      assert(
-        finalPlan2.collect {
-          case r @ CoalescedShuffleRead() => r
-        }.length == 2, "finalPlan2")
-
-      level1Stages.foreach(qs =>
-        assert(qs.plan.collect {
-          case r @ CoalescedShuffleRead() => r
-        }.length == 1,
-          "Wrong CoalescedShuffleRead below " + qs.simpleString(3)))
-
-      val leafStages = level1Stages.flatMap { stage =>
-        // All of the child stages of result stage have only one child stage.
-        val children = stage.plan.collect { case q: QueryStageExec => q }
-        assert(children.length == 1)
-        children
-      }
-      assert(leafStages.length == 2)
-
-      val reusedStages = level1Stages.flatMap { stage =>
-        stage.plan.collect {
+        // test case 1: a query stage has 3 child stages but they are the same stage.
+        // Final Stage 1
+        //   ShuffleQueryStage 0
+        //   ReusedQueryStage 0
+        //   ReusedQueryStage 0
+        val resultDf = df.join(df, "key").join(df, "key")
+        QueryTest.checkAnswer(resultDf, (0 to 5).map(i => Row(i, i, i, i)))
+        val finalPlan = stripAQEPlan(resultDf.queryExecution.executedPlan)
+        assert(finalPlan.collect {
           case ShuffleQueryStageExec(_, r: ReusedExchangeExec, _) => r
+        }.length == 2)
+        assert(
+          finalPlan.collect {
+            case r@CoalescedShuffleRead() => r
+          }.length == 3)
+
+
+        // test case 2: a query stage has 2 parent stages.
+        // Final Stage 3
+        //   ShuffleQueryStage 1
+        //     ShuffleQueryStage 0
+        //   ShuffleQueryStage 2
+        //     ReusedQueryStage 0
+        val grouped = df.groupBy((col("key") + 1).as("key")).agg(max("value").as("value"))
+        val resultDf2 = grouped.groupBy(col("key") + 1).max("value")
+          .union(grouped.groupBy(col("key") + 2).max("value"))
+        QueryTest.checkAnswer(resultDf2, Row(2, 0) :: Row(3, 0) :: Row(3, 1) :: Row(4, 1) ::
+          Row(4, 2) :: Row(5, 2) :: Row(5, 3) :: Row(6, 3) :: Row(6, 4) :: Row(7, 4) :: Row(7, 5) ::
+          Row(8, 5) :: Nil)
+
+        val finalPlan2 = stripAQEPlan(resultDf2.queryExecution.executedPlan)
+
+        // The result stage has 2 children
+        val level1Stages = finalPlan2.collect { case q: QueryStageExec => q }
+        assert(level1Stages.length == 2)
+
+        assert(
+          finalPlan2.collect {
+            case r@CoalescedShuffleRead() => r
+          }.length == 2, "finalPlan2")
+
+        level1Stages.foreach(qs =>
+          assert(qs.plan.collect {
+            case r@CoalescedShuffleRead() => r
+          }.length == 1,
+            "Wrong CoalescedShuffleRead below " + qs.simpleString(3)))
+
+        val leafStages = level1Stages.flatMap { stage =>
+          // All of the child stages of result stage have only one child stage.
+          val children = stage.plan.collect { case q: QueryStageExec => q }
+          assert(children.length == 1)
+          children
         }
+        assert(leafStages.length == 2)
+
+        val reusedStages = level1Stages.flatMap { stage =>
+          stage.plan.collect {
+            case ShuffleQueryStageExec(_, r: ReusedExchangeExec, _) => r
+          }
+        }
+        assert(reusedStages.length == 1)
       }
-      assert(reusedStages.length == 1)
     }
     withSparkSession(test, 400, None)
   }
@@ -450,8 +447,7 @@ class CoalesceShufflePartitionsSuite extends SparkFunSuite {
 
       QueryTest.checkAnswer(resultDf,
         Seq(0, 1, 2).map(i => Row(i)))
-      val finalPlan = resultDf.queryExecution.executedPlan
-        .asInstanceOf[AdaptiveSparkPlanExec].executedPlan
+      val finalPlan = stripAQEPlan(resultDf.queryExecution.executedPlan)
       assert(
         finalPlan.collect {
           case r @ CoalescedShuffleRead() => r
@@ -471,8 +467,7 @@ class CoalesceShufflePartitionsSuite extends SparkFunSuite {
 
       // Shuffle partition coalescing of the join is performed independent of the non-grouping
       // aggregate on the other side of the union.
-      val finalPlan = resultDf.queryExecution.executedPlan
-        .asInstanceOf[AdaptiveSparkPlanExec].executedPlan
+      val finalPlan = stripAQEPlan(resultDf.queryExecution.executedPlan)
       assert(
         finalPlan.collect {
           case r @ CoalescedShuffleRead() => r
@@ -487,8 +482,7 @@ class CoalesceShufflePartitionsSuite extends SparkFunSuite {
       val resultDf = ds.repartition(ds.col("id"))
       resultDf.collect()
 
-      val finalPlan = resultDf.queryExecution.executedPlan
-        .asInstanceOf[AdaptiveSparkPlanExec].executedPlan
+      val finalPlan = stripAQEPlan(resultDf.queryExecution.executedPlan)
       assert(
         finalPlan.collect {
           case r @ CoalescedShuffleRead() => r
