@@ -17,7 +17,7 @@
 
 package org.apache.spark.sql.execution
 
-import org.apache.spark.{SparkArithmeticException, SparkException, SparkFileNotFoundException}
+import org.apache.spark.{SparkArithmeticException, SparkException}
 import org.apache.spark.sql._
 import org.apache.spark.sql.catalyst.TableIdentifier
 import org.apache.spark.sql.catalyst.expressions.{Add, Alias, Divide}
@@ -85,19 +85,19 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
             exception = intercept[AnalysisException] {
               sql("CREATE VIEW jtv1 AS SELECT * FROM temp_jtv1 WHERE id < 6")
             },
-            errorClass = "INVALID_TEMP_OBJ_REFERENCE",
+            condition = "INVALID_TEMP_OBJ_REFERENCE",
             parameters = Map(
               "obj" -> "VIEW",
               "objName" -> s"`$SESSION_CATALOG_NAME`.`default`.`jtv1`",
               "tempObj" -> "VIEW",
               "tempObjName" -> "`temp_jtv1`"))
-          val globalTempDB = spark.sharedState.globalTempViewManager.database
+          val globalTempDB = spark.sharedState.globalTempDB
           sql("CREATE GLOBAL TEMP VIEW global_temp_jtv1 AS SELECT * FROM jt WHERE id > 0")
           checkError(
             exception = intercept[AnalysisException] {
               sql(s"CREATE VIEW jtv1 AS SELECT * FROM $globalTempDB.global_temp_jtv1 WHERE id < 6")
             },
-            errorClass = "INVALID_TEMP_OBJ_REFERENCE",
+            condition = "INVALID_TEMP_OBJ_REFERENCE",
             parameters = Map(
               "obj" -> "VIEW",
               "objName" -> s"`$SESSION_CATALOG_NAME`.`default`.`jtv1`",
@@ -115,25 +115,27 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql("CREATE OR REPLACE VIEW tab1 AS SELECT * FROM jt")
         },
-        errorClass = "_LEGACY_ERROR_TEMP_1278",
-        parameters = Map("name" -> s"`$SESSION_CATALOG_NAME`.`default`.`tab1`")
+        condition = "EXPECT_VIEW_NOT_TABLE.NO_ALTERNATIVE",
+        parameters = Map(
+          "tableName" -> s"`$SESSION_CATALOG_NAME`.`default`.`tab1`",
+          "operation" -> "CREATE OR REPLACE VIEW")
       )
       checkError(
         exception = intercept[AnalysisException] {
           sql("CREATE VIEW tab1 AS SELECT * FROM jt")
         },
-        errorClass = "_LEGACY_ERROR_TEMP_1278",
-        parameters = Map("name" -> s"`$SESSION_CATALOG_NAME`.`default`.`tab1`")
+        condition = "TABLE_OR_VIEW_ALREADY_EXISTS",
+        parameters = Map(
+          "relationName" -> s"`$SESSION_CATALOG_NAME`.`default`.`tab1`")
       )
       checkError(
         exception = intercept[AnalysisException] {
           sql("ALTER VIEW tab1 AS SELECT * FROM jt")
         },
-        errorClass = "_LEGACY_ERROR_TEMP_1015",
+        condition = "EXPECT_VIEW_NOT_TABLE.NO_ALTERNATIVE",
         parameters = Map(
-          "identifier" -> "default.tab1",
-          "cmd" -> "ALTER VIEW ... AS",
-          "hintStr" -> ""
+          "tableName" -> s"`$SESSION_CATALOG_NAME`.`default`.`tab1`",
+          "operation" -> "ALTER VIEW ... AS"
         ),
         context = ExpectedContext(
           fragment = "tab1",
@@ -155,12 +157,36 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
     val viewName = "testView"
     withTempView(viewName) {
       spark.range(10).createTempView(viewName)
-      assertAnalysisError(
-        s"ALTER VIEW $viewName SET TBLPROPERTIES ('p' = 'an')",
-        "testView is a temp view. 'ALTER VIEW ... SET TBLPROPERTIES' expects a permanent view.")
-      assertAnalysisError(
-        s"ALTER VIEW $viewName UNSET TBLPROPERTIES ('p')",
-        "testView is a temp view. 'ALTER VIEW ... UNSET TBLPROPERTIES' expects a permanent view.")
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER VIEW $viewName SET TBLPROPERTIES ('p' = 'an')")
+        },
+        condition = "EXPECT_PERMANENT_VIEW_NOT_TEMP",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER VIEW ... SET TBLPROPERTIES"
+        ),
+        context = ExpectedContext(
+          fragment = "testView",
+          start = 11,
+          stop = 18
+        )
+      )
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER VIEW $viewName UNSET TBLPROPERTIES ('p')")
+        },
+        condition = "EXPECT_PERMANENT_VIEW_NOT_TEMP",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER VIEW ... UNSET TBLPROPERTIES"
+        ),
+        context = ExpectedContext(
+          fragment = "testView",
+          start = 11,
+          stop = 18
+        )
+      )
     }
   }
 
@@ -168,50 +194,127 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
     val viewName = "testView"
     withTempView(viewName) {
       spark.range(10).createTempView(viewName)
-      assertErrorForAlterTableOnTempView(
-        s"ALTER TABLE $viewName SET SERDE 'whatever'",
-        viewName,
-        "ALTER TABLE ... SET [SERDE|SERDEPROPERTIES]")
-      assertErrorForAlterTableOnTempView(
-        s"ALTER TABLE $viewName PARTITION (a=1, b=2) SET SERDE 'whatever'",
-        viewName,
-        "ALTER TABLE ... SET [SERDE|SERDEPROPERTIES]")
-      assertErrorForAlterTableOnTempView(
-        s"ALTER TABLE $viewName SET SERDEPROPERTIES ('p' = 'an')",
-        viewName,
-        "ALTER TABLE ... SET [SERDE|SERDEPROPERTIES]")
-      assertErrorForAlterTableOnTempView(
-        s"ALTER TABLE $viewName PARTITION (a='4') RENAME TO PARTITION (a='5')",
-        viewName,
-        "ALTER TABLE ... RENAME TO PARTITION")
-      assertErrorForAlterTableOnTempView(
-        s"ALTER TABLE $viewName RECOVER PARTITIONS",
-        viewName,
-        "ALTER TABLE ... RECOVER PARTITIONS")
-      assertErrorForAlterTableOnTempView(
-        s"ALTER TABLE $viewName SET LOCATION '/path/to/your/lovely/heart'",
-        viewName,
-        "ALTER TABLE ... SET LOCATION ...")
-      assertErrorForAlterTableOnTempView(
-        s"ALTER TABLE $viewName PARTITION (a='4') SET LOCATION '/path/to/home'",
-        viewName,
-        "ALTER TABLE ... SET LOCATION ...")
-      assertErrorForAlterTableOnTempView(
-        s"ALTER TABLE $viewName ADD IF NOT EXISTS PARTITION (a='4', b='8')",
-        viewName,
-        "ALTER TABLE ... ADD PARTITION ...")
-      assertErrorForAlterTableOnTempView(
-        s"ALTER TABLE $viewName DROP PARTITION (a='4', b='8')",
-        viewName,
-        "ALTER TABLE ... DROP PARTITION ...")
-      assertErrorForAlterTableOnTempView(
-        s"ALTER TABLE $viewName SET TBLPROPERTIES ('p' = 'an')",
-        viewName,
-        "ALTER TABLE ... SET TBLPROPERTIES")
-      assertErrorForAlterTableOnTempView(
-        s"ALTER TABLE $viewName UNSET TBLPROPERTIES ('p')",
-        viewName,
-        "ALTER TABLE ... UNSET TBLPROPERTIES")
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER TABLE $viewName SET SERDE 'whatever'")
+        },
+        condition = "EXPECT_TABLE_NOT_VIEW.USE_ALTER_VIEW",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER TABLE ... SET [SERDE|SERDEPROPERTIES]"
+        ),
+        context = ExpectedContext(fragment = viewName, start = 12, stop = 19)
+      )
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER TABLE $viewName PARTITION (a=1, b=2) SET SERDE 'whatever'")
+        },
+        condition = "EXPECT_TABLE_NOT_VIEW.USE_ALTER_VIEW",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER TABLE ... SET [SERDE|SERDEPROPERTIES]"
+        ),
+        context = ExpectedContext(fragment = viewName, start = 12, stop = 19)
+      )
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER TABLE $viewName SET SERDEPROPERTIES ('p' = 'an')")
+        },
+        condition = "EXPECT_TABLE_NOT_VIEW.USE_ALTER_VIEW",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER TABLE ... SET [SERDE|SERDEPROPERTIES]"
+        ),
+        context = ExpectedContext(fragment = viewName, start = 12, stop = 19)
+      )
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER TABLE $viewName PARTITION (a='4') RENAME TO PARTITION (a='5')")
+        },
+        condition = "EXPECT_TABLE_NOT_VIEW.NO_ALTERNATIVE",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER TABLE ... RENAME TO PARTITION"
+        ),
+        context = ExpectedContext(fragment = viewName, start = 12, stop = 19)
+      )
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER TABLE $viewName RECOVER PARTITIONS")
+        },
+        condition = "EXPECT_TABLE_NOT_VIEW.NO_ALTERNATIVE",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER TABLE ... RECOVER PARTITIONS"
+        ),
+        context = ExpectedContext(fragment = viewName, start = 12, stop = 19)
+      )
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER TABLE $viewName SET LOCATION '/path/to/your/lovely/heart'")
+        },
+        condition = "EXPECT_TABLE_NOT_VIEW.NO_ALTERNATIVE",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER TABLE ... SET LOCATION ..."
+        ),
+        context = ExpectedContext(fragment = viewName, start = 12, stop = 19)
+      )
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER TABLE $viewName PARTITION (a='4') SET LOCATION '/path/to/home'")
+        },
+        condition = "EXPECT_TABLE_NOT_VIEW.NO_ALTERNATIVE",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER TABLE ... SET LOCATION ..."
+        ),
+        context = ExpectedContext(fragment = viewName, start = 12, stop = 19)
+      )
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER TABLE $viewName ADD IF NOT EXISTS PARTITION (a='4', b='8')")
+        },
+        condition = "EXPECT_TABLE_NOT_VIEW.NO_ALTERNATIVE",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER TABLE ... ADD PARTITION ..."
+        ),
+        context = ExpectedContext(fragment = viewName, start = 12, stop = 19)
+      )
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER TABLE $viewName DROP PARTITION (a='4', b='8')")
+        },
+        condition = "EXPECT_TABLE_NOT_VIEW.NO_ALTERNATIVE",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER TABLE ... DROP PARTITION ..."
+        ),
+        context = ExpectedContext(fragment = viewName, start = 12, stop = 19)
+      )
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER TABLE $viewName SET TBLPROPERTIES ('p' = 'an')")
+        },
+        condition = "EXPECT_TABLE_NOT_VIEW.USE_ALTER_VIEW",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER TABLE ... SET TBLPROPERTIES"
+        ),
+        context = ExpectedContext(fragment = viewName, start = 12, stop = 19)
+      )
+      checkError(
+        exception = intercept[AnalysisException] {
+          sql(s"ALTER TABLE $viewName UNSET TBLPROPERTIES ('p')")
+        },
+        condition = "EXPECT_TABLE_NOT_VIEW.USE_ALTER_VIEW",
+        parameters = Map(
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ALTER TABLE ... UNSET TBLPROPERTIES"
+        ),
+        context = ExpectedContext(fragment = viewName, start = 12, stop = 19)
+      )
     }
   }
 
@@ -224,7 +327,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql(s"INSERT INTO TABLE $viewName SELECT 1")
         },
-        errorClass = "UNSUPPORTED_INSERT.RDD_BASED",
+        condition = "UNSUPPORTED_INSERT.RDD_BASED",
         parameters = Map.empty
       )
 
@@ -235,12 +338,10 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql(sqlText)
         },
-        errorClass = "_LEGACY_ERROR_TEMP_1013",
+        condition = "EXPECT_TABLE_NOT_VIEW.NO_ALTERNATIVE",
         parameters = Map(
-          "nameParts" -> viewName,
-          "viewStr" -> "temp view",
-          "cmd" -> "LOAD DATA",
-          "hintStr" -> ""
+          "viewName" -> s"`$viewName`",
+          "operation" -> "LOAD DATA"
         ),
         context = ExpectedContext(
           fragment = viewName,
@@ -252,10 +353,10 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql(s"SHOW CREATE TABLE $viewName")
         },
-        errorClass = "_LEGACY_ERROR_TEMP_1016",
+        condition = "EXPECT_PERMANENT_VIEW_NOT_TEMP",
         parameters = Map(
-          "nameParts" -> "testView",
-          "cmd" -> "SHOW CREATE TABLE"
+          "viewName" -> s"`$viewName`",
+          "operation" -> "SHOW CREATE TABLE"
         ),
         context = ExpectedContext(
           fragment = viewName,
@@ -267,10 +368,10 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql(s"ANALYZE TABLE $viewName COMPUTE STATISTICS")
         },
-        errorClass = "_LEGACY_ERROR_TEMP_1016",
+        condition = "EXPECT_PERMANENT_VIEW_NOT_TEMP",
         parameters = Map(
-          "nameParts" -> "testView",
-          "cmd" -> "ANALYZE TABLE"
+          "viewName" -> s"`$viewName`",
+          "operation" -> "ANALYZE TABLE"
         ),
         context = ExpectedContext(
           fragment = viewName,
@@ -282,30 +383,19 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql(s"ANALYZE TABLE $viewName COMPUTE STATISTICS FOR COLUMNS id")
         },
-        errorClass = "UNSUPPORTED_FEATURE.ANALYZE_UNCACHED_TEMP_VIEW",
+        condition = "UNSUPPORTED_FEATURE.ANALYZE_UNCACHED_TEMP_VIEW",
         parameters = Map("viewName" -> s"`$viewName`")
       )
     }
   }
 
-  private def assertAnalysisError(query: String, message: String): Unit = {
-    val e = intercept[AnalysisException](sql(query))
-    assert(e.message.contains(message))
-  }
-
-  private def assertAnalysisErrorClass(query: String,
-      errorClass: String,
+  private def assertAnalysisErrorCondition(
+      query: String,
+      condition: String,
       parameters: Map[String, String],
       context: ExpectedContext): Unit = {
     val e = intercept[AnalysisException](sql(query))
-    checkError(e, errorClass = errorClass, parameters = parameters, context = context)
-  }
-
-  private def assertErrorForAlterTableOnTempView(
-    sqlText: String, viewName: String, cmdName: String): Unit = {
-    assertAnalysisError(
-      sqlText,
-      s"$viewName is a temp view. '$cmdName' expects a table. Please use ALTER VIEW instead.")
+    checkError(e, condition = condition, parameters = parameters, context = context)
   }
 
   test("error handling: insert/load table commands against a view") {
@@ -316,8 +406,11 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql(s"INSERT INTO TABLE $viewName SELECT 1")
         },
-        errorClass = "_LEGACY_ERROR_TEMP_1010",
-        parameters = Map("identifier" -> s"`$SESSION_CATALOG_NAME`.`default`.`testview`"),
+        condition = "EXPECT_TABLE_NOT_VIEW.NO_ALTERNATIVE",
+        parameters = Map(
+          "viewName" -> s"`$SESSION_CATALOG_NAME`.`default`.`testview`",
+          "operation" -> "INSERT"
+        ),
         context = ExpectedContext(fragment = viewName, start = 18, stop = 25)
       )
 
@@ -328,12 +421,10 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql(sqlText)
         },
-        errorClass = "_LEGACY_ERROR_TEMP_1013",
+        condition = "EXPECT_TABLE_NOT_VIEW.NO_ALTERNATIVE",
         parameters = Map(
-          "nameParts" -> "spark_catalog.default.testview",
-          "viewStr" -> "view",
-          "cmd" -> "LOAD DATA",
-          "hintStr" -> ""),
+          "viewName" -> s"`$SESSION_CATALOG_NAME`.`default`.`testview`",
+          "operation" -> "LOAD DATA"),
         context = ExpectedContext(
           fragment = viewName,
           start = sqlText.length - 8,
@@ -398,7 +489,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
 
   test("error handling: fail if the temp view sql itself is invalid") {
     // A database that does not exist
-    assertAnalysisErrorClass(
+    assertAnalysisErrorCondition(
       "CREATE OR REPLACE TEMPORARY VIEW myabcdview AS SELECT * FROM db_not_exist234.jt",
       "TABLE_OR_VIEW_NOT_FOUND",
       Map("relationName" -> "`db_not_exist234`.`jt`"),
@@ -423,7 +514,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[ParseException] {
           sql(sqlText)
         },
-        errorClass = "_LEGACY_ERROR_TEMP_0035",
+        condition = "_LEGACY_ERROR_TEMP_0035",
         parameters = Map("message" -> "TBLPROPERTIES can't coexist with CREATE TEMPORARY VIEW"),
         context = ExpectedContext(sqlText, 0, 77))
     }
@@ -777,7 +868,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
           exception = intercept[AnalysisException] {
             sql("CREATE VIEW testView2(x, y, z) AS SELECT * FROM tab1")
           },
-          errorClass = "CREATE_VIEW_COLUMN_ARITY_MISMATCH.NOT_ENOUGH_DATA_COLUMNS",
+          condition = "CREATE_VIEW_COLUMN_ARITY_MISMATCH.NOT_ENOUGH_DATA_COLUMNS",
           parameters = Map(
             "viewName" -> s"`$SESSION_CATALOG_NAME`.`default`.`testView2`",
             "viewColumns" -> "`x`, `y`, `z`",
@@ -794,7 +885,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
           .write.mode(SaveMode.Overwrite).saveAsTable("tab1")
         checkError(
           exception = intercept[AnalysisException](sql("SELECT * FROM testView")),
-          errorClass = "INCOMPATIBLE_VIEW_SCHEMA_CHANGE",
+          condition = "INCOMPATIBLE_VIEW_SCHEMA_CHANGE",
           parameters = Map(
             "viewName" -> s"`$SESSION_CATALOG_NAME`.`default`.`testview`",
             "actualCols" -> "[]",
@@ -809,46 +900,48 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
 
   test("resolve a view when the dataTypes of referenced table columns changed") {
     withTable("tab1") {
-      spark.range(1, 10).selectExpr("id", "id + 1 id1").write.saveAsTable("tab1")
-      withView("testView") {
-        sql("CREATE VIEW testView AS SELECT * FROM tab1")
+      withSQLConf("spark.sql.legacy.viewSchemaCompensation" -> "false") {
+        spark.range(1, 10).selectExpr("id", "id + 1 id1").write.saveAsTable("tab1")
+        withView("testView") {
+          sql("CREATE VIEW testView AS SELECT * FROM tab1")
 
-        // Allow casting from IntegerType to LongType
-        val df = (1 until 10).map(i => (i, i + 1)).toDF("id", "id1")
-        df.write.format("json").mode(SaveMode.Overwrite).saveAsTable("tab1")
-        checkAnswer(sql("SELECT * FROM testView ORDER BY id1"), (1 to 9).map(i => Row(i, i + 1)))
+          // Allow casting from IntegerType to LongType
+          val df = (1 until 10).map(i => (i, i + 1)).toDF("id", "id1")
+          df.write.format("json").mode(SaveMode.Overwrite).saveAsTable("tab1")
+          checkAnswer(sql("SELECT * FROM testView ORDER BY id1"), (1 to 9).map(i => Row(i, i + 1)))
 
-        // Casting from DoubleType to LongType might truncate, throw an AnalysisException.
-        val df2 = (1 until 10).map(i => (i.toDouble, i.toDouble)).toDF("id", "id1")
-        df2.write.format("json").mode(SaveMode.Overwrite).saveAsTable("tab1")
-        checkError(
-          exception = intercept[AnalysisException](sql("SELECT * FROM testView")),
-          errorClass = "CANNOT_UP_CAST_DATATYPE",
-          parameters = Map(
-            "expression" -> s"$SESSION_CATALOG_NAME.default.tab1.id",
-            "sourceType" -> "\"DOUBLE\"",
-            "targetType" -> "\"BIGINT\"",
-            "details" -> ("The type path of the target object is:\n\n" +
-              "You can either add an explicit cast to the input data or " +
-              "choose a higher precision type of the field in the target object")
+          // Casting from DoubleType to LongType might truncate, throw an AnalysisException.
+          val df2 = (1 until 10).map(i => (i.toDouble, i.toDouble)).toDF("id", "id1")
+          df2.write.format("json").mode(SaveMode.Overwrite).saveAsTable("tab1")
+          checkError(
+            exception = intercept[AnalysisException](sql("SELECT * FROM testView")),
+            condition = "CANNOT_UP_CAST_DATATYPE",
+            parameters = Map(
+              "expression" -> s"$SESSION_CATALOG_NAME.default.tab1.id",
+              "sourceType" -> "\"DOUBLE\"",
+              "targetType" -> "\"BIGINT\"",
+              "details" -> ("The type path of the target object is:\n\n" +
+                "You can either add an explicit cast to the input data or " +
+                "choose a higher precision type of the field in the target object")
+            )
           )
-        )
 
-        // Can't cast from ArrayType to LongType, throw an AnalysisException.
-        val df3 = (1 until 10).map(i => (i, Seq(i))).toDF("id", "id1")
-        df3.write.format("json").mode(SaveMode.Overwrite).saveAsTable("tab1")
-        checkError(
-          exception = intercept[AnalysisException](sql("SELECT * FROM testView")),
-          errorClass = "CANNOT_UP_CAST_DATATYPE",
-          parameters = Map(
-            "expression" -> s"$SESSION_CATALOG_NAME.default.tab1.id1",
-            "sourceType" -> "\"ARRAY<INT>\"",
-            "targetType" -> "\"BIGINT\"",
-            "details" -> ("The type path of the target object is:\n\n" +
-              "You can either add an explicit cast to the input data or " +
-              "choose a higher precision type of the field in the target object")
+          // Can't cast from ArrayType to LongType, throw an AnalysisException.
+          val df3 = (1 until 10).map(i => (i, Seq(i))).toDF("id", "id1")
+          df3.write.format("json").mode(SaveMode.Overwrite).saveAsTable("tab1")
+          checkError(
+            exception = intercept[AnalysisException](sql("SELECT * FROM testView")),
+            condition = "CANNOT_UP_CAST_DATATYPE",
+            parameters = Map(
+              "expression" -> s"$SESSION_CATALOG_NAME.default.tab1.id1",
+              "sourceType" -> "\"ARRAY<INT>\"",
+              "targetType" -> "\"BIGINT\"",
+              "details" -> ("The type path of the target object is:\n\n" +
+                "You can either add an explicit cast to the input data or " +
+                "choose a higher precision type of the field in the target object")
+            )
           )
-        )
+        }
       }
     }
   }
@@ -864,7 +957,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql("ALTER VIEW view1 AS SELECT * FROM view2")
         },
-        errorClass = "RECURSIVE_VIEW",
+        condition = "RECURSIVE_VIEW",
         parameters = Map(
           "viewIdent" -> s"`$SESSION_CATALOG_NAME`.`default`.`view1`",
           "newPath" -> (s"`$SESSION_CATALOG_NAME`.`default`.`view1` -> " +
@@ -878,7 +971,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql("ALTER VIEW view1 AS SELECT * FROM view3 JOIN view2")
         },
-        errorClass = "RECURSIVE_VIEW",
+        condition = "RECURSIVE_VIEW",
         parameters = Map(
           "viewIdent" -> s"`$SESSION_CATALOG_NAME`.`default`.`view1`",
           "newPath" -> (s"`$SESSION_CATALOG_NAME`.`default`.`view1` -> " +
@@ -893,7 +986,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql("CREATE OR REPLACE VIEW view1 AS SELECT * FROM view2")
         },
-        errorClass = "RECURSIVE_VIEW",
+        condition = "RECURSIVE_VIEW",
         parameters = Map(
           "viewIdent" -> s"`$SESSION_CATALOG_NAME`.`default`.`view1`",
           "newPath" -> (s"`$SESSION_CATALOG_NAME`.`default`.`view1` -> " +
@@ -907,7 +1000,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
         exception = intercept[AnalysisException] {
           sql("ALTER VIEW view1 AS SELECT * FROM jt WHERE EXISTS (SELECT 1 FROM view2)")
         },
-        errorClass = "RECURSIVE_VIEW",
+        condition = "RECURSIVE_VIEW",
         parameters = Map(
           "viewIdent" -> s"`$SESSION_CATALOG_NAME`.`default`.`view1`",
           "newPath" -> (s"`$SESSION_CATALOG_NAME`.`default`.`view1` -> " +
@@ -978,9 +1071,9 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
           checkErrorMatchPVals(
             exception = intercept[SparkException] {
               sql("SELECT * FROM v1").collect()
-            }.getCause.asInstanceOf[SparkFileNotFoundException],
-            errorClass = "_LEGACY_ERROR_TEMP_2055",
-            parameters = Map("message" -> ".* does not exist")
+            },
+            condition = "FAILED_READ_FILE.FILE_NOT_EXIST",
+            parameters = Map("path" -> ".*")
           )
         }
 
@@ -998,9 +1091,9 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
           checkErrorMatchPVals(
             exception = intercept[SparkException] {
               sql("SELECT * FROM v1").collect()
-            }.getCause.asInstanceOf[SparkFileNotFoundException],
-            errorClass = "_LEGACY_ERROR_TEMP_2055",
-            parameters = Map("message" -> ".* does not exist")
+            },
+            condition = "FAILED_READ_FILE.FILE_NOT_EXIST",
+            parameters = Map("path" -> ".*")
           )
         }
       }
@@ -1010,7 +1103,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
   test("local temp view refers global temp view") {
     withGlobalTempView("v1") {
       withTempView("v2") {
-        val globalTempDB = spark.sharedState.globalTempViewManager.database
+        val globalTempDB = spark.sharedState.globalTempDB
         sql("CREATE GLOBAL TEMPORARY VIEW v1 AS SELECT 1")
         sql(s"CREATE TEMPORARY VIEW v2 AS SELECT * FROM ${globalTempDB}.v1")
         checkAnswer(sql("SELECT * FROM v2"), Seq(Row(1)))
@@ -1021,7 +1114,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
   test("global temp view refers local temp view") {
     withTempView("v1") {
       withGlobalTempView("v2") {
-        val globalTempDB = spark.sharedState.globalTempViewManager.database
+        val globalTempDB = spark.sharedState.globalTempDB
         sql("CREATE TEMPORARY VIEW v1 AS SELECT 1")
         sql(s"CREATE GLOBAL TEMPORARY VIEW v2 AS SELECT * FROM v1")
         checkAnswer(sql(s"SELECT * FROM ${globalTempDB}.v2"), Seq(Row(1)))
@@ -1065,7 +1158,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
               sql("SELECT * FROM v1")
             }
             checkError(e,
-              errorClass = "UNRESOLVED_COLUMN.WITH_SUGGESTION",
+              condition = "UNRESOLVED_COLUMN.WITH_SUGGESTION",
               sqlState = None,
               parameters = Map(
                 "objectName" -> "`C1`",
@@ -1086,7 +1179,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
               sql("SELECT * FROM v3")
             }
             checkError(e,
-              errorClass = "MISSING_AGGREGATION",
+              condition = "MISSING_AGGREGATION",
               parameters = Map(
                 "expression" -> "\"c1\"",
                 "expressionAnyValue" -> "\"any_value(c1)\""))
@@ -1096,7 +1189,7 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
               sql("SELECT * FROM v4")
             }
             checkError(e,
-              errorClass = "UNRESOLVED_COLUMN.WITH_SUGGESTION",
+              condition = "UNRESOLVED_COLUMN.WITH_SUGGESTION",
               sqlState = None,
               parameters = Map(
                 "objectName" -> "`a`",
@@ -1114,9 +1207,9 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
               exception = intercept[SparkArithmeticException] {
                 sql("SELECT * FROM v5").collect()
               },
-              errorClass = "DIVIDE_BY_ZERO",
+              condition = "DIVIDE_BY_ZERO",
               parameters = Map("config" -> "\"spark.sql.ansi.enabled\""),
-              context = new ExpectedContext(
+              context = ExpectedContext(
                 objectType = "VIEW",
                 objectName = s"$SESSION_CATALOG_NAME.default.v5",
                 fragment = "1/0",
@@ -1133,9 +1226,9 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
           exception = intercept[SparkArithmeticException] {
             sql("SELECT * FROM v1").collect()
           },
-          errorClass = "DIVIDE_BY_ZERO",
+          condition = "DIVIDE_BY_ZERO",
           parameters = Map("config" -> "\"spark.sql.ansi.enabled\""),
-          context = new ExpectedContext(
+          context = ExpectedContext(
             objectType = "VIEW",
             objectName = s"$SESSION_CATALOG_NAME.default.v1",
             fragment = "1/0",
@@ -1230,4 +1323,21 @@ abstract class SQLViewSuite extends QueryTest with SQLTestUtils {
       assert(ts1._1.getTime < ts2._1.getTime)
     }
   }
+
+  test("SPARK-52521: view with ANSI expressions") {
+    withView("v1") {
+      withSQLConf(ANSI_ENABLED.key -> "true") {
+        sql(
+          """
+            |CREATE VIEW v1 AS
+            |SELECT RIGHT(CAST(id AS STRING), 1) AS c
+            |FROM range(1)
+            |GROUP BY RIGHT(CAST(id AS STRING), 1)
+            |""".stripMargin)
+      }
+      withSQLConf(ANSI_ENABLED.key -> "false") {
+        checkAnswer(sql("SELECT * FROM v1"), Row("0"))
+      }
+    }
+  }
 }
