@@ -26,12 +26,14 @@ import org.apache.hadoop.fs.{BlockLocation, FileStatus, Path, RawLocalFileSystem
 import org.apache.hadoop.mapreduce.Job
 
 import org.apache.spark.SparkException
+import org.apache.spark.internal.config
 import org.apache.spark.paths.SparkPath.{fromUrlString => sp}
-import org.apache.spark.sql._
+import org.apache.spark.sql.{execution, DataFrame, QueryTest, Row, SparkSession}
 import org.apache.spark.sql.catalyst.InternalRow
 import org.apache.spark.sql.catalyst.catalog.BucketSpec
 import org.apache.spark.sql.catalyst.expressions.{Expression, ExpressionSet}
 import org.apache.spark.sql.catalyst.util
+import org.apache.spark.sql.classic.Dataset
 import org.apache.spark.sql.execution.{DataSourceScanExec, FileSourceScanExec, SparkPlan}
 import org.apache.spark.sql.execution.datasources.v2.DataSourceV2ScanRelation
 import org.apache.spark.sql.functions._
@@ -44,7 +46,7 @@ import org.apache.spark.util.Utils
 class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
   import testImplicits._
 
-  protected override def sparkConf = super.sparkConf.set("spark.default.parallelism", "1")
+  protected override def sparkConf = super.sparkConf.set(config.DEFAULT_PARALLELISM.key, "1")
 
   test("unpartitioned table, single partition") {
     val table =
@@ -64,7 +66,7 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
     checkScan(table.select($"c1")) { partitions =>
       // 10 one byte files should fit in a single partition with 10 files.
       assert(partitions.size == 1, "when checking partitions")
-      assert(partitions.head.files.size == 10, "when checking partition 1")
+      assert(partitions.head.files.length == 10, "when checking partition 1")
       // 1 byte files are too small to split so we should read the whole thing.
       assert(partitions.head.files.head.start == 0)
       assert(partitions.head.files.head.length == 1)
@@ -87,8 +89,8 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
       checkScan(table.select($"c1")) { partitions =>
         // 5 byte files should be laid out [(5, 5), (5)]
         assert(partitions.size == 2, "when checking partitions")
-        assert(partitions(0).files.size == 2, "when checking partition 1")
-        assert(partitions(1).files.size == 1, "when checking partition 2")
+        assert(partitions(0).files.length == 2, "when checking partition 1")
+        assert(partitions(1).files.length == 1, "when checking partition 2")
 
         // 5 byte files are too small to split so we should read the whole thing.
         assert(partitions.head.files.head.start == 0)
@@ -112,8 +114,8 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
       checkScan(table.select($"c1")) { partitions =>
         // Files should be laid out [(0-10), (10-15, 4)]
         assert(partitions.size == 2, "when checking partitions")
-        assert(partitions(0).files.size == 1, "when checking partition 1")
-        assert(partitions(1).files.size == 2, "when checking partition 2")
+        assert(partitions(0).files.length == 1, "when checking partition 1")
+        assert(partitions(1).files.length == 2, "when checking partition 2")
 
         // Start by reading 10 bytes of the first file
         assert(partitions.head.files.head.start == 0)
@@ -145,10 +147,10 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
       checkScan(table.select($"c1")) { partitions =>
         // Files should be laid out [(file1), (file2, file3), (file4, file5), (file6)]
         assert(partitions.size == 4, "when checking partitions")
-        assert(partitions(0).files.size == 1, "when checking partition 1")
-        assert(partitions(1).files.size == 2, "when checking partition 2")
-        assert(partitions(2).files.size == 2, "when checking partition 3")
-        assert(partitions(3).files.size == 1, "when checking partition 4")
+        assert(partitions(0).files.length == 1, "when checking partition 1")
+        assert(partitions(1).files.length == 2, "when checking partition 2")
+        assert(partitions(2).files.length == 2, "when checking partition 3")
+        assert(partitions(3).files.length == 1, "when checking partition 4")
 
         // First partition reads (file1)
         assert(partitions(0).files(0).start == 0)
@@ -186,7 +188,7 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
     // Only one file should be read.
     checkScan(table.where("p1 = 1")) { partitions =>
       assert(partitions.size == 1, "when checking partitions")
-      assert(partitions.head.files.size == 1, "when files in partition 1")
+      assert(partitions.head.files.length == 1, "when files in partition 1")
     }
     // We don't need to reevaluate filters that are only on partitions.
     checkDataFilters(Set.empty)
@@ -194,7 +196,7 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
     // Only one file should be read.
     checkScan(table.where("p1 = 1 AND c1 = 1 AND (p1 + c1) = 2")) { partitions =>
       assert(partitions.size == 1, "when checking partitions")
-      assert(partitions.head.files.size == 1, "when checking files in partition 1")
+      assert(partitions.head.files.length == 1, "when checking files in partition 1")
       assert(partitions.head.files.head.partitionValues.getInt(0) == 1,
         "when checking partition values")
     }
@@ -213,7 +215,7 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
       // Only one file should be read.
       checkScan(table.where("P1 = 1")) { partitions =>
         assert(partitions.size == 1, "when checking partitions")
-        assert(partitions.head.files.size == 1, "when files in partition 1")
+        assert(partitions.head.files.length == 1, "when files in partition 1")
       }
       // We don't need to reevaluate filters that are only on partitions.
       checkDataFilters(Set.empty)
@@ -221,7 +223,7 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
       // Only one file should be read.
       checkScan(table.where("P1 = 1 AND C1 = 1 AND (P1 + C1) = 2")) { partitions =>
         assert(partitions.size == 1, "when checking partitions")
-        assert(partitions.head.files.size == 1, "when checking files in partition 1")
+        assert(partitions.head.files.length == 1, "when checking files in partition 1")
         assert(partitions.head.files.head.partitionValues.getInt(0) == 1,
           "when checking partition values")
       }
@@ -267,17 +269,17 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
       // No partition pruning
       checkScan(table) { partitions =>
         assert(partitions.size == 3)
-        assert(partitions(0).files.size == 5)
-        assert(partitions(1).files.size == 0)
-        assert(partitions(2).files.size == 2)
+        assert(partitions(0).files.length == 5)
+        assert(partitions(1).files.length == 0)
+        assert(partitions(2).files.length == 2)
       }
 
       // With partition pruning
       checkScan(table.where("p1=2")) { partitions =>
         assert(partitions.size == 3)
-        assert(partitions(0).files.size == 3)
-        assert(partitions(1).files.size == 0)
-        assert(partitions(2).files.size == 1)
+        assert(partitions(0).files.length == 3)
+        assert(partitions(1).files.length == 0)
+        assert(partitions(2).files.length == 1)
       }
     }
   }
@@ -362,8 +364,8 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
         SQLConf.FILES_OPEN_COST_IN_BYTES.key -> "0") {
         checkScan(table.select($"c1")) { partitions =>
           assert(partitions.size == 2)
-          assert(partitions(0).files.size == 1)
-          assert(partitions(1).files.size == 2)
+          assert(partitions(0).files.length == 1)
+          assert(partitions(1).files.length == 2)
         }
       }
     }
@@ -378,9 +380,9 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
         SQLConf.FILES_OPEN_COST_IN_BYTES.key -> "0") {
         checkScan(table.select($"c1")) { partitions =>
           assert(partitions.size == 3)
-          assert(partitions(0).files.size == 1)
-          assert(partitions(1).files.size == 2)
-          assert(partitions(2).files.size == 1)
+          assert(partitions(0).files.length == 1)
+          assert(partitions(1).files.length == 2)
+          assert(partitions(2).files.length == 1)
         }
       }
     }
@@ -480,8 +482,8 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
         val e = intercept[SparkException] {
           spark.read.text(inputFile.toURI.toString).collect()
         }
-        assert(e.getCause.getCause.isInstanceOf[EOFException])
-        assert(e.getCause.getCause.getMessage === "Unexpected end of input stream")
+        assert(e.getCause.isInstanceOf[EOFException])
+        assert(e.getCause.getMessage === "Unexpected end of input stream")
       }
       withSQLConf(SQLConf.IGNORE_CORRUPT_FILES.key -> "true") {
         assert(spark.read.text(inputFile.toURI.toString).collect().isEmpty)
@@ -690,7 +692,7 @@ class FileSourceStrategySuite extends QueryTest with SharedSparkSession {
 
     if (buckets > 0) {
       val bucketed = df.queryExecution.analyzed transform {
-        case l @ LogicalRelation(r: HadoopFsRelation, _, _, _) =>
+        case l @ LogicalRelationWithTable(r: HadoopFsRelation, _) =>
           l.copy(relation =
             r.copy(bucketSpec =
               Some(BucketSpec(numBuckets = buckets, "c1" :: Nil, Nil)))(r.sparkSession))
