@@ -658,7 +658,6 @@ abstract class SchemaPruningSuite
             |where not exists (select null from employees e where e.name.first = c.name.first
             |  and e.employer.name = c.employer.company.name)
             |""".stripMargin)
-        // TODO: SPARK-51381: Fix the schema pruning for nested columns
         checkScan(query,
           "struct<name:struct<first:string,middle:string,last:string>," +
             "employer:struct<id:int,company:struct<name:string,address:string>>>",
@@ -669,21 +668,6 @@ abstract class SchemaPruningSuite
     }
   }
 
-  testSchemaPruning("SPARK-51831: Column pruning with exists Join") {
-    withContacts {
-      val query = sql(
-        """
-          |select sum(t1.id) as sum_id
-          |from contacts as t1
-          |where exists(select * from contacts as t2 where t1.id == t2.id)
-          |""".stripMargin)
-      checkScan(query,
-        "struct<id:int>",
-        "struct<id:int>")
-      checkAnswer(query, Row(6))
-    }
-  }
-
   protected def testSchemaPruning(testName: String)(testThunk: => Unit): Unit = {
     test(s"Spark vectorized reader - without partition data column - $testName") {
       withSQLConf(vectorizedReaderEnabledKey -> "true") {
@@ -982,7 +966,7 @@ abstract class SchemaPruningSuite
             |  {"itemId": 2, "itemData": "b"}
             |]}
             |""".stripMargin
-        val df = spark.read.json(Seq(jsonStr).toDS)
+        val df = spark.read.json(Seq(jsonStr).toDS())
         makeDataSourceFile(df, new File(path))
 
         spark.read.format(dataSourceName).load(path)
