@@ -21,28 +21,29 @@ import java.net.URI
 import java.util
 import java.util.Collections
 
-import scala.collection.JavaConverters._
+import scala.jdk.CollectionConverters._
 
 import org.apache.hadoop.fs.Path
 import org.scalatest.BeforeAndAfter
 
+import org.apache.spark.{SparkIllegalArgumentException, SparkUnsupportedOperationException}
 import org.apache.spark.sql.AnalysisException
-import org.apache.spark.sql.catalyst.analysis.{NamespaceAlreadyExistsException, NoSuchDatabaseException, NoSuchNamespaceException, NoSuchTableException, TableAlreadyExistsException}
+import org.apache.spark.sql.catalyst.analysis.{NamespaceAlreadyExistsException, NoSuchNamespaceException, NoSuchTableException, TableAlreadyExistsException}
 import org.apache.spark.sql.catalyst.parser.CatalystSqlParser
 import org.apache.spark.sql.catalyst.util.quoteIdentifier
-import org.apache.spark.sql.connector.catalog.{CatalogV2Util, Identifier, NamespaceChange, SupportsNamespaces, TableCatalog, TableChange, V1Table}
+import org.apache.spark.sql.connector.catalog.{CatalogV2Util, Column, Identifier, NamespaceChange, SupportsNamespaces, TableCatalog, TableChange, V1Table}
 import org.apache.spark.sql.connector.expressions.Transform
 import org.apache.spark.sql.test.SharedSparkSession
-import org.apache.spark.sql.types.{DoubleType, IntegerType, LongType, StringType, StructField, StructType, TimestampType}
+import org.apache.spark.sql.types.{DoubleType, IntegerType, LongType, StringType, StructType, TimestampType}
 import org.apache.spark.sql.util.CaseInsensitiveStringMap
 
 abstract class V2SessionCatalogBaseSuite extends SharedSparkSession with BeforeAndAfter {
 
   val emptyProps: util.Map[String, String] = Collections.emptyMap[String, String]
   val emptyTrans: Array[Transform] = Array.empty
-  val schema: StructType = new StructType()
-      .add("id", IntegerType)
-      .add("data", StringType)
+  val columns: Array[Column] = Array(
+    Column.create("id", IntegerType),
+    Column.create("data", StringType))
 
   val testNs: Array[String] = Array("db")
   val defaultNs: Array[String] = Array("default")
@@ -97,13 +98,13 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
 
     assert(catalog.listTables(Array("ns")).isEmpty)
 
-    catalog.createTable(ident1, schema, emptyTrans, emptyProps)
+    catalog.createTable(ident1, columns, emptyTrans, emptyProps)
 
     assert(catalog.listTables(Array("ns")).toSet == Set(ident1))
     assert(catalog.listTables(Array("ns2")).isEmpty)
 
-    catalog.createTable(ident3, schema, emptyTrans, emptyProps)
-    catalog.createTable(ident2, schema, emptyTrans, emptyProps)
+    catalog.createTable(ident3, columns, emptyTrans, emptyProps)
+    catalog.createTable(ident2, columns, emptyTrans, emptyProps)
 
     assert(catalog.listTables(Array("ns")).toSet == Set(ident1, ident2))
     assert(catalog.listTables(Array("ns2")).toSet == Set(ident3))
@@ -125,12 +126,12 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
 
     assert(!catalog.tableExists(testIdent))
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
     val parsed = CatalystSqlParser.parseMultipartIdentifier(table.name)
     assert(parsed == Seq("db", "test_table"))
-    assert(table.schema == schema)
+    assert(table.columns === columns)
     assert(filterV2TableProperties(table.properties) == Map())
 
     assert(catalog.tableExists(testIdent))
@@ -144,12 +145,12 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
 
     assert(!catalog.tableExists(testIdent))
 
-    catalog.createTable(testIdent, schema, emptyTrans, properties)
+    catalog.createTable(testIdent, columns, emptyTrans, properties)
     val table = catalog.loadTable(testIdent)
 
     val parsed = CatalystSqlParser.parseMultipartIdentifier(table.name)
     assert(parsed == Seq("db", "test_table"))
-    assert(table.schema == schema)
+    assert(table.columns === columns)
     assert(filterV2TableProperties(table.properties).asJava == properties)
 
     assert(catalog.tableExists(testIdent))
@@ -160,14 +161,14 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
 
     assert(!catalog.tableExists(testIdent))
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
     val parsed = CatalystSqlParser.parseMultipartIdentifier(table.name)
       .map(part => quoteIdentifier(part)).mkString(".")
 
     val exc = intercept[TableAlreadyExistsException] {
-      catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+      catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     }
 
     checkErrorTableAlreadyExists(exc, parsed)
@@ -188,7 +189,7 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
     assert(!catalog.tableExists(testIdent))
 
     // default location
-    catalog.createTable(testIdent, schema, emptyTrans, properties)
+    catalog.createTable(testIdent, columns, emptyTrans, properties)
     val t1 = catalog.loadTable(testIdent).asInstanceOf[V1Table]
     assert(t1.catalogTable.location ===
       spark.sessionState.catalog.defaultTablePath(testIdent.asTableIdentifier))
@@ -196,21 +197,21 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
 
     // relative path
     properties.put(TableCatalog.PROP_LOCATION, "relative/path")
-    catalog.createTable(testIdent, schema, emptyTrans, properties)
+    catalog.createTable(testIdent, columns, emptyTrans, properties)
     val t2 = catalog.loadTable(testIdent).asInstanceOf[V1Table]
     assert(t2.catalogTable.location === makeQualifiedPathWithWarehouse("db.db/relative/path"))
     catalog.dropTable(testIdent)
 
     // absolute path without scheme
     properties.put(TableCatalog.PROP_LOCATION, "/absolute/path")
-    catalog.createTable(testIdent, schema, emptyTrans, properties)
+    catalog.createTable(testIdent, columns, emptyTrans, properties)
     val t3 = catalog.loadTable(testIdent).asInstanceOf[V1Table]
     assert(t3.catalogTable.location.toString === "file:///absolute/path")
     catalog.dropTable(testIdent)
 
     // absolute path with scheme
     properties.put(TableCatalog.PROP_LOCATION, "file:/absolute/path")
-    catalog.createTable(testIdent, schema, emptyTrans, properties)
+    catalog.createTable(testIdent, columns, emptyTrans, properties)
     val t4 = catalog.loadTable(testIdent).asInstanceOf[V1Table]
     assert(t4.catalogTable.location.toString === "file:/absolute/path")
     catalog.dropTable(testIdent)
@@ -221,7 +222,7 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
 
     assert(!catalog.tableExists(testIdent))
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
 
     assert(catalog.tableExists(testIdent))
 
@@ -233,11 +234,11 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
   test("loadTable") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val loaded = catalog.loadTable(testIdent)
 
     assert(loaded.name == testIdent.toString)
-    assert(loaded.schema == schema)
+    assert(loaded.columns === columns)
   }
 
   test("loadTable: table does not exist") {
@@ -253,7 +254,7 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
   test("invalidateTable") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
     catalog.invalidateTable(testIdent)
 
@@ -275,7 +276,7 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
   test("alterTable: add property") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
     assert(filterV2TableProperties(table.properties) == Map())
@@ -296,7 +297,7 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
     val properties = new util.HashMap[String, String]()
     properties.put("prop-1", "1")
 
-    catalog.createTable(testIdent, schema, emptyTrans, properties)
+    catalog.createTable(testIdent, columns, emptyTrans, properties)
     val table = catalog.loadTable(testIdent)
 
     assert(filterV2TableProperties(table.properties) == Map("prop-1" -> "1"))
@@ -317,7 +318,7 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
     val properties = new util.HashMap[String, String]()
     properties.put("prop-1", "1")
 
-    catalog.createTable(testIdent, schema, emptyTrans, properties)
+    catalog.createTable(testIdent, columns, emptyTrans, properties)
     val table = catalog.loadTable(testIdent)
 
     assert(filterV2TableProperties(table.properties) == Map("prop-1" -> "1"))
@@ -335,7 +336,7 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
   test("alterTable: remove missing property") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
     assert(filterV2TableProperties(table.properties) == Map())
@@ -353,75 +354,75 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
   test("alterTable: add top-level column") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
     catalog.alterTable(testIdent, TableChange.addColumn(Array("ts"), TimestampType))
     val updated = catalog.loadTable(testIdent)
 
-    assert(updated.schema == schema.add("ts", TimestampType))
+    assert(updated.columns() === columns :+ Column.create("ts", TimestampType))
   }
 
   test("alterTable: add required column") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
     catalog.alterTable(testIdent,
       TableChange.addColumn(Array("ts"), TimestampType, false))
     val updated = catalog.loadTable(testIdent)
 
-    assert(updated.schema == schema.add("ts", TimestampType, nullable = false))
+    assert(updated.columns() === columns :+ Column.create("ts", TimestampType, false))
   }
 
   test("alterTable: add column with comment") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
     catalog.alterTable(testIdent,
       TableChange.addColumn(Array("ts"), TimestampType, false, "comment text"))
     val updated = catalog.loadTable(testIdent)
 
-    val field = StructField("ts", TimestampType, nullable = false).withComment("comment text")
-    assert(updated.schema == schema.add(field))
+    val tsColumn = Column.create("ts", TimestampType, false, "comment text", null)
+    assert(updated.columns === columns :+ tsColumn)
   }
 
   test("alterTable: add nested column") {
     val catalog = newCatalog()
 
     val pointStruct = new StructType().add("x", DoubleType).add("y", DoubleType)
-    val tableSchema = schema.add("point", pointStruct)
+    val tableColumns = columns :+ Column.create("point", pointStruct)
 
-    catalog.createTable(testIdent, tableSchema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, tableColumns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == tableSchema)
+    assert(table.columns === tableColumns)
 
     catalog.alterTable(testIdent,
       TableChange.addColumn(Array("point", "z"), DoubleType))
     val updated = catalog.loadTable(testIdent)
 
-    val expectedSchema = schema.add("point", pointStruct.add("z", DoubleType))
+    val expectedColumns = columns :+ Column.create("point", pointStruct.add("z", DoubleType))
 
-    assert(updated.schema == expectedSchema)
+    assert(updated.columns === expectedColumns)
   }
 
   test("alterTable: add column to primitive field fails") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
     val exc = intercept[IllegalArgumentException] {
       catalog.alterTable(testIdent, TableChange.addColumn(Array("data", "ts"), TimestampType))
@@ -431,33 +432,33 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
     assert(exc.getMessage.contains("data"))
 
     // the table has not changed
-    assert(catalog.loadTable(testIdent).schema == schema)
+    assert(catalog.loadTable(testIdent).columns === columns)
   }
 
   test("alterTable: add field to missing column fails") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
-    val exc = intercept[IllegalArgumentException] {
-      catalog.alterTable(testIdent,
-        TableChange.addColumn(Array("missing_col", "new_field"), StringType))
-    }
-
-    assert(exc.getMessage.contains("missing_col"))
-    assert(exc.getMessage.contains("Cannot find"))
+    checkError(
+      exception = intercept[SparkIllegalArgumentException] {
+        catalog.alterTable(testIdent,
+          TableChange.addColumn(Array("missing_col", "new_field"), StringType))
+      },
+      condition = "FIELD_NOT_FOUND",
+      parameters = Map("fieldName" -> "`missing_col`", "fields" -> "`id`, `data`"))
   }
 
   test("alterTable: update column data type") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
     catalog.alterTable(testIdent, TableChange.updateColumnType(Array("id"), LongType))
     val updated = catalog.loadTable(testIdent)
@@ -469,46 +470,50 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
   test("alterTable: update column nullability") {
     val catalog = newCatalog()
 
-    val originalSchema = new StructType()
-        .add("id", IntegerType, nullable = false)
-        .add("data", StringType)
-    catalog.createTable(testIdent, originalSchema, emptyTrans, emptyProps)
+    val originalColumns: Array[Column] = Array(
+      Column.create("id", IntegerType, false),
+      Column.create("data", StringType))
+    catalog.createTable(testIdent, originalColumns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == originalSchema)
+    assert(table.columns === originalColumns)
 
     catalog.alterTable(testIdent,
       TableChange.updateColumnNullability(Array("id"), true))
     val updated = catalog.loadTable(testIdent)
 
     val expectedSchema = new StructType().add("id", IntegerType).add("data", StringType)
-    assert(updated.schema == expectedSchema)
+    val expectedColumns: Array[Column] = Array(
+      Column.create("id", IntegerType),
+      Column.create("data", StringType)
+    )
+    assert(updated.columns === expectedColumns)
   }
 
   test("alterTable: update missing column fails") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
-    val exc = intercept[IllegalArgumentException] {
-      catalog.alterTable(testIdent,
-        TableChange.updateColumnType(Array("missing_col"), LongType))
-    }
-
-    assert(exc.getMessage.contains("missing_col"))
-    assert(exc.getMessage.contains("Cannot find"))
+    checkError(
+      exception = intercept[SparkIllegalArgumentException] {
+        catalog.alterTable(testIdent,
+          TableChange.updateColumnType(Array("missing_col"), LongType))
+      },
+      condition = "FIELD_NOT_FOUND",
+      parameters = Map("fieldName" -> "`missing_col`", "fields" -> "`id`, `data`"))
   }
 
   test("alterTable: add comment") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
     catalog.alterTable(testIdent,
       TableChange.updateColumnComment(Array("id"), "comment text"))
@@ -523,10 +528,10 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
   test("alterTable: replace comment") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
     catalog.alterTable(testIdent, TableChange.updateColumnComment(Array("id"), "comment text"))
 
@@ -544,27 +549,27 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
   test("alterTable: add comment to missing column fails") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
-    val exc = intercept[IllegalArgumentException] {
-      catalog.alterTable(testIdent,
-        TableChange.updateColumnComment(Array("missing_col"), "comment"))
-    }
-
-    assert(exc.getMessage.contains("missing_col"))
-    assert(exc.getMessage.contains("Cannot find"))
+    checkError(
+      exception = intercept[SparkIllegalArgumentException] {
+        catalog.alterTable(testIdent,
+          TableChange.updateColumnComment(Array("missing_col"), "comment"))
+      },
+      condition = "FIELD_NOT_FOUND",
+      parameters = Map("fieldName" -> "`missing_col`", "fields" -> "`id`, `data`"))
   }
 
   test("alterTable: rename top-level column") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
     catalog.alterTable(testIdent, TableChange.renameColumn(Array("id"), "some_id"))
     val updated = catalog.loadTable(testIdent)
@@ -578,70 +583,70 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
     val catalog = newCatalog()
 
     val pointStruct = new StructType().add("x", DoubleType).add("y", DoubleType)
-    val tableSchema = schema.add("point", pointStruct)
+    val tableColumns = columns :+ Column.create("point", pointStruct)
 
-    catalog.createTable(testIdent, tableSchema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, tableColumns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == tableSchema)
+    assert(table.columns === tableColumns)
 
     catalog.alterTable(testIdent,
       TableChange.renameColumn(Array("point", "x"), "first"))
     val updated = catalog.loadTable(testIdent)
 
     val newPointStruct = new StructType().add("first", DoubleType).add("y", DoubleType)
-    val expectedSchema = schema.add("point", newPointStruct)
+    val expectedColumns = columns :+ Column.create("point", newPointStruct)
 
-    assert(updated.schema == expectedSchema)
+    assert(updated.columns === expectedColumns)
   }
 
   test("alterTable: rename struct column") {
     val catalog = newCatalog()
 
     val pointStruct = new StructType().add("x", DoubleType).add("y", DoubleType)
-    val tableSchema = schema.add("point", pointStruct)
+    val tableColumns = columns :+ Column.create("point", pointStruct)
 
-    catalog.createTable(testIdent, tableSchema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, tableColumns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == tableSchema)
+    assert(table.columns === tableColumns)
 
     catalog.alterTable(testIdent, TableChange.renameColumn(Array("point"), "p"))
     val updated = catalog.loadTable(testIdent)
 
     val newPointStruct = new StructType().add("x", DoubleType).add("y", DoubleType)
-    val expectedSchema = schema.add("p", newPointStruct)
+    val expectedColumns = columns :+ Column.create("p", newPointStruct)
 
-    assert(updated.schema == expectedSchema)
+    assert(updated.columns === expectedColumns)
   }
 
   test("alterTable: rename missing column fails") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
-    val exc = intercept[IllegalArgumentException] {
-      catalog.alterTable(testIdent,
-        TableChange.renameColumn(Array("missing_col"), "new_name"))
-    }
-
-    assert(exc.getMessage.contains("missing_col"))
-    assert(exc.getMessage.contains("Cannot find"))
+    checkError(
+      exception = intercept[SparkIllegalArgumentException] {
+        catalog.alterTable(testIdent,
+          TableChange.renameColumn(Array("missing_col"), "new_name"))
+      },
+      condition = "FIELD_NOT_FOUND",
+      parameters = Map("fieldName" -> "`missing_col`", "fields" -> "`id`, `data`"))
   }
 
   test("alterTable: multiple changes") {
     val catalog = newCatalog()
 
     val pointStruct = new StructType().add("x", DoubleType).add("y", DoubleType)
-    val tableSchema = schema.add("point", pointStruct)
+    val tableColumns = columns :+ Column.create("point", pointStruct)
 
-    catalog.createTable(testIdent, tableSchema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, tableColumns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == tableSchema)
+    assert(table.columns === tableColumns)
 
     catalog.alterTable(testIdent,
       TableChange.renameColumn(Array("point", "x"), "first"),
@@ -649,18 +654,18 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
     val updated = catalog.loadTable(testIdent)
 
     val newPointStruct = new StructType().add("first", DoubleType).add("second", DoubleType)
-    val expectedSchema = schema.add("point", newPointStruct)
+    val expectedColumns = columns :+ Column.create("point", newPointStruct)
 
-    assert(updated.schema == expectedSchema)
+    assert(updated.columns === expectedColumns)
   }
 
   test("alterTable: delete top-level column") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
     catalog.alterTable(testIdent, TableChange.deleteColumn(Array("id"), false))
     val updated = catalog.loadTable(testIdent)
@@ -673,63 +678,63 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
     val catalog = newCatalog()
 
     val pointStruct = new StructType().add("x", DoubleType).add("y", DoubleType)
-    val tableSchema = schema.add("point", pointStruct)
+    val tableColumns = columns :+ Column.create("point", pointStruct)
 
-    catalog.createTable(testIdent, tableSchema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, tableColumns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == tableSchema)
+    assert(table.columns === tableColumns)
 
     catalog.alterTable(testIdent, TableChange.deleteColumn(Array("point", "y"), false))
     val updated = catalog.loadTable(testIdent)
 
     val newPointStruct = new StructType().add("x", DoubleType)
-    val expectedSchema = schema.add("point", newPointStruct)
+    val expectedColumns = columns :+ Column.create("point", newPointStruct)
 
-    assert(updated.schema == expectedSchema)
+    assert(updated.columns === expectedColumns)
   }
 
   test("alterTable: delete missing column fails") {
     val catalog = newCatalog()
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == schema)
+    assert(table.columns === columns)
 
-    val exc = intercept[IllegalArgumentException] {
-      catalog.alterTable(testIdent, TableChange.deleteColumn(Array("missing_col"), false))
-    }
-
-    assert(exc.getMessage.contains("missing_col"))
-    assert(exc.getMessage.contains("Cannot find"))
+    checkError(
+      exception = intercept[SparkIllegalArgumentException] {
+        catalog.alterTable(testIdent, TableChange.deleteColumn(Array("missing_col"), false))
+      },
+      condition = "FIELD_NOT_FOUND",
+      parameters = Map("fieldName" -> "`missing_col`", "fields" -> "`id`, `data`"))
 
     // with if exists it should pass
     catalog.alterTable(testIdent, TableChange.deleteColumn(Array("missing_col"), true))
-    assert(table.schema == schema)
+    assert(table.columns === columns)
   }
 
   test("alterTable: delete missing nested column fails") {
     val catalog = newCatalog()
 
     val pointStruct = new StructType().add("x", DoubleType).add("y", DoubleType)
-    val tableSchema = schema.add("point", pointStruct)
+    val tableColumns = columns :+ Column.create("point", pointStruct)
 
-    catalog.createTable(testIdent, tableSchema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, tableColumns, emptyTrans, emptyProps)
     val table = catalog.loadTable(testIdent)
 
-    assert(table.schema == tableSchema)
+    assert(table.columns === tableColumns)
 
-    val exc = intercept[IllegalArgumentException] {
-      catalog.alterTable(testIdent, TableChange.deleteColumn(Array("point", "z"), false))
-    }
-
-    assert(exc.getMessage.contains("z"))
-    assert(exc.getMessage.contains("Cannot find"))
+    checkError(
+      exception = intercept[SparkIllegalArgumentException] {
+        catalog.alterTable(testIdent, TableChange.deleteColumn(Array("point", "z"), false))
+      },
+      condition = "FIELD_NOT_FOUND",
+      parameters = Map("fieldName" -> "`z`", "fields" -> "`x`, `y`"))
 
     // with if exists it should pass
     catalog.alterTable(testIdent, TableChange.deleteColumn(Array("point", "z"), true))
-    assert(table.schema == tableSchema)
+    assert(table.columns === tableColumns)
   }
 
   test("alterTable: table does not exist") {
@@ -747,7 +752,7 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
     assert(!catalog.tableExists(testIdent))
 
     // default location
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     val t1 = catalog.loadTable(testIdent).asInstanceOf[V1Table]
     assert(t1.catalogTable.location ===
       spark.sessionState.catalog.defaultTablePath(testIdent.asTableIdentifier))
@@ -776,7 +781,7 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
 
     assert(!catalog.tableExists(testIdent))
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
 
     assert(catalog.tableExists(testIdent))
 
@@ -803,7 +808,7 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
     assert(!catalog.tableExists(testIdent))
     assert(!catalog.tableExists(testIdentNew))
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
 
     assert(catalog.tableExists(testIdent))
     catalog.renameTable(testIdent, testIdentNew)
@@ -828,8 +833,8 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
     assert(!catalog.tableExists(testIdent))
     assert(!catalog.tableExists(testIdentNew))
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
-    catalog.createTable(testIdentNew, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
+    catalog.createTable(testIdentNew, columns, emptyTrans, emptyProps)
 
     assert(catalog.tableExists(testIdent))
     assert(catalog.tableExists(testIdentNew))
@@ -848,7 +853,7 @@ class V2SessionCatalogTableSuite extends V2SessionCatalogBaseSuite {
     assert(!catalog.tableExists(testIdent))
     assert(!catalog.tableExists(testIdentNewOtherDb))
 
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
 
     assert(catalog.tableExists(testIdent))
 
@@ -878,7 +883,7 @@ class V2SessionCatalogNamespaceSuite extends V2SessionCatalogBaseSuite {
     // remove location and comment that are automatically added by HMS unless they are expected
     val toRemove =
       CatalogV2Util.NAMESPACE_RESERVED_PROPERTIES.filter(expected.contains)
-    assert(expected -- toRemove === actual)
+    assert(expected.toMap -- toRemove === actual)
   }
 
   test("listNamespaces: basic behavior") {
@@ -948,7 +953,7 @@ class V2SessionCatalogNamespaceSuite extends V2SessionCatalogBaseSuite {
   test("createNamespace: basic behavior") {
     val catalog = newCatalog()
 
-    val sessionCatalog = sqlContext.sessionState.catalog
+    val sessionCatalog = spark.sessionState.catalog
     val expectedPath =
       new Path(spark.sessionState.conf.warehousePath,
         sessionCatalog.getDefaultDBPath(testNs(0)).toString).toString
@@ -1034,8 +1039,8 @@ class V2SessionCatalogNamespaceSuite extends V2SessionCatalogBaseSuite {
 
     assert(catalog.namespaceExists(testNs) === false)
 
-    val exc = intercept[NoSuchDatabaseException] {
-      catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    val exc = intercept[NoSuchNamespaceException] {
+      catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
     }
 
     assert(exc.getMessage.contains(testNs.quoted))
@@ -1069,7 +1074,7 @@ class V2SessionCatalogNamespaceSuite extends V2SessionCatalogBaseSuite {
     val catalog = newCatalog()
 
     catalog.createNamespace(testNs, Map("property" -> "value").asJava)
-    catalog.createTable(testIdent, schema, emptyTrans, emptyProps)
+    catalog.createTable(testIdent, columns, emptyTrans, emptyProps)
 
     val exc = intercept[AnalysisException] {
       catalog.dropNamespace(testNs, cascade = false)
@@ -1151,7 +1156,7 @@ class V2SessionCatalogNamespaceSuite extends V2SessionCatalogBaseSuite {
 
     assert(catalog.namespaceExists(testNs) === false)
 
-    val exc = intercept[NoSuchDatabaseException] {
+    val exc = intercept[NoSuchNamespaceException] {
       catalog.alterNamespace(testNs, NamespaceChange.setProperty("property", "value"))
     }
 
@@ -1164,10 +1169,12 @@ class V2SessionCatalogNamespaceSuite extends V2SessionCatalogBaseSuite {
     catalog.createNamespace(testNs, emptyProps)
 
     CatalogV2Util.NAMESPACE_RESERVED_PROPERTIES.foreach { p =>
-      val exc = intercept[UnsupportedOperationException] {
-        catalog.alterNamespace(testNs, NamespaceChange.removeProperty(p))
-      }
-      assert(exc.getMessage.contains(s"Cannot remove reserved property: $p"))
+      checkError(
+        exception = intercept[SparkUnsupportedOperationException] {
+          catalog.alterNamespace(testNs, NamespaceChange.removeProperty(p))
+        },
+        condition = "CANNOT_REMOVE_RESERVED_PROPERTY",
+        parameters = Map("property" -> p))
 
     }
     catalog.dropNamespace(testNs, cascade = false)
@@ -1177,7 +1184,7 @@ class V2SessionCatalogNamespaceSuite extends V2SessionCatalogBaseSuite {
     val testIdent: IdentifierHelper = Identifier.of(Array("a", "b"), "c")
     checkError(
       exception = intercept[AnalysisException](testIdent.asTableIdentifier),
-      errorClass = "IDENTIFIER_TOO_MANY_NAME_PARTS",
+      condition = "IDENTIFIER_TOO_MANY_NAME_PARTS",
       parameters = Map("identifier" -> "`a`.`b`.`c`")
     )
   }
@@ -1186,7 +1193,7 @@ class V2SessionCatalogNamespaceSuite extends V2SessionCatalogBaseSuite {
     val testIdent: MultipartIdentifierHelper = Seq("a", "b", "c")
     checkError(
       exception = intercept[AnalysisException](testIdent.asFunctionIdentifier),
-      errorClass = "IDENTIFIER_TOO_MANY_NAME_PARTS",
+      condition = "IDENTIFIER_TOO_MANY_NAME_PARTS",
       parameters = Map("identifier" -> "`a`.`b`.`c`")
     )
   }
