@@ -19,10 +19,10 @@ package org.apache.spark.sql.execution.ui
 
 import java.util
 import java.util.{Locale, Properties}
-import javax.servlet.http.HttpServletRequest
 
 import scala.xml.Node
 
+import jakarta.servlet.http.HttpServletRequest
 import org.mockito.Mockito.{mock, when, RETURNS_SMART_NULLS}
 import org.scalatest.BeforeAndAfter
 
@@ -275,7 +275,7 @@ class AllExecutionsPageWithInMemoryStoreSuite extends AllExecutionsPageSuite {
 @SlowSQLTest
 class AllExecutionsPageWithRocksDBBackendSuite extends AllExecutionsPageSuite {
   private val storePath = Utils.createTempDir()
-  override protected def createStatusStore(): SQLAppStatusStore = {
+  override protected def createStatusStore: SQLAppStatusStore = {
     val conf = sparkContext.conf
     conf.set(LIVE_UI_LOCAL_STORE_DIR, storePath.getCanonicalPath)
     val appStatusStore = AppStatusStore.createLiveStore(conf)
