@@ -19,10 +19,9 @@ package org.apache.spark.sql.streaming
 
 import org.apache.spark.sql.IntegratedUDFTestUtils._
 import org.apache.spark.sql.Row
-import org.apache.spark.sql.catalyst.expressions.PythonUDF
 import org.apache.spark.sql.catalyst.plans.logical.{NoTimeout, ProcessingTimeTimeout}
 import org.apache.spark.sql.catalyst.streaming.InternalOutputModes.{Complete, Update}
-import org.apache.spark.sql.execution.python.FlatMapGroupsInPandasWithStateExec
+import org.apache.spark.sql.execution.python.streaming.FlatMapGroupsInPandasWithStateExec
 import org.apache.spark.sql.execution.streaming.MemoryStream
 import org.apache.spark.sql.functions.{lit, timestamp_seconds}
 import org.apache.spark.sql.internal.SQLConf
@@ -87,7 +86,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
       inputDataDS
         .groupBy("value")
         .applyInPandasWithState(
-          pythonFunc(inputDataDS("value")).expr.asInstanceOf[PythonUDF],
+          pythonFunc(inputDataDS("value")),
           outputStructType,
           stateStructType,
           "Update",
@@ -161,7 +160,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
         inputDataDS
           .groupBy("key")
           .applyInPandasWithState(
-            pythonFunc(inputDataDS("key"), inputDataDS("value")).expr.asInstanceOf[PythonUDF],
+            pythonFunc(inputDataDS("key"), inputDataDS("value")),
             outputStructType,
             stateStructType,
             "Update",
@@ -224,7 +223,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
       name = "pandas_grouped_map_with_state", pythonScript = pythonScript)
 
     val inputData = MemoryStream[String]
-    val inputDataDS = inputData.toDS
+    val inputDataDS = inputData.toDS()
     val outputStructType = StructType(
       Seq(
         StructField("key", StringType),
@@ -234,7 +233,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
       inputDataDS
         .groupBy("value")
         .applyInPandasWithState(
-          pythonFunc(inputDataDS("value")).expr.asInstanceOf[PythonUDF],
+          pythonFunc(inputDataDS("value")),
           outputStructType,
           stateStructType,
           "Append",
@@ -308,7 +307,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
 
     val clock = new StreamManualClock
     val inputData = MemoryStream[String]
-    val inputDataDS = inputData.toDS
+    val inputDataDS = inputData.toDS()
     val outputStructType = StructType(
       Seq(
         StructField("key", StringType),
@@ -318,7 +317,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
       inputDataDS
         .groupBy("value")
         .applyInPandasWithState(
-          pythonFunc(inputDataDS("value")).expr.asInstanceOf[PythonUDF],
+          pythonFunc(inputDataDS("value")),
           outputStructType,
           stateStructType,
           "Update",
@@ -382,42 +381,42 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
 
         val pythonScript =
           s"""
-             |import calendar
-             |import os
-             |import datetime
-             |import pandas as pd
-             |from pyspark.sql.types import StructType, StringType, StructField, IntegerType
-             |
-             |tpe = StructType([
-             |    StructField("key", StringType()),
-             |    StructField("maxEventTimeSec", IntegerType())])
-             |
-             |def func(key, pdf_iter, state):
-             |    assert state.getCurrentProcessingTimeMs() >= 0
-             |    assert state.getCurrentWatermarkMs() >= -1
-             |
-             |    timeout_delay_sec = 5
-             |    if state.hasTimedOut:
-             |        state.remove()
-             |        yield pd.DataFrame({'key': [key[0]], 'maxEventTimeSec': [-1]})
-             |    else:
-             |        m = state.getOption
-             |        if m is None:
-             |            max_event_time_sec = 0
-             |        else:
-             |            max_event_time_sec = m[0]
-             |
-             |        for pdf in pdf_iter:
-             |            pser = pdf.eventTime.apply(
-             |                lambda dt: (int(calendar.timegm(dt.utctimetuple()) + dt.microsecond)))
-             |            max_event_time_sec = int(max(pser.max(), max_event_time_sec))
-             |
-             |        state.update((max_event_time_sec,))
-             |        timeout_timestamp_sec = max_event_time_sec + timeout_delay_sec
-             |        state.setTimeoutTimestamp($timeoutMs)
-             |        yield pd.DataFrame({'key': [key[0]],
-             |                            'maxEventTimeSec': [max_event_time_sec]})
-             |""".stripMargin.format("")
+            |import calendar
+            |import os
+            |import datetime
+            |import pandas as pd
+            |from pyspark.sql.types import StructType, StringType, StructField, IntegerType
+            |
+            |tpe = StructType([
+            |    StructField("key", StringType()),
+            |    StructField("maxEventTimeSec", IntegerType())])
+            |
+            |def func(key, pdf_iter, state):
+            |    assert state.getCurrentProcessingTimeMs() >= 0
+            |    assert state.getCurrentWatermarkMs() >= -1
+            |
+            |    timeout_delay_sec = 5
+            |    if state.hasTimedOut:
+            |        state.remove()
+            |        yield pd.DataFrame({'key': [key[0]], 'maxEventTimeSec': [-1]})
+            |    else:
+            |        m = state.getOption
+            |        if m is None:
+            |            max_event_time_sec = 0
+            |        else:
+            |            max_event_time_sec = m[0]
+            |
+            |        for pdf in pdf_iter:
+            |            pser = pdf.eventTime.apply(
+            |                lambda dt: (int(calendar.timegm(dt.utctimetuple()) + dt.microsecond)))
+            |            max_event_time_sec = int(max(pser.max(), max_event_time_sec))
+            |
+            |        state.update((max_event_time_sec,))
+            |        timeout_timestamp_sec = max_event_time_sec + timeout_delay_sec
+            |        state.setTimeoutTimestamp($timeoutMs)
+            |        yield pd.DataFrame({'key': [key[0]],
+            |                            'maxEventTimeSec': [max_event_time_sec]})
+            |""".stripMargin.format("")
         val pythonFunc = TestGroupedMapPandasUDFWithState(
           name = "pandas_grouped_map_with_state", pythonScript = pythonScript)
 
@@ -434,7 +433,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
             .withWatermark("eventTime", "10 seconds")
             .groupBy("key")
             .applyInPandasWithState(
-              pythonFunc(inputDataDF("key"), inputDataDF("eventTime")).expr.asInstanceOf[PythonUDF],
+              pythonFunc(inputDataDF("key"), inputDataDF("eventTime")),
               outputStructType,
               stateStructType,
               "Update",
@@ -507,7 +506,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
         val clock = new StreamManualClock
         val inputData = MemoryStream[(String, Long)]
         val inputDataDF = inputData
-          .toDF.toDF("key", "time")
+          .toDF().toDF("key", "time")
           .selectExpr("key", "timestamp_seconds(time) as timestamp")
         val outputStructType = StructType(
           Seq(
@@ -519,7 +518,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
             .withWatermark("timestamp", "10 second")
             .groupBy("key")
             .applyInPandasWithState(
-              pythonFunc(inputDataDF("key"), inputDataDF("timestamp")).expr.asInstanceOf[PythonUDF],
+              pythonFunc(inputDataDF("key"), inputDataDF("timestamp")),
               outputStructType,
               stateStructType,
               "Update",
@@ -589,7 +588,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
       inputDataDS
         .groupBy("value")
         .applyInPandasWithState(
-          pythonFunc(inputDataDS("value")).expr.asInstanceOf[PythonUDF],
+          pythonFunc(inputDataDS("value")),
           outputStructType,
           stateStructType,
           "Update",
@@ -656,7 +655,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
           inputDataDS
             .groupBy("key")
             .applyInPandasWithState(
-              pythonFunc(inputDataDS("key")).expr.asInstanceOf[PythonUDF],
+              pythonFunc(inputDataDS("key")),
               outputStructType,
               stateStructType,
               "Update",
@@ -713,7 +712,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
           inputDataDS
             .groupBy("key")
             .applyInPandasWithState(
-              pythonFunc(inputDataDS("key")).expr.asInstanceOf[PythonUDF],
+              pythonFunc(inputDataDS("key")),
               outputStructType,
               stateStructType,
               "Update",
@@ -772,7 +771,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
 
     val clock = new StreamManualClock
     val inputData = MemoryStream[String]
-    val inputDataDS = inputData.toDS
+    val inputDataDS = inputData.toDS()
       .withColumnRenamed("value", "key1")
       // the type of columns with string literal will be non-nullable
       .withColumn("key2", lit("__FAKE__"))
@@ -789,8 +788,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
         .groupBy("key1", "key2")
         .applyInPandasWithState(
           pythonFunc(
-            inputDataDS("key1"), inputDataDS("key2"), inputDataDS("val1"), inputDataDS("val2")
-          ).expr.asInstanceOf[PythonUDF],
+            inputDataDS("key1"), inputDataDS("key2"), inputDataDS("val1"), inputDataDS("val2")),
           outputStructType,
           stateStructType,
           "Update",
@@ -857,7 +855,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
     val clock = new StreamManualClock
     val inputData = MemoryStream[String]
     // schema: val1, key2, val2, key1, val3
-    val inputDataDS = inputData.toDS
+    val inputDataDS = inputData.toDS()
       .withColumnRenamed("value", "val1")
       .withColumn("key2", $"val1")
       // the type of columns with string literal will be non-nullable
@@ -877,8 +875,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
         .applyInPandasWithState(
           pythonFunc(
             inputDataDS("val1"), inputDataDS("key2"), inputDataDS("val2"), inputDataDS("key1"),
-            inputDataDS("val3")
-          ).expr.asInstanceOf[PythonUDF],
+            inputDataDS("val3")),
           outputStructType,
           stateStructType,
           "Update",
@@ -949,7 +946,7 @@ class FlatMapGroupsInPandasWithStateSuite extends StateStoreMetricsTest {
       inputDataDS
         .groupBy("value")
         .applyInPandasWithState(
-          pythonFunc(inputDataDS("value")).expr.asInstanceOf[PythonUDF],
+          pythonFunc(inputDataDS("value")),
           outputStructType,
           stateStructType,
           "Update",
