@@ -27,16 +27,16 @@ import org.apache.spark.sql.catalyst.TableIdentifier
 import org.apache.spark.sql.catalyst.analysis.TableAlreadyExistsException
 import org.apache.spark.sql.catalyst.catalog.{CatalogStorageFormat, CatalogTable, CatalogTableType}
 import org.apache.spark.sql.catalyst.streaming.StreamingRelationV2
-import org.apache.spark.sql.connector.{FakeV2Provider, InMemoryTableSessionCatalog}
-import org.apache.spark.sql.connector.catalog.{Identifier, InMemoryTableCatalog, SupportsRead, Table, TableCapability, V2TableWithV1Fallback}
-import org.apache.spark.sql.connector.expressions.Transform
+import org.apache.spark.sql.connector.{FakeV2Provider, FakeV2ProviderWithCustomSchema, InMemoryTableSessionCatalog}
+import org.apache.spark.sql.connector.catalog.{Column, Identifier, InMemoryTableCatalog, MetadataColumn, SupportsMetadataColumns, SupportsRead, Table, TableCapability, V2TableWithV1Fallback}
+import org.apache.spark.sql.connector.expressions.{ClusterByTransform, FieldReference, Transform}
 import org.apache.spark.sql.connector.read.ScanBuilder
 import org.apache.spark.sql.execution.streaming.{MemoryStream, MemoryStreamScanBuilder, StreamingQueryWrapper}
 import org.apache.spark.sql.functions.lit
 import org.apache.spark.sql.internal.SQLConf
 import org.apache.spark.sql.streaming.StreamTest
 import org.apache.spark.sql.streaming.sources.FakeScanBuilder
-import org.apache.spark.sql.types.StructType
+import org.apache.spark.sql.types.{DataType, IntegerType, StructType}
 import org.apache.spark.sql.util.CaseInsensitiveStringMap
 import org.apache.spark.tags.SlowSQLTest
 import org.apache.spark.util.Utils
@@ -113,9 +113,16 @@ class DataStreamTableAPISuite extends StreamTest with BeforeAndAfter {
 
     spark.sql(s"CREATE TABLE $tableIdentifier (id bigint, data string) USING foo")
 
-    intercept[AnalysisException] {
-      spark.readStream.table(tableIdentifier)
-    }.message.contains("does not support either micro-batch or continuous scan")
+    checkError(
+      exception = intercept[AnalysisException] {
+        spark.readStream.table(tableIdentifier)
+      },
+      condition = "UNSUPPORTED_FEATURE.TABLE_OPERATION",
+      parameters = Map(
+        "tableName" -> "`testcat`.`table_name`",
+        "operation" -> "either micro-batch or continuous scan"
+      )
+    )
   }
 
   test("read: read table with custom catalog") {
@@ -197,7 +204,7 @@ class DataStreamTableAPISuite extends StreamTest with BeforeAndAfter {
   }
 
   test("write: write to table with default session catalog") {
-    val v2Source = classOf[FakeV2Provider].getName
+    val v2Source = classOf[FakeV2ProviderWithCustomSchema].getName
     spark.conf.set(SQLConf.V2_SESSION_CATALOG_IMPLEMENTATION.key,
       classOf[InMemoryTableSessionCatalog].getName)
 
@@ -327,6 +334,31 @@ class DataStreamTableAPISuite extends StreamTest with BeforeAndAfter {
     }
   }
 
+  test("write: write to new table with clusterBy") {
+    val tableIdentifier = "testcat.cluster_test"
+
+    withTable(tableIdentifier) {
+      runStreamAppendWithClusterBy(tableIdentifier)
+
+      val table = spark.sessionState.catalogManager.catalog("testcat").asTableCatalog
+        .loadTable(Identifier.of(Array(), "cluster_test"))
+      assert(table.partitioning === Seq(ClusterByTransform(Seq(FieldReference("id")))))
+    }
+  }
+
+  test("write: write to existing table with matching clustering column using clusterBy") {
+    val tableIdentifier = "testcat.cluster_test"
+
+    withTable(tableIdentifier) {
+      sql(s"CREATE TABLE $tableIdentifier (id BIGINT, data STRING) CLUSTER BY (id)")
+      runStreamAppendWithClusterBy(tableIdentifier)
+
+      val table = spark.sessionState.catalogManager.catalog("testcat").asTableCatalog
+        .loadTable(Identifier.of(Array(), "cluster_test"))
+      assert(table.partitioning === Seq(ClusterByTransform(Seq(FieldReference("id")))))
+    }
+  }
+
   test("explain with table on DSv1 data source") {
     val tblSourceName = "tbl_src"
     val tblTargetName = "tbl_target"
@@ -426,7 +458,7 @@ class DataStreamTableAPISuite extends StreamTest with BeforeAndAfter {
           val explainWithExtended = sq.explainInternal(true)
           // `extended = true` displays 3 logical plans (Parsed/Analyzed/Optimized) and 1 physical
           // plan.
-          assert("StreamingDataSourceV2Relation".r
+          assert("StreamingDataSourceV2ScanRelation".r
             .findAllMatchIn(explainWithExtended).size === 3)
           // WriteToMicroBatchDataSource is used for both parsed and analyzed logical plan
           assert("WriteToMicroBatchDataSource".r
@@ -499,6 +531,20 @@ class DataStreamTableAPISuite extends StreamTest with BeforeAndAfter {
     }
   }
 
+  test("SPARK-44865: Test StreamingRelationV2 with metadata column") {
+    val tblName = "teststream.table_name"
+    withTable(tblName) {
+      spark.sql(s"CREATE TABLE $tblName (data int) USING foo")
+      val stream = MemoryStream[Int]
+      val testCatalog = spark.sessionState.catalogManager.catalog("teststream").asTableCatalog
+      val table = testCatalog.loadTable(Identifier.of(Array(), "table_name"))
+      table.asInstanceOf[InMemoryStreamTable].setStream(stream)
+      // It will not throw UNRESOLVED_COLUMN exception because
+      // we add metadata column to StreamingRelationV2
+      spark.readStream.table(tblName).select("value", "_seq")
+    }
+  }
+
   private def checkForStreamTable(dir: Option[File], tableName: String): Unit = {
     val memory = MemoryStream[Int]
     val dsw = memory.toDS().writeStream.format("parquet")
@@ -570,13 +616,34 @@ class DataStreamTableAPISuite extends StreamTest with BeforeAndAfter {
       expectedOutputs.map { case (id, data) => Row(id, data) }
     )
   }
+
+  private def runStreamAppendWithClusterBy(tableIdentifier: String): Unit = {
+    withTempDir { ckptDir =>
+      val inputData = MemoryStream[(Long, String)]
+      val inputDF = inputData.toDF().toDF("id", "data")
+
+      val query = inputDF
+        .writeStream
+        .clusterBy("id")
+        .option("checkpointLocation", ckptDir.getAbsolutePath)
+        .toTable(tableIdentifier)
+
+      inputData.addData(Seq((1L, "a"), (2L, "b"), (3L, "c")))
+
+      query.processAllAvailable()
+      query.stop()
+    }
+  }
 }
 
 object DataStreamTableAPISuite {
   val V1FallbackTestTableName = "fallbackV1Test"
 }
 
-class InMemoryStreamTable(override val name: String) extends Table with SupportsRead {
+class InMemoryStreamTable(override val name: String)
+  extends Table
+  with SupportsRead
+  with SupportsMetadataColumns {
   var stream: MemoryStream[Int] = _
 
   def setStream(inputData: MemoryStream[Int]): Unit = stream = inputData
@@ -590,6 +657,14 @@ class InMemoryStreamTable(override val name: String) extends Table with Supports
   override def newScanBuilder(options: CaseInsensitiveStringMap): ScanBuilder = {
     new MemoryStreamScanBuilder(stream)
   }
+
+  private object SeqColumn extends MetadataColumn {
+    override def name: String = "_seq"
+    override def dataType: DataType = IntegerType
+    override def comment: String = "Seq"
+  }
+
+  override val metadataColumns: Array[MetadataColumn] = Array(SeqColumn)
 }
 
 class NonStreamV2Table(override val name: String)
@@ -617,7 +692,7 @@ class InMemoryStreamTableCatalog extends InMemoryTableCatalog {
 
   override def createTable(
       ident: Identifier,
-      schema: StructType,
+      columns: Array[Column],
       partitions: Array[Transform],
       properties: util.Map[String, String]): Table = {
     if (tables.containsKey(ident)) {
