@@ -18,10 +18,10 @@
 package org.apache.spark.sql.streaming.ui
 
 import java.util.{Locale, UUID}
-import javax.servlet.http.HttpServletRequest
 
 import scala.xml.Node
 
+import jakarta.servlet.http.HttpServletRequest
 import org.mockito.Mockito.{mock, when, RETURNS_SMART_NULLS}
 import org.scalatest.BeforeAndAfter
 
