@@ -21,7 +21,7 @@ import java.io.File
 import java.util.{Locale, Random}
 import java.util.concurrent.ConcurrentLinkedQueue
 
-import scala.collection.JavaConverters._
+import scala.jdk.CollectionConverters._
 
 import org.apache.hadoop.conf.Configuration
 import org.apache.hadoop.fs.Path
@@ -165,7 +165,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
     }
     checkError(
       exception = e,
-      errorClass = "WRITE_STREAM_NOT_ALLOWED",
+      condition = "WRITE_STREAM_NOT_ALLOWED",
       parameters = Map.empty
     )
   }
@@ -252,7 +252,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
   }
 
   test("SPARK-32364: later option should override earlier options for save()") {
-    Seq(1).toDF.write
+    Seq(1).toDF().write
       .format("org.apache.spark.sql.test")
       .option("paTh", "1")
       .option("PATH", "2")
@@ -264,7 +264,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
 
     withClue("SPARK-32516: legacy path option behavior") {
       withSQLConf(SQLConf.LEGACY_PATH_OPTION_BEHAVIOR.key -> "true") {
-        Seq(1).toDF.write
+        Seq(1).toDF().write
           .format("org.apache.spark.sql.test")
           .option("paTh", "1")
           .option("PATH", "2")
@@ -277,7 +277,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
   }
 
   test("pass partitionBy as options") {
-    Seq(1).toDF.write
+    Seq(1).toDF().write
       .format("org.apache.spark.sql.test")
       .partitionBy("col1", "col2")
       .save()
@@ -286,6 +286,75 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
     assert(DataSourceUtils.decodePartitioningColumns(partColumns) === Seq("col1", "col2"))
   }
 
+  test("pass clusterBy as options") {
+    Seq(1).toDF().write
+      .format("org.apache.spark.sql.test")
+      .clusterBy("col1", "col2")
+      .save()
+
+    val clusteringColumns = LastOptions.parameters(DataSourceUtils.CLUSTERING_COLUMNS_KEY)
+    assert(DataSourceUtils.decodePartitioningColumns(clusteringColumns) === Seq("col1", "col2"))
+  }
+
+  test("Clustering columns should match when appending to existing data source tables") {
+    import testImplicits._
+    val df = Seq((1, 2, 3)).toDF("a", "b", "c")
+    withTable("clusteredTable") {
+      df.write.mode("overwrite").clusterBy("a", "b").saveAsTable("clusteredTable")
+      // Misses some clustering columns
+      checkError(
+        exception = intercept[AnalysisException] {
+          df.write.mode("append").clusterBy("a").saveAsTable("clusteredTable")
+        },
+        condition = "CLUSTERING_COLUMNS_MISMATCH",
+        parameters = Map(
+          "tableName" -> "spark_catalog.default.clusteredtable",
+          "specifiedClusteringString" -> """[["a"]]""",
+          "existingClusteringString" -> """[["a"],["b"]]""")
+      )
+      // Wrong order
+      checkError(
+        exception = intercept[AnalysisException] {
+          df.write.mode("append").clusterBy("b", "a").saveAsTable("clusteredTable")
+        },
+        condition = "CLUSTERING_COLUMNS_MISMATCH",
+        parameters = Map(
+          "tableName" -> "spark_catalog.default.clusteredtable",
+          "specifiedClusteringString" -> """[["b"],["a"]]""",
+          "existingClusteringString" -> """[["a"],["b"]]""")
+      )
+      // Clustering columns not specified
+      checkError(
+        exception = intercept[AnalysisException] {
+          df.write.mode("append").saveAsTable("clusteredTable")
+        },
+        condition = "CLUSTERING_COLUMNS_MISMATCH",
+        parameters = Map(
+          "tableName" -> "spark_catalog.default.clusteredtable",
+          "specifiedClusteringString" -> "", "existingClusteringString" -> """[["a"],["b"]]""")
+      )
+      assert(sql("select * from clusteredTable").collect().length == 1)
+      // Inserts new data successfully when clustering columns are correctly specified in
+      // clusterBy(...).
+      Seq((4, 5, 6)).toDF("a", "b", "c")
+        .write
+        .mode("append")
+        .clusterBy("a", "b")
+        .saveAsTable("clusteredTable")
+
+      Seq((7, 8, 9)).toDF("a", "b", "c")
+        .write
+        .mode("append")
+        .clusterBy("a", "b")
+        .saveAsTable("clusteredTable")
+
+      checkAnswer(
+        sql("select a, b, c from clusteredTable"),
+        Row(1, 2, 3) :: Row(4, 5, 6) :: Row(7, 8, 9) :: Nil
+      )
+    }
+  }
+
   test ("SPARK-29537: throw exception when user defined a wrong base path") {
     withTempPath { p =>
       val path = new Path(p.toURI).toString
@@ -386,7 +455,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
           exception = intercept[AnalysisException] {
             Seq((1L, 2.0)).toDF("i", "d").write.mode("append").saveAsTable("t")
           },
-          errorClass = "INCOMPATIBLE_DATA_FOR_TABLE.CANNOT_SAFELY_CAST",
+          condition = "INCOMPATIBLE_DATA_FOR_TABLE.CANNOT_SAFELY_CAST",
           parameters = Map(
             "tableName" -> "`spark_catalog`.`default`.`t`",
             "colName" -> "`i`",
@@ -414,7 +483,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
           exception = intercept[AnalysisException] {
             Seq(("a", "b")).toDF("i", "d").write.mode("append").saveAsTable("t")
           },
-          errorClass = "INCOMPATIBLE_DATA_FOR_TABLE.CANNOT_SAFELY_CAST",
+          condition = "INCOMPATIBLE_DATA_FOR_TABLE.CANNOT_SAFELY_CAST",
           parameters = Map(
             "tableName" -> "`spark_catalog`.`default`.`t`",
             "colName" -> "`i`",
@@ -426,7 +495,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
           exception = intercept[AnalysisException] {
             Seq((true, false)).toDF("i", "d").write.mode("append").saveAsTable("t")
           },
-          errorClass = "INCOMPATIBLE_DATA_FOR_TABLE.CANNOT_SAFELY_CAST",
+          condition = "INCOMPATIBLE_DATA_FOR_TABLE.CANNOT_SAFELY_CAST",
           parameters = Map(
             "tableName" -> "`spark_catalog`.`default`.`t`",
             "colName" -> "`i`",
@@ -473,6 +542,10 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
       .option("doubleOpt", 6.7)
       .load("/test")
 
+    assert(
+      !df.queryExecution.logical.resolved,
+      "DataFrameReader should create an unresolved plan")
+
     assert(LastOptions.parameters("intOpt") == "56")
     assert(LastOptions.parameters("boolOpt") == "false")
     assert(LastOptions.parameters("doubleOpt") == "6.7")
@@ -490,7 +563,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
     assert(LastOptions.parameters("doubleOpt") == "6.7")
   }
 
-  test("check jdbc() does not support partitioning, bucketBy or sortBy") {
+  test("check jdbc() does not support partitioning, bucketBy, clusterBy or sortBy") {
     val df = spark.read.text(Utils.createTempDir(namePrefix = "text").getCanonicalPath)
 
     var w = df.write.partitionBy("value")
@@ -505,6 +578,12 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
       assert(e.getMessage.toLowerCase(Locale.ROOT).contains(s.toLowerCase(Locale.ROOT)))
     }
 
+    w = df.write.clusterBy("value")
+    e = intercept[AnalysisException](w.jdbc(null, null, null))
+    Seq("jdbc", "clustering").foreach { s =>
+      assert(e.getMessage.toLowerCase(Locale.ROOT).contains(s.toLowerCase(Locale.ROOT)))
+    }
+
     w = df.write.sortBy("value")
     e = intercept[AnalysisException](w.jdbc(null, null, null))
     Seq("sortBy must be used together with bucketBy").foreach { s =>
@@ -531,10 +610,13 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
   }
 
   test("load API") {
-    spark.read.format("org.apache.spark.sql.test").load()
-    spark.read.format("org.apache.spark.sql.test").load(dir)
-    spark.read.format("org.apache.spark.sql.test").load(dir, dir, dir)
-    spark.read.format("org.apache.spark.sql.test").load(Seq(dir, dir): _*)
+    def assertFirstUnresolved(df: DataFrame): Unit = {
+      assert(!df.queryExecution.logical.resolved)
+    }
+    assertFirstUnresolved(spark.read.format("org.apache.spark.sql.test").load())
+    assertFirstUnresolved(spark.read.format("org.apache.spark.sql.test").load(dir))
+    assertFirstUnresolved(spark.read.format("org.apache.spark.sql.test").load(dir, dir, dir))
+    assertFirstUnresolved(spark.read.format("org.apache.spark.sql.test").load(Seq(dir, dir): _*))
     Option(dir).map(spark.read.format("org.apache.spark.sql.test").load)
   }
 
@@ -625,10 +707,19 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
     spark.createDataset(data).write.mode(SaveMode.Overwrite).text(dir)
 
     // Reader, without user specified schema
-    testRead(spark.read.textFile().toDF(), Seq.empty, textSchema)
-    testRead(spark.read.textFile(dir).toDF(), data, textSchema)
-    testRead(spark.read.textFile(dir, dir).toDF(), data ++ data, textSchema)
-    testRead(spark.read.textFile(Seq(dir, dir): _*).toDF(), data ++ data, textSchema)
+    testRead(spark.read.text().toDF(), Seq.empty, textSchema)
+    testRead(spark.read.text(dir).toDF(), data, textSchema)
+    testRead(spark.read.text(dir, dir).toDF(), data ++ data, textSchema)
+    testRead(spark.read.text(Seq(dir, dir): _*).toDF(), data ++ data, textSchema)
+    testRead(spark.read.textFile().toDF(), Seq.empty, textSchema, checkLogicalPlan = false)
+    testRead(spark.read.textFile(dir).toDF(), data, textSchema, checkLogicalPlan = false)
+    testRead(
+      spark.read.textFile(dir, dir).toDF(), data ++ data, textSchema, checkLogicalPlan = false)
+    testRead(
+      spark.read.textFile(Seq(dir, dir): _*).toDF(),
+      data ++ data,
+      textSchema,
+      checkLogicalPlan = false)
     // Test explicit calls to single arg method - SPARK-16009
     testRead(Option(dir).map(spark.read.text).get, data, textSchema)
 
@@ -653,7 +744,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
       exception = intercept[AnalysisException] {
         testRead(spark.read.csv(), Seq.empty, schema)
       },
-      errorClass = "UNABLE_TO_INFER_SCHEMA",
+      condition = "UNABLE_TO_INFER_SCHEMA",
       parameters = Map("format" -> "CSV")
     )
 
@@ -818,9 +909,15 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
   private def testRead(
       df: => DataFrame,
       expectedResult: Seq[String],
-      expectedSchema: StructType): Unit = {
+      expectedSchema: StructType,
+      checkLogicalPlan: Boolean = true): Unit = {
     checkAnswer(df, spark.createDataset(expectedResult).toDF())
     assert(df.schema === expectedSchema)
+    if (checkLogicalPlan) {
+      // While the `textfile` API also leverages our UnresolvedDataSource plan, it later overrides
+      // the plan with encoders. Therefore this check fails
+      assert(!df.queryExecution.logical.resolved, "Should've created an unresolved plan")
+    }
   }
 
   test("saveAsTable with mode Append should not fail if the table not exists " +
@@ -991,13 +1088,13 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
           exception = intercept[AnalysisException] {
             Seq((1, 1)).toDF("col", c0).write.bucketBy(2, c0, c1).saveAsTable("t")
           },
-          errorClass = "COLUMN_ALREADY_EXISTS",
+          condition = "COLUMN_ALREADY_EXISTS",
           parameters = Map("columnName" -> s"`${c1.toLowerCase(Locale.ROOT)}`"))
         checkError(
           exception = intercept[AnalysisException] {
             Seq((1, 1)).toDF("col", c0).write.bucketBy(2, "col").sortBy(c0, c1).saveAsTable("t")
           },
-          errorClass = "COLUMN_ALREADY_EXISTS",
+          condition = "COLUMN_ALREADY_EXISTS",
           parameters = Map("columnName" -> s"`${c1.toLowerCase(Locale.ROOT)}`"))
       }
     }
@@ -1011,7 +1108,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
           Seq((1, 1)).toDF(colName0, colName1).write.format(format).mode("overwrite")
             .save(tempDir.getAbsolutePath)
         },
-        errorClass = "COLUMN_ALREADY_EXISTS",
+        condition = "COLUMN_ALREADY_EXISTS",
         parameters = Map("columnName" -> s"`${colName1.toLowerCase(Locale.ROOT)}`"))
     }
 
@@ -1024,7 +1121,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
           spark.read.format(format).schema(s"$colName0 INT, $colName1 INT")
             .load(testDir.getAbsolutePath)
         },
-        errorClass = "COLUMN_ALREADY_EXISTS",
+        condition = "COLUMN_ALREADY_EXISTS",
         parameters = Map("columnName" -> s"`${colName1.toLowerCase(Locale.ROOT)}`"))
     }
 
@@ -1037,7 +1134,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
         exception = intercept[AnalysisException] {
           spark.read.format(format).load(testDir.getAbsolutePath)
         },
-        errorClass = "COLUMN_ALREADY_EXISTS",
+        condition = "COLUMN_ALREADY_EXISTS",
         parameters = Map("columnName" -> s"`${colName1.toLowerCase(Locale.ROOT)}`"))
     }
 
@@ -1045,7 +1142,6 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
       withSQLConf(SQLConf.CASE_SENSITIVE.key -> caseSensitive.toString) {
         withTempDir { src =>
           // Check CSV format
-          checkWriteDataColumnDuplication("csv", c0, c1, src)
           checkReadUserSpecifiedDataColumnDuplication(
             Seq((1, 1)).toDF("c0", "c1"), "csv", c0, c1, src)
           // If `inferSchema` is true, a CSV format is duplicate-safe (See SPARK-16896)
@@ -1068,7 +1164,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
             exception = intercept[AnalysisException] {
               spark.read.format("json").option("inferSchema", true).load(testDir.getAbsolutePath)
             },
-            errorClass = "COLUMN_ALREADY_EXISTS",
+            condition = "COLUMN_ALREADY_EXISTS",
             parameters = Map("columnName" -> s"`${c1.toLowerCase(Locale.ROOT)}`"))
           checkReadPartitionColumnDuplication("json", c0, c1, src)
 
@@ -1212,7 +1308,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
     withSQLConf(SQLConf.LEGACY_PATH_OPTION_BEHAVIOR.key -> "true") {
       withTempDir { dir =>
         val path = dir.getCanonicalPath
-        Seq(1).toDF.write.mode("overwrite").parquet(path)
+        Seq(1).toDF().write.mode("overwrite").parquet(path)
 
         // When there is one path parameter to load(), "path" option is overwritten.
         checkAnswer(spark.read.format("parquet").option("path", path).load(path), Row(1))
@@ -1239,7 +1335,7 @@ class DataFrameReaderWriterSuite extends QueryTest with SharedSparkSession with
         "Either remove the path option, or call save() without the parameter"))
     }
 
-    val df = Seq(1).toDF
+    val df = Seq(1).toDF()
     val path = "tmp"
     verifyLoadFails(df.write.option("path", path).parquet(path))
     verifyLoadFails(df.write.option("path", path).parquet(""))
