@@ -17,12 +17,13 @@
 
 package org.apache.spark.status.api.v1.sql
 
-import java.net.URL
+import java.net.{URI, URL}
 import java.text.SimpleDateFormat
-import javax.servlet.http.HttpServletResponse
 
+import jakarta.servlet.http.HttpServletResponse
 import org.json4s.DefaultFormats
 import org.json4s.jackson.JsonMethods
+import org.scalatest.time.SpanSugar._
 
 import org.apache.spark.SparkConf
 import org.apache.spark.deploy.history.HistoryServerSuite.getContentAndCode
@@ -47,7 +48,7 @@ class SqlResourceWithActualMetricsSuite
   // Exclude nodes which may not have the metrics
   val excludedNodes = List("WholeStageCodegen", "Project", "SerializeFromObject")
 
-  implicit val formats = new DefaultFormats {
+  implicit val formats: DefaultFormats = new DefaultFormats {
     override def dateFormatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss")
   }
 
@@ -69,8 +70,8 @@ class SqlResourceWithActualMetricsSuite
   }
 
   private def callSqlRestEndpointAndVerifyResult(): Long = {
-    val url = new URL(spark.sparkContext.ui.get.webUrl
-      + s"/api/v1/applications/${spark.sparkContext.applicationId}/sql")
+    val url = new URI(spark.sparkContext.ui.get.webUrl
+      + s"/api/v1/applications/${spark.sparkContext.applicationId}/sql").toURL
     val jsonResult = verifyAndGetSqlRestResult(url)
     val executionDatas = JsonMethods.parse(jsonResult).extract[Seq[ExecutionData]]
     assert(executionDatas.size > 0,
@@ -81,8 +82,8 @@ class SqlResourceWithActualMetricsSuite
   }
 
   private def callSqlRestEndpointByExecutionIdAndVerifyResult(executionId: Long): Unit = {
-    val url = new URL(spark.sparkContext.ui.get.webUrl
-      + s"/api/v1/applications/${spark.sparkContext.applicationId}/sql/${executionId}")
+    val url = new URI(spark.sparkContext.ui.get.webUrl
+      + s"/api/v1/applications/${spark.sparkContext.applicationId}/sql/${executionId}").toURL
     val jsonResult = verifyAndGetSqlRestResult(url)
     val executionData = JsonMethods.parse(jsonResult).extract[ExecutionData]
     verifySqlRestContent(executionData)
@@ -128,7 +129,7 @@ class SqlResourceWithActualMetricsSuite
       .filter(_.getAs[Int]("age") <= 30)
       .sort()
 
-    ds.toDF
+    ds.toDF()
   }
 
   test("SPARK-44334: Status of a failed DDL/DML with no jobs should be FAILED") {
@@ -137,21 +138,23 @@ class SqlResourceWithActualMetricsSuite
       sql(sqlStr)
       intercept[TableAlreadyExistsException](sql(sqlStr))
 
-      val url = new URL(spark.sparkContext.ui.get.webUrl +
-        s"/api/v1/applications/${spark.sparkContext.applicationId}/sql")
-      val result = verifyAndGetSqlRestResult(url)
-      val executionDataList = JsonMethods.parse(result)
-        .extract[Seq[ExecutionData]]
-        .filter(_.planDescription.contains("SPARK_44334"))
-      assert(executionDataList.size == 2)
-      assert(executionDataList.head.status == "COMPLETED")
-      assert(executionDataList.last.status == "FAILED")
+      val url = new URI(spark.sparkContext.ui.get.webUrl +
+        s"/api/v1/applications/${spark.sparkContext.applicationId}/sql").toURL
+      eventually(timeout(20.seconds), interval(50.milliseconds)) {
+        val result = verifyAndGetSqlRestResult(url)
+        val executionDataList = JsonMethods.parse(result)
+          .extract[Seq[ExecutionData]]
+          .filter(_.planDescription.contains("SPARK_44334"))
+        assert(executionDataList.size == 2)
+        assert(executionDataList.head.status == "COMPLETED")
+        assert(executionDataList.last.status == "FAILED")
+      }
     }
   }
 
   test("SPARK-45291: Use unknown query execution id instead of no such app when id is invalid") {
-    val url = new URL(spark.sparkContext.ui.get.webUrl +
-      s"/api/v1/applications/${spark.sparkContext.applicationId}/sql/${Long.MaxValue}")
+    val url = new URI(spark.sparkContext.ui.get.webUrl +
+      s"/api/v1/applications/${spark.sparkContext.applicationId}/sql/${Long.MaxValue}").toURL
     val (code, resultOpt, error) = getContentAndCode(url)
     assert(code === HttpServletResponse.SC_NOT_FOUND)
     assert(resultOpt.isEmpty)
