@@ -41,8 +41,11 @@ import org.apache.thrift.protocol.TProtocol;
 import org.apache.thrift.server.ServerContext;
 import org.apache.thrift.server.TServerEventHandler;
 import org.apache.thrift.transport.TTransport;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
+
+import org.apache.spark.internal.SparkLogger;
+import org.apache.spark.internal.SparkLoggerFactory;
+import org.apache.spark.internal.LogKeys;
+import org.apache.spark.internal.MDC;
 
 /**
  * ThriftCLIService.
@@ -50,7 +53,7 @@ import org.slf4j.LoggerFactory;
  */
 public abstract class ThriftCLIService extends AbstractService implements TCLIService.Iface, Runnable {
 
-  public static final Logger LOG = LoggerFactory.getLogger(ThriftCLIService.class.getName());
+  public static final SparkLogger LOG = SparkLoggerFactory.getLogger(ThriftCLIService.class);
 
   protected CLIService cliService;
   private static final TStatus OK_STATUS = new TStatus(TStatusCode.SUCCESS_STATUS);
@@ -83,6 +86,16 @@ public abstract class ThriftCLIService extends AbstractService implements TCLISe
     public SessionHandle getSessionHandle() {
       return sessionHandle;
     }
+
+    @Override
+    public <T> T unwrap(Class<T> aClass) {
+      return null;
+    }
+
+    @Override
+    public boolean isWrapperFor(Class<?> aClass) {
+      return false;
+    }
   }
 
   public ThriftCLIService(CLIService service, String serviceName) {
@@ -106,7 +119,7 @@ public abstract class ThriftCLIService extends AbstractService implements TCLISe
           try {
             cliService.closeSession(sessionHandle);
           } catch (HiveSQLException e) {
-            LOG.warn("Failed to close session: " + e, e);
+            LOG.warn("Failed to close session: ", e);
           }
         }
       }
@@ -236,7 +249,8 @@ public abstract class ThriftCLIService extends AbstractService implements TCLISe
 
   @Override
   public TOpenSessionResp OpenSession(TOpenSessionReq req) throws TException {
-    LOG.info("Client protocol version: " + req.getClient_protocol());
+    LOG.info("Client protocol version: {}",
+      MDC.of(LogKeys.PROTOCOL_VERSION$.MODULE$, req.getClient_protocol()));
     TOpenSessionResp resp = new TOpenSessionResp();
     try {
       SessionHandle sessionHandle = getSessionHandle(req, resp);
@@ -272,7 +286,7 @@ public abstract class ThriftCLIService extends AbstractService implements TCLISe
         sb.append(e.getKey()).append(" = ").append(e.getValue());
       }
       if (sb != null) {
-        LOG.info("{}", sb);
+        LOG.info("{}", MDC.of(LogKeys.SET_CLIENT_INFO_REQUEST$.MODULE$, sb));
       }
     }
     return new TSetClientInfoResp(OK_STATUS);
@@ -696,6 +710,16 @@ public abstract class ThriftCLIService extends AbstractService implements TCLISe
     }
   }
 
+  @Override
+  public TUploadDataResp UploadData(TUploadDataReq req) throws TException {
+    throw new UnsupportedOperationException("Method UploadData has not been implemented.");
+  }
+
+  @Override
+  public TDownloadDataResp DownloadData(TDownloadDataReq req) throws TException {
+    throw new UnsupportedOperationException("Method DownloadData has not been implemented.");
+  }
+
   @Override
   public abstract void run();
 
