@@ -21,8 +21,9 @@ import org.apache.hive.service.cli.{HiveSQLException, OperationState}
 import org.apache.hive.service.cli.operation.Operation
 
 import org.apache.spark.SparkContext
-import org.apache.spark.internal.Logging
-import org.apache.spark.sql.{SparkSession, SQLContext}
+import org.apache.spark.internal.{Logging, MDC}
+import org.apache.spark.internal.LogKeys.{HIVE_OPERATION_TYPE, STATEMENT_ID}
+import org.apache.spark.sql.SparkSession
 import org.apache.spark.sql.catalyst.CurrentUserContext.CURRENT_USER
 import org.apache.spark.sql.catalyst.catalog.CatalogTableType
 import org.apache.spark.sql.catalyst.catalog.CatalogTableType.{EXTERNAL, MANAGED, VIEW}
@@ -34,7 +35,7 @@ import org.apache.spark.util.Utils
  */
 private[hive] trait SparkOperation extends Operation with Logging {
 
-  protected def sqlContext: SQLContext
+  protected def session: SparkSession
 
   protected var statementId = getHandle().getHandleIdentifier().getPublicId().toString()
 
@@ -49,7 +50,7 @@ private[hive] trait SparkOperation extends Operation with Logging {
   abstract override def close(): Unit = {
     super.close()
     cleanup()
-    logInfo(s"Close statement with $statementId")
+    logInfo(log"Close statement with ${MDC(STATEMENT_ID, statementId)}")
     HiveThriftServer2.eventManager.onOperationClosed(statementId)
   }
 
@@ -61,17 +62,17 @@ private[hive] trait SparkOperation extends Operation with Logging {
   // - set appropriate SparkSession
   // - set scheduler pool for the operation
   def withLocalProperties[T](f: => T): T = {
-    val originalProps = Utils.cloneProperties(sqlContext.sparkContext.getLocalProperties)
+    val originalProps = Utils.cloneProperties(session.sparkContext.getLocalProperties)
     val originalSession = SparkSession.getActiveSession
 
     try {
       // Set active SparkSession
-      SparkSession.setActiveSession(sqlContext.sparkSession)
+      SparkSession.setActiveSession(session)
 
       // Set scheduler pool
-      sqlContext.sparkSession.conf.getOption(SQLConf.THRIFTSERVER_POOL.key) match {
+      session.conf.getOption(SQLConf.THRIFTSERVER_POOL.key) match {
         case Some(pool) =>
-          sqlContext.sparkContext.setLocalProperty(SparkContext.SPARK_SCHEDULER_POOL, pool)
+          session.sparkContext.setLocalProperty(SparkContext.SPARK_SCHEDULER_POOL, pool)
         case None =>
       }
       CURRENT_USER.set(getParentSession.getUserName)
@@ -80,7 +81,7 @@ private[hive] trait SparkOperation extends Operation with Logging {
     } finally {
       CURRENT_USER.remove()
       // reset local properties, will also reset SPARK_SCHEDULER_POOL
-      sqlContext.sparkContext.setLocalProperties(originalProps)
+      session.sparkContext.setLocalProperties(originalProps)
 
       originalSession match {
         case Some(session) => SparkSession.setActiveSession(session)
@@ -98,7 +99,8 @@ private[hive] trait SparkOperation extends Operation with Logging {
 
   protected def onError(): PartialFunction[Throwable, Unit] = {
     case e: Throwable =>
-      logError(s"Error operating $getType with $statementId", e)
+      logError(log"Error operating ${MDC(HIVE_OPERATION_TYPE, getType)} with " +
+        log"${MDC(STATEMENT_ID, statementId)}", e)
       super.setState(OperationState.ERROR)
       HiveThriftServer2.eventManager.onStatementError(
         statementId, e.getMessage, Utils.exceptionString(e))
